@@ -1,4 +1,4 @@
-/* medit.c -- simple multilingual editor.
+/* medit.c -- simple multilingual editor.		-*- coding: euc-jp; -*-
    Copyright (C) 2003, 2004
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
@@ -21,13 +21,13 @@
    02111-1307, USA.  */
 
 /***en
-    @page medit edit multilingual text
+    @enpage m17n-edit edit multilingual text
 
-    @section medit-synopsis SYNOPSIS
+    @section m17n-edit-synopsis SYNOPSIS
 
-    medit [ XT-OPTION ...] [ OPTION ... ] FILE
+    m17n-edit [ XT-OPTION ...] [ OPTION ... ] FILE
 
-    @section medit-description DESCRIPTION
+    @section m17n-edit-description DESCRIPTION
 
     Display FILE on a window and allow users to edit it.
 
@@ -48,9 +48,41 @@
     </ul>
 
     This program is to demonstrate how to use the m17n GUI API.
-    Although medit directly uses the GUI API, the API is mainly for
-    toolkit libraries or to implement XOM (X Outout Method), not for
-    direct use from application programs.
+    Although m17n-edit directly uses the GUI API, the API is mainly
+    for toolkit libraries or to implement XOM (X Outout Method), not
+    for direct use from application programs.
+*/
+/***ja
+    @japage m17n-edit ¿ƥȤԽ
+
+    @section m17n-edit-synopsis SYNOPSIS
+
+    m17n-edit [ XT-OPTION ...] [ OPTION ... ] FILE
+
+    @section m17n-edit-description DESCRIPTION
+
+    FILE 򥦥ɥɽ桼ԽǤ褦ˤ롣
+
+    XT-OPTIONs  Xt ɸΰǤ롣 (e.g. -fn, -fg). 
+
+    ʲΥץѤǤ롣 
+
+    <ul>
+
+    <li> --version
+
+    Сֹɽ롣 
+
+    <li> -h, --help
+
+    Υåɽ롣 
+
+    </ul>
+
+    Υץ m17n GUI API λȤ򼨤ΤǤ롣m17n-edit
+    ľ GUI API ȤäƤ뤬 API ϼ˥ġ륭åȥ饤֥
+    XOM (X Outout Method) μѤǤꡢץꥱץ
+    फľܤѤտޤƤʤ
 */
 
 #ifndef FOR_DOXYGEN
@@ -84,7 +116,7 @@
 #include <m17n-misc.h>
 #include <m17n-X.h>
 
-#define VERSION "1.0.1"
+#define VERSION "1.2.0"
 
 /* Global variables.  */
 
@@ -144,13 +176,22 @@ MFace *face_default;
 MFace *face_xxx_large;
 MFace *face_box;
 MFace *face_courier, *face_helvetica, *face_times;
-MFace *face_dv_ttyogesh, *face_freesans, *face_freemono;
+MFace *face_dv_ttyogesh, *face_freesans, *face_freeserif, *face_freemono;
 MFace *face_default_fontset, *face_no_ctl_fontset;
 MFace *face_input_status;
 
+MSymbol Mcoding_compound_text;
+
 int logical_move = 1;		/* If 0, move cursor visually.  */
 
-MInputMethod **input_method_table;
+typedef struct {
+  int available;
+  MSymbol language, name;
+  MInputMethod *im;
+} InputMethodInfo;
+
+InputMethodInfo *input_method_table;
+
 int num_input_methods;
 int current_input_method = -1;	/* i.e. none */
 int auto_input_method = 0;
@@ -177,6 +218,7 @@ struct FaceRec
     {"times", &face_times},
     {"dv-ttyogesh", &face_dv_ttyogesh},
     {"freesans", &face_freesans},
+    {"freeserif", &face_freeserif},
     {"freemono", &face_freemono},
 
     {"Menu Style", NULL},
@@ -292,8 +334,8 @@ update_top (int pos)
   top.from = info.line_from;
   top.to = info.line_to;
   top.y0 = 0;
-  top.y1 = info.this.height;
-  top.ascent = - info.this.y;
+  top.y1 = info.metrics.height;
+  top.ascent = - info.metrics.y;
 }
 
 
@@ -325,7 +367,7 @@ redraw (int y0, int y1, int clear, int scroll_bar)
   int sel_y0 = SELECTEDP () ? sel_start.y0 : 0;
   struct LineInfo *line;
   
-  if (clear)
+  if (clear || control.anti_alias)
     CLEAR_AREA (0, y0, win_width, y1 - y0);
 
   /* Find a line closest to y0.  It is a cursor line if the cursor is
@@ -340,21 +382,21 @@ redraw (int y0, int y1, int clear, int scroll_bar)
 
   from = line->from;
   y = line->y0;
-  info.this.height = line->y1 - y;
-  info.this.y = - line->ascent;
+  info.metrics.height = line->y1 - y;
+  info.metrics.y = - line->ascent;
   info.line_to = line->to;
-  while (from < nchars && y + info.this.height <= y0)
+  while (from < nchars && y + info.metrics.height <= y0)
     {
-      y += info.this.height;
+      y += info.metrics.height;
       from = info.line_to;
       GLYPH_INFO (from, from, info);
     }
-  y0 = y - info.this.y;
+  y0 = y - info.metrics.y;
   to = from;
   while (to < nchars && y < y1)
     {
       GLYPH_INFO (to, to, info);
-      y += info.this.height;
+      y += info.metrics.height;
       to = info.line_to;
     }
   if (to == nchars)
@@ -366,10 +408,10 @@ redraw (int y0, int y1, int clear, int scroll_bar)
       while (to < nchars)
 	{
 	  GLYPH_INFO (to, to, info);
-	  if (y + info.this.height >= win_height)
+	  if (y + info.metrics.height >= win_height)
 	    break;
 	  to = info.line_to;
-	  y += info.this.height;
+	  y += info.metrics.height;
 	}
       update_scroll_bar (top.from, to);
     }
@@ -383,16 +425,19 @@ set_input_method_spot ()
   int x = cursor.x + (control.orientation_reversed ? win_width : 0);
   int pos = cursor.from > 0 ? cursor.from - 1 : 0;
   MFace *faces[256];
-  int n = mtext_get_prop_values (mt, pos, Mface, (void **) faces, 256);
+  int n = 0;
   int size = 0, ratio = 0, i;
 
-  for (i = n - 1; i >= 0; i--)
-    {
-      if (! size)
-	size = (int) mface_get_prop (faces[i], Msize);
-      if (! ratio)
-	ratio = (int) mface_get_prop (faces[i], Mratio);
-    }
+  if (nchars > 0)
+    n = mtext_get_prop_values (mt, pos, Mface, (void **) faces, 256);
+  if (n > 0)
+    for (i = n - 1; i >= 0; i--)
+      {
+	if (! size)
+	  size = (int) mface_get_prop (faces[i], Msize);
+	if (! ratio)
+	  ratio = (int) mface_get_prop (faces[i], Mratio);
+      }
   if (! size)
     size = default_font_size;
   if (ratio)
@@ -416,12 +461,12 @@ redraw_cursor (int clear)
       MDrawMetric rect;
       int y0 = cur.y0, y1 = cur.y1;
 
-      if (beg != cur.from)
+      if (beg < cur.from)
 	{
 	  TEXT_EXTENTS (beg, cur.from, rect);
 	  y0 -= rect.height;
 	}
-      if (end != cur.to)
+      if (end > cur.to)
 	{
 	  TEXT_EXTENTS (cur.to, end, rect);
 	  y1 += rect.height; 
@@ -435,8 +480,8 @@ redraw_cursor (int clear)
 	  int x = cursor.x;
 
 	  if (control.orientation_reversed)
-	    x += win_width - cursor.this.width;
-	  CLEAR_AREA (x, cur.y0, cursor.this.width, cursor.this.height);
+	    x += win_width - cursor.logical_width;
+	  CLEAR_AREA (x, cur.y0, cursor.logical_width, cursor.metrics.height);
 	}
       DRAW_TEXT (cursor.x, cur.y0 + cur.ascent, cursor.from, cursor.to);
     }
@@ -456,7 +501,7 @@ update_cursor (int pos, int full)
     {
       /* CUR is inaccurate.  We can trust only TOP.  */
       GLYPH_INFO (top.from, pos, cursor);
-      cur.y0 = top.ascent + cursor.y + cursor.this.y;
+      cur.y0 = top.ascent + cursor.y + cursor.metrics.y;
     }
   else if (pos < cur.from)
     {
@@ -464,7 +509,7 @@ update_cursor (int pos, int full)
 
       TEXT_EXTENTS (from, cur.from, rect);
       GLYPH_INFO (from, pos, cursor);
-      cur.y0 -= (rect.height + rect.y) - (cursor.y + cursor.this.y);
+      cur.y0 -= (rect.height + rect.y) - (cursor.y + cursor.metrics.y);
     }
   else if (pos < cur.to)
     {
@@ -473,13 +518,13 @@ update_cursor (int pos, int full)
   else
     {
       GLYPH_INFO (cur.from, pos, cursor);
-      cur.y0 += cur.ascent + cursor.y + cursor.this.y;
+      cur.y0 += cur.ascent + cursor.y + cursor.metrics.y;
     }
 
   cur.from = cursor.line_from;
   cur.to = cursor.line_to;
-  cur.y1 = cur.y0 + cursor.this.height;
-  cur.ascent = - cursor.this.y;
+  cur.y1 = cur.y0 + cursor.metrics.height;
+  cur.ascent = - cursor.metrics.y;
 }
 
 
@@ -505,34 +550,28 @@ update_selection ()
       sel_start.ascent = - rect.y;
       GLYPH_INFO (pos, from, info);
       if (pos < info.line_from)
-	sel_start.y0 += - rect.y + info.y + info.this.y;
+	sel_start.y0 += - rect.y + info.y + info.metrics.y;
     }
   else
     {
       GLYPH_INFO (top.from, from, info);
-      sel_start.y0 = top.ascent + info.y + info.this.y;
+      sel_start.y0 = top.ascent + info.y + info.metrics.y;
     }
-  sel_start.ascent = -info.this.y;
-  sel_start.y1 = sel_start.y0 + info.this.height;
+  sel_start.ascent = -info.metrics.y;
+  sel_start.y1 = sel_start.y0 + info.metrics.height;
   sel_start.from = info.line_from;
   sel_start.to = info.line_to;
 
   if (to <= sel_start.to)
     {
       sel_end = sel_start;
-      if (to >= sel_end.to)
-	{
-	  GLYPH_INFO (sel_start.from, to, info);
-	  sel_end.y1 = sel_end.y0 + info.y + info.this.height;
-	  sel_end.to = info.line_to;
-	}
     }
   else
     {
       GLYPH_INFO (sel_start.from, to, info);
-      sel_end.y0 = sel_start.y0 + sel_start.ascent + info.y + info.this.y;
-      sel_end.y1 = sel_end.y0 + info.this.height;
-      sel_end.ascent = - info.this.y;
+      sel_end.y0 = sel_start.y0 + sel_start.ascent + info.y + info.metrics.y;
+      sel_end.y1 = sel_end.y0 + info.metrics.height;
+      sel_end.ascent = - info.metrics.y;
       sel_end.from = info.line_from;
       sel_end.to = info.line_to;
     }
@@ -626,7 +665,7 @@ select_input_method (idx)
     }
   if (idx >= 0)
     {
-      MInputMethod *im = input_method_table[idx];
+      InputMethodInfo *im = input_method_table + idx;
 
       if (im->language == Mnil)
 	{
@@ -636,7 +675,7 @@ select_input_method (idx)
 	  arg_xic.input_style = 0;
 	  arg_xic.client_win = arg_xic.focus_win = win;
 	  arg_xic.preedit_attrs =  arg_xic.status_attrs = NULL;
-	  current_input_context = minput_create_ic (im, &arg_xic);
+	  current_input_context = minput_create_ic (im->im, &arg_xic);
 	}
       else
 	{
@@ -645,7 +684,7 @@ select_input_method (idx)
 	  arg_ic.frame = frame;
 	  arg_ic.client = (MDrawWindow) XtWindow (ShellWidget);
 	  arg_ic.focus = (MDrawWindow) XtWindow (TextWidget);
-	  current_input_context = minput_create_ic (im, &arg_ic);
+	  current_input_context = minput_create_ic (im->im, &arg_ic);
 	}
 
       if (current_input_context)
@@ -676,6 +715,9 @@ static void MenuHelpProc (Widget w, XEvent *event, String *str, Cardinal *num);
 Boolean
 show_cursor (XtPointer client_data)
 {
+  MFaceHLineProp *hline;
+  MFaceBoxProp *box;
+
   if (cur.y0 < 0)
     {
       reseat (cur.from);
@@ -696,57 +738,60 @@ show_cursor (XtPointer client_data)
   if (current_input_context)
     set_input_method_spot ();
 
-  {
-    int pos = (SELECTEDP () ? mtext_property_start (selection)
-	       : cursor.from > 0 ? cursor.from - 1
-	       : cursor.from);
-    MFace *face = mface ();
-    MTextProperty *props[256];
-    int n = mtext_get_properties (mt, pos, Mface, props, 256);
-    int i;
-    char buf[256], *p = buf;
-    MSymbol sym;
+  if (nchars > 0)
+    {
+      int pos = (SELECTEDP () ? mtext_property_start (selection)
+		 : cursor.from > 0 ? cursor.from - 1
+		 : cursor.from);
+      MFace *face = mface ();
+      MTextProperty *props[256];
+      int n = mtext_get_properties (mt, pos, Mface, props, 256);
+      int i;
+      char buf[256], *p = buf;
+      MSymbol sym;
 
-    buf[0] = '\0';
-    if (cursor.font)
-      {
-	int size = (int) mfont_get_prop (cursor.font, Msize);
-	MSymbol family = mfont_get_prop (cursor.font, Mfamily);
-	MSymbol weight = mfont_get_prop (cursor.font, Mweight);
-	MSymbol style = mfont_get_prop (cursor.font, Mstyle);
-	MSymbol registry = mfont_get_prop (cursor.font, Mregistry);
+      buf[0] = '\0';
+      if (cursor.font)
+	{
+	  int size = (int) mfont_get_prop (cursor.font, Msize);
+	  MSymbol family = mfont_get_prop (cursor.font, Mfamily);
+	  MSymbol weight = mfont_get_prop (cursor.font, Mweight);
+	  MSymbol style = mfont_get_prop (cursor.font, Mstyle);
+	  MSymbol registry = mfont_get_prop (cursor.font, Mregistry);
 
-	sprintf (p, "%dpt", size / 10), p += strlen (p);
-	if (family)
-	  strcat (p, ","), strcat (p, msymbol_name (family)), p += strlen (p);
-	if (weight)
-	  strcat (p, ","), strcat (p, msymbol_name (weight)), p += strlen (p);
-	if (style)
-	  strcat (p, ","), strcat (p, msymbol_name (style)), p += strlen (p);
-	if (registry)
-	  strcat (p, ","), strcat (p, msymbol_name (registry)), p += strlen (p);
-	p += strlen (p);
-      }
+	  sprintf (p, "%dpt", size / 10), p += strlen (p);
+	  if (family)
+	    strcat (p, ","), strcat (p, msymbol_name (family)), p += strlen (p);
+	  if (weight)
+	    strcat (p, ","), strcat (p, msymbol_name (weight)), p += strlen (p);
+	  if (style)
+	    strcat (p, ","), strcat (p, msymbol_name (style)), p += strlen (p);
+	  if (registry)
+	    strcat (p, ","), strcat (p, msymbol_name (registry)), p += strlen (p);
+	  p += strlen (p);
+	}
 
-    mface_merge (face, face_default);
-    for (i = 0; i < n; i++)
-      if (props[i] != selection)
-	mface_merge (face, (MFace *) mtext_property_value (props[i]));
-    sym = (MSymbol) mface_get_prop (face, Mforeground);
-    if (sym != Mnil)
-      strcat (p, ","), strcat (p, msymbol_name (sym)), p += strlen (p);
-    if ((MSymbol) mface_get_prop (face, Mvideomode) == Mreverse)
-      strcat (p, ",rev"), p += strlen (p);
-    if (mface_get_prop (face, Mhline))
-      strcat (p, ",ul"), p += strlen (p);
-    if (mface_get_prop (face, Mbox))
-      strcat (p, ",box"), p += strlen (p);
-    m17n_object_unref (face);
+      mface_merge (face, face_default);
+      for (i = 0; i < n; i++)
+	if (props[i] != selection)
+	  mface_merge (face, (MFace *) mtext_property_value (props[i]));
+      sym = (MSymbol) mface_get_prop (face, Mforeground);
+      if (sym != Mnil)
+	strcat (p, ","), strcat (p, msymbol_name (sym)), p += strlen (p);
+      if ((MSymbol) mface_get_prop (face, Mvideomode) == Mreverse)
+	strcat (p, ",rev"), p += strlen (p);
+      hline = mface_get_prop (face, Mhline);
+      if (hline && hline->width > 0)
+	strcat (p, ",ul"), p += strlen (p);
+      box = mface_get_prop (face, Mbox);
+      if (box && box->width > 0)
+	strcat (p, ",box"), p += strlen (p);
+      m17n_object_unref (face);
 
-    XtSetArg (arg[0], XtNborderWidth, 1);
-    XtSetArg (arg[1], XtNlabel, buf);
-    XtSetValues (CurFaceWidget, arg, 2);
-  }
+      XtSetArg (arg[0], XtNborderWidth, 1);
+      XtSetArg (arg[1], XtNlabel, buf);
+      XtSetValues (CurFaceWidget, arg, 2);
+    }
 
   if (control.cursor_pos < nchars)
     {
@@ -781,10 +826,24 @@ show_cursor (XtPointer client_data)
 	      int i;
 
 	      for (i = 0; i < num_input_methods; i++)
-		if (input_method_table[i]->language == sym)
+		if (input_method_table[i].language == sym)
 		  break;
-	      if (i < num_input_methods)
-		select_input_method (i);
+	      if (i < num_input_methods
+		  && input_method_table[i].available >= 0)
+		{
+		  if (! input_method_table[i].im)
+		    {
+		      input_method_table[i].im = 
+			minput_open_im (input_method_table[i].language,
+					input_method_table[i].name, NULL);
+		      if (! input_method_table[i].im)
+			input_method_table[i].available = -1;
+		    }
+		  if (input_method_table[i].im)
+		    select_input_method (i);
+		  else
+		    select_input_method (-1);
+		}
 	      else
 		select_input_method (-1);
 	    }
@@ -878,7 +937,7 @@ delete_char (int n)
 
   /* Now delete a character.  */
   mtext_del (mt, from, to);
-  nchars--;
+  nchars -= to - from;
   if (from >= top.from && from < top.to)
     update_top (top.from);
   update_cursor (from, 1);
@@ -925,8 +984,9 @@ insert_chars (MText *newtext)
 }
 
 
-/* Convert the currently selected text to COMPOUND-TEXT.  It is called
-   when someone requests the current value of the selection.  */
+/* Convert the currently selected text to UTF8-STRING or
+   COMPOUND-TEXT.  It is called when someone requests the current
+   value of the selection.  */
 Boolean
 covert_selection (Widget w, Atom *selection_atom,
 		  Atom *target, Atom *return_type,
@@ -936,14 +996,52 @@ covert_selection (Widget w, Atom *selection_atom,
   MText *this_mt = mtext ();
   int from = mtext_property_start (selection);
   int to = mtext_property_end (selection);
+  MSymbol coding;
+  int len;
 
   mtext_copy (this_mt, 0, mt, from, to);
-  *length = mconv_encode_buffer (msymbol ("compound-text"),
-				 this_mt, buf, 4096);
-  *return_type = XA_COMPOUND_TEXT;
+  if (*target == XA_TEXT)
+    {
+#ifdef X_HAVE_UTF8_STRING
+      coding = Mcoding_utf_8;
+      *return_type = XA_UTF8_STRING;
+#else
+      coding = Mcoding_compound_text;
+      *return_type = XA_COMPOUND_TEXT;
+#endif
+    }
+  else if (*target == XA_UTF8_STRING)
+    {
+      coding = Mcoding_utf_8;
+      *return_type = XA_UTF8_STRING;
+    }
+  else if (*target == XA_STRING)
+    {
+      int i;
+
+      len = to - from;
+      for (i = 0; i < len; i++)
+	if (mtext_ref_char (this_mt, i) >= 0x100)
+	  /* Can't encode in XA_STRING */
+	  return False;
+      coding = Mcoding_iso_8859_1;
+      *return_type = XA_STRING;
+    }
+  else if (*target == XA_COMPOUND_TEXT)
+    {
+      coding = Mcoding_compound_text;
+      *return_type = XA_COMPOUND_TEXT;
+    }
+  else
+    return False;
+
+  len = mconv_encode_buffer (coding, this_mt, buf, 4096);
+  m17n_object_unref (this_mt);
+  if (len < 0)
+    return False;
+  *length = len;
   *value = (XtPointer) buf;
   *format = 8;
-  m17n_object_unref (this_mt);
   return True;
 }
 
@@ -972,12 +1070,20 @@ get_selection (Widget w, XtPointer cliend_data, Atom *selection,  Atom *type,
     coding = Mnil;
   else if (*type == XA_COMPOUND_TEXT)
     coding = msymbol ("compound-text");
+#ifdef X_HAVE_UTF8_STRING
   else if (*type == XA_UTF8_STRING)
     coding = msymbol ("utf-8");
+#endif
   else
     goto err;
 
   this_mt = mconv_decode_buffer (coding, (unsigned char *) value, *length);
+  if (! this_mt && *type != XA_UTF8_STRING)
+    {
+      XtGetSelectionValue (w, XA_PRIMARY, XA_UTF8_STRING, get_selection, NULL,
+			   CurrentTime);
+      goto err;
+    }
   if (this_mt)
     {
       hide_cursor ();
@@ -1021,6 +1127,19 @@ ExposeProc (Widget w, XEvent *event, String *str, Cardinal *num)
       update_top (0);
       update_cursor (0, 1);
       redraw (0, win_height, 0, 1);
+      if (current_input_method >= 0)
+      {
+	int idx = current_input_method;
+
+	current_input_method = -1;
+	input_method_table[idx].im = 
+	  minput_open_im (input_method_table[idx].language,
+			    input_method_table[idx].name, NULL);
+	if (input_method_table[idx].im)
+	  select_input_method (idx);
+	else
+	  input_method_table[idx].available = -1;
+      }
       show_cursor (NULL);
     }
   else
@@ -1065,6 +1184,20 @@ ButtonProc (Widget w, XEvent *event, String *str, Cardinal *num)
       redraw (sel_start.y0, sel_end.y1, 1, 0);
     }
   hide_cursor ();
+  if (current_input_context)
+    {
+      MText *produced = mtext ();
+
+      minput_reset_ic (current_input_context);
+      minput_lookup (current_input_context, Mnil, NULL, produced);
+      if (mtext_len (produced) > 0)
+	{
+	  insert_chars (produced);
+	  if (pos >= cursor.from)
+	    pos += mtext_len (produced);
+	}
+      m17n_object_unref (produced);
+    }
   update_cursor (pos, 0);
 }
 
@@ -1227,9 +1360,9 @@ ScrollProc (Widget w, XtPointer client_data, XtPointer position)
 	{
 	  pos = bol (from - 1, 0);
 	  GLYPH_INFO (pos, from - 1, info);
-	  if (height + info.this.height > win_height)
+	  if (height + info.metrics.height > win_height)
 	    break;
-	  height += info.this.height;
+	  height += info.metrics.height;
 	  from = info.line_from;
 	}
       if (cursor_pos >= top.to)
@@ -1239,9 +1372,9 @@ ScrollProc (Widget w, XtPointer client_data, XtPointer position)
 	  while (cursor_pos < nchars)
 	    {
 	      GLYPH_INFO (pos, pos, info);
-	      if (height + info.this.height > win_height)
+	      if (height + info.metrics.height > win_height)
 		break;
-	      height += info.this.height;
+	      height += info.metrics.height;
 	      cursor_pos = pos;
 	      pos = info.line_to;
 	    }
@@ -1255,10 +1388,10 @@ ScrollProc (Widget w, XtPointer client_data, XtPointer position)
       while (from < nchars)
 	{
 	  GLYPH_INFO (from, from, info);
-	  if (height + info.this.height > win_height
+	  if (height + info.metrics.height > win_height
 	      || info.line_to >= nchars)
 	    break;
-	  height += info.this.height;
+	  height += info.metrics.height;
 	  from = info.line_to;
 	}
       if (from == nchars)
@@ -1473,7 +1606,8 @@ KeyProc (Widget w, XEvent *event, String *str, Cardinal *num)
 	    {
 	      MText *temp = mtext ();
 
-	      mtext_cat_char (temp, buf[0] == '\r' ? '\n' : buf[0]);
+	      mtext_cat_char (temp, buf[0] == '\r' ? '\n'
+			      : ((unsigned char *) buf)[0]);
 	      if (current_input_context)
 		mtext_put_prop (temp, 0, 1, Mlanguage,
 				current_input_context->im->language);
@@ -1514,7 +1648,7 @@ SaveProc (Widget w, XtPointer client_data, XtPointer call_data)
       mtext_detach_property (selection);
     }
 
-  mconv_encode_stream (Mcoding_utf_8, mt, fp);
+  mconv_encode_stream (Mcoding_utf_8_full, mt, fp);
   fclose (fp);
   if (from >= 0)
     select_region (from, to);
@@ -1565,7 +1699,7 @@ read_file ()
 
   if (! fp)
     FATAL_ERROR ("Can't read \"%s\"!\n", filename);
-  mt = mconv_decode_stream (Mcoding_utf_8, fp);
+  mt = mconv_decode_stream (Mcoding_utf_8_full, fp);
   fclose (fp);
   if (! mt)
     FATAL_ERROR ("Can't decode \"%s\" by UTF-8!\n", filename);
@@ -1668,6 +1802,7 @@ CursorProc (Widget w, XtPointer client_data, XtPointer call_data)
       XtSetValues (CursorMenus[i], arg, 1);
     }
 
+  update_cursor (cursor.from, 0);
   redraw (0, win_height, 1, 0);
 }
 
@@ -1697,12 +1832,25 @@ InputMethodProc (Widget w, XtPointer client_data, XtPointer call_data)
       auto_input_method = 1;
       hide_cursor ();
     }
-  else
-    select_input_method (idx);
+  else if (input_method_table[idx].available >= 0)
+    {
+      if (! input_method_table[idx].im)
+	{
+	  input_method_table[idx].im = 
+	    minput_open_im (input_method_table[idx].language,
+			    input_method_table[idx].name, NULL);
+	  if (! input_method_table[idx].im)
+	    input_method_table[idx].available = -1;
+	}
+      if (input_method_table[idx].im)
+	select_input_method (idx);
+    }
   XtSetArg (arg[0], XtNleftBitmap, CheckPixmap);
   XtSetValues (InputMethodMenus[idx + 2], arg, 1);
 }
 
+MPlist *default_face_list;
+
 void
 FaceProc (Widget w, XtPointer client_data, XtPointer call_data)
 {
@@ -1710,8 +1858,37 @@ FaceProc (Widget w, XtPointer client_data, XtPointer call_data)
   int from, to;
   int old_y1;
 
+  hide_cursor ();
   if (! SELECTEDP ())
-    return;
+    {
+      MPlist *plist;
+
+      if (idx >= 0)
+	{
+	  MFace *face = mframe_get_prop (frame, Mface);
+
+	  for (plist = default_face_list; mplist_key (plist) != Mnil;
+	       plist = mplist_next (plist)) 
+	    mface_merge (face, mplist_value (plist));
+	  mplist_add (plist, Mt, *face_table[idx].face);
+	  mface_merge (face, *face_table[idx].face);
+	}
+      else if (mplist_key (mplist_next (default_face_list)) != Mnil)
+	{
+	  MFace *face = mframe_get_prop (frame, Mface);
+
+	  for (plist = default_face_list;
+	       mplist_key (mplist_next (plist)) != Mnil;
+	       plist = mplist_next (plist)) 
+	    mface_merge (face, mplist_value (plist));
+	  mplist_pop (plist);
+	}
+      update_top (0);
+      update_cursor (0, 1);
+      redraw (0, win_height, 1, 1);
+      show_cursor (NULL);
+      return;
+    }
 
   XtAppAddWorkProc (context, show_cursor, NULL);
   from = mtext_property_start (selection);
@@ -1806,7 +1983,7 @@ DumpImageProc (Widget w, XtPointer client_data, XtPointer call_data)
     mdump = popen ("mdump -q", "w");
   if (! mdump)
     return;
-  converter = mconv_stream_converter (Mcoding_utf_8, mdump);
+  converter = mconv_stream_converter (Mcoding_utf_8_full, mdump);
   mconv_encode_range (converter, mt, from, to);
   mconv_free_converter (converter);
   fclose (mdump);
@@ -1840,8 +2017,8 @@ input_status (MInputContext *ic, MSymbol command)
 int
 compare_input_method (const void *elt1, const void *elt2)
 {
-  const MInputMethod *im1 = *(MInputMethod **) elt1;
-  const MInputMethod *im2 = *(MInputMethod **) elt2;
+  const InputMethodInfo *im1 = elt1;
+  const InputMethodInfo *im2 = elt2;
   MSymbol lang1, lang2;
 
   if (im1->language == Mnil)
@@ -1858,18 +2035,29 @@ compare_input_method (const void *elt1, const void *elt2)
 }
 
 void
-setup_input_methods (int with_xim)
+setup_input_methods (int with_xim, char *initial_input_method)
 {
   MInputMethod *im = NULL;
-  MInputXIMArgIM arg_xim;
   MPlist *plist = mdatabase_list (msymbol ("input-method"), Mnil, Mnil, Mnil);
   MPlist *pl;
   int i = 0;
+  char *lang_name = NULL, *method_name = NULL;
+
+  if (initial_input_method)
+    {
+      char *p = strchr (initial_input_method, '-');
+      if (p)
+	lang_name = initial_input_method, method_name = p + 1, *p = '\0';
+      else
+	method_name = initial_input_method;
+    }
 
   num_input_methods = mplist_length (plist);
 
   if (with_xim)
     {
+      MInputXIMArgIM arg_xim;
+
       arg_xim.display = display;
       arg_xim.db = NULL;  
       arg_xim.res_name = arg_xim.res_class = NULL;
@@ -1879,9 +2067,16 @@ setup_input_methods (int with_xim)
       if (im)
 	num_input_methods++;
     }
-  input_method_table = calloc (num_input_methods, sizeof (MInputMethod *));
+  input_method_table = calloc (num_input_methods, sizeof (InputMethodInfo));
   if (im)
-    input_method_table[i++] = im;
+    {
+      input_method_table[i].available = 1;
+      input_method_table[i].language = Mnil;
+      input_method_table[i].name = im->name;
+      input_method_table[i].im = im;
+      i++;
+    }
+
   for (pl = plist; mplist_key (pl) != Mnil; pl = mplist_next (pl))
     {
       MDatabase *mdb = mplist_value (pl);
@@ -1889,9 +2084,9 @@ setup_input_methods (int with_xim)
 
       if (tag[1] != Mnil)
 	{
-	  im = minput_open_im (tag[1], tag[2], NULL);
-	  if (im)
-	    input_method_table[i++] = im;
+	  input_method_table[i].language = tag[1];
+	  input_method_table[i].name = tag[2];
+	  i++;
 	}
     }
 
@@ -1907,6 +2102,17 @@ setup_input_methods (int with_xim)
 	      (void *) input_status);
   mplist_put (minput_driver->callback_list, Minput_status_done,
 	      (void *) input_status);
+
+  if (method_name)
+    for (i = 0; i < num_input_methods; i++)
+      if (strcmp (method_name, msymbol_name (input_method_table[i].name)) == 0
+	  && (lang_name
+	      ? strcmp (lang_name, msymbol_name (input_method_table[i].language)) == 0
+	      : input_method_table[i].language == Mt))
+	{
+	  current_input_method = i;
+	  break;
+	}
 }
 
 
@@ -2008,7 +2214,7 @@ FileDialogProc (Widget w, XtPointer client_data, XtPointer call_data)
       m17n_object_unref (mt);
       if (fp)
 	{
-	  mt = mconv_decode_stream (Mcoding_utf_8, fp);
+	  mt = mconv_decode_stream (Mcoding_utf_8_full, fp);
 	  fclose (fp);
 	  if (! mt)
 	    mt = mtext ();
@@ -2167,8 +2373,14 @@ help_exit (char *prog, int exit_code)
   printf ("Display FILE on a window and allow users to edit it.\n");
   printf ("XT-OPTIONs are standard Xt arguments (e.g. -fn, -fg).\n");
   printf ("The following OPTIONs are available.\n");
+  printf ("  %-13s\n\t\t%s", "--fontset FONTSET",
+	  "Use the specified fontset\n");
+  printf ("  %-13s %s", "-s SIZE", "Font size in 1/10 point (default 120).\n");
+  printf ("  %-13s\n\t\t%s", "--im INPUT-METHOD",
+	  "Input method activated initially.\n");
   printf ("  %-13s %s", "--version", "print version number\n");
   printf ("  %-13s %s", "-h, --help", "print this message\n");
+	  
   exit (exit_code);
 }
 
@@ -2177,6 +2389,8 @@ main (int argc, char **argv)
 {
   Widget form, BodyWidget, w;
   char *fontset_name = NULL;
+  int fontsize = 0;
+  char *initial_input_method = NULL;
   int col = 80, row = 32;
   /* Translation table for TextWidget.  */
   String trans = "<Expose>: Expose()\n\
@@ -2207,7 +2421,7 @@ main (int argc, char **argv)
   setlocale (LC_ALL, "");
   /* Create the top shell.  */
   XtSetLanguageProc (NULL, NULL, NULL);
-  ShellWidget = XtOpenApplication (&context, "MEdit", NULL, 0, &argc, argv,
+  ShellWidget = XtOpenApplication (&context, "M17NEdit", NULL, 0, &argc, argv,
 				   NULL, sessionShellWidgetClass, NULL, 0);
   display = XtDisplay (ShellWidget);
   screen = XScreenNumberOfScreen (XtScreen (ShellWidget));
@@ -2220,7 +2434,7 @@ main (int argc, char **argv)
 	help_exit (argv[0], 0);
       else if (! strcmp (argv[i], "--version"))
 	{
-	  printf ("medit (m17n library) %s\n", VERSION);
+	  printf ("m17n-edit (m17n library) %s\n", VERSION);
 	  printf ("Copyright (C) 2003 AIST, JAPAN\n");
 	  exit (0);
 	}
@@ -2230,11 +2444,23 @@ main (int argc, char **argv)
 	  if (sscanf (argv[i], "%dx%d", &col, &row) != 2)
 	    help_exit (argv[0], 1);
 	}
+      else if (! strcmp (argv[i], "-s"))
+	{
+	  i++;
+	  fontsize = atoi (argv[i]);
+	  if (fontsize < 0)
+	    fontsize = 120;
+	}
       else if (! strcmp (argv[i], "--fontset"))
 	{
 	  i++;
 	  fontset_name = strdup (argv[i]);
 	}
+      else if (! strcmp (argv[i], "--im"))
+	{
+	  i++;
+	  initial_input_method = strdup (argv[i]);
+	}
       else if (! strcmp (argv[i], "--with-xim"))
 	{
 	  with_xim = 1;
@@ -2245,12 +2471,12 @@ main (int argc, char **argv)
 	}
       else
 	{
-	  fprintf (stderr, "Unknown option: %s", argv[i]);
+	  fprintf (stderr, "Unknown option: %s\n", argv[i]);
 	  help_exit (argv[0], 1);
 	}
     }
   if (! filename)
-    help_exit (argv[0], 1);
+    filename = strdup ("/dev/null");
 
   mdatabase_dir = ".";
   /* Initialize the m17n library.  */
@@ -2266,8 +2492,9 @@ main (int argc, char **argv)
   {
     MFace *face = mface ();
 
-    mface_put_prop (face, Mbackground, msymbol ("blue"));
-    mface_put_prop (face, Mforeground, msymbol ("yellow"));
+    mface_put_prop (face, Mforeground, msymbol ("blue"));
+    mface_put_prop (face, Mbackground, msymbol ("yellow"));
+    mface_put_prop (face, Mvideomode, Mreverse);
     selection = mtext_property (Mface, face, MTEXTPROP_NO_MERGE);
     m17n_object_unref (face);
   }
@@ -2278,25 +2505,34 @@ main (int argc, char **argv)
   XA_TEXT = XInternAtom (display, "TEXT", False);
   XA_COMPOUND_TEXT = XInternAtom (display, "COMPOUND_TEXT", False);
   XA_UTF8_STRING = XInternAtom (display, "UTF8_STRING", False);
+  Mcoding_compound_text = mconv_resolve_coding (msymbol ("compound-text"));
+  if (Mcoding_compound_text == Mnil)
+    FATAL_ERROR ("%s\n", "Don't know about COMPOUND-TEXT encoding!");
+
   {
     MPlist *plist = mplist ();
     MFace *face;
     MFont *font;
 
     mplist_put (plist, msymbol ("widget"), ShellWidget);
-    if (fontset_name)
+    if (fontset_name || fontsize > 0)
       {
 	MFontset *fontset = mfontset (fontset_name);
 	
 	face = mface ();
 	mface_put_prop (face, Mfontset, fontset);
+	mface_put_prop (face, Msize, (void *) fontsize);
 	m17n_object_unref (fontset);
 	mplist_add (plist, Mface, face);
 	m17n_object_unref (face);
       }
     frame = mframe (plist);
+    if (! frame)
+      FATAL_ERROR ("%s\n", "Fail to create a frame!");
     m17n_object_unref (plist);
-    face_default = (MFace *) mframe_get_prop (frame, Mface);
+    face_default = mface_copy ((MFace *) mframe_get_prop (frame, Mface));
+    default_face_list = mplist ();
+    mplist_add (default_face_list, Mt, face_default);
     face_default_fontset = mface ();
     mface_put_prop (face_default_fontset, Mfontset,
 		    mface_get_prop (face_default, Mfontset));
@@ -2334,6 +2570,8 @@ main (int argc, char **argv)
   mface_put_prop (face_dv_ttyogesh, Mfamily, msymbol ("dv-ttyogesh"));
   face_freesans = mface ();
   mface_put_prop (face_freesans, Mfamily, msymbol ("freesans"));
+  face_freeserif = mface ();
+  mface_put_prop (face_freeserif, Mfamily, msymbol ("freeserif"));
   face_freemono = mface ();
   mface_put_prop (face_freemono, Mfamily, msymbol ("freemono"));
 
@@ -2373,7 +2611,7 @@ main (int argc, char **argv)
     free (tib_font);
   }
 
-  setup_input_methods (with_xim);
+  setup_input_methods (with_xim, initial_input_method);
 
   gc = DefaultGC (display, screen);
 
@@ -2473,7 +2711,7 @@ main (int argc, char **argv)
     SetMenu (menus[1], 0, "auto", NULL, InputMethodProc, -1, 0);
     for (i = 0; i < num_input_methods; i++)
       {
-	MInputMethod *im = input_method_table[i];
+	InputMethodInfo *im = input_method_table + i;
 	char *name1, *name2;
 
 	if (im->language != Mnil && im->language != Mt)
@@ -2522,7 +2760,7 @@ main (int argc, char **argv)
 	= prop.color_left = prop.color_right = Mnil;
       prop.inner_hmargin = prop.inner_vmargin = 1;
       prop.outer_hmargin = prop.outer_vmargin = 0;
-      face_input_status = mface ();
+      face_input_status = mface_copy (face_default);
       mface_put_prop (face_input_status, Mbox, &prop);
     }
 
@@ -2687,9 +2925,7 @@ main (int argc, char **argv)
   memset (&control, 0, sizeof control);
   control.two_dimensional = 1;
   control.enable_bidi = 1;
-#if 0
   control.anti_alias = 1;
-#endif
   control.min_line_ascent = font_ascent;
   control.min_line_descent = font_descent;
   control.max_line_width = win_width;
@@ -2711,7 +2947,8 @@ main (int argc, char **argv)
   if (current_input_context)
     minput_destroy_ic (current_input_context);
   for (i = 0; i < num_input_methods; i++)
-    minput_close_im (input_method_table[i]);
+    if (input_method_table[i].im)
+      minput_close_im (input_method_table[i].im);
   m17n_object_unref (frame);
   m17n_object_unref (mt);
   m17n_object_unref (face_xxx_large);
@@ -2721,19 +2958,15 @@ main (int argc, char **argv)
   m17n_object_unref (face_times);
   m17n_object_unref (face_dv_ttyogesh);
   m17n_object_unref (face_freesans);
+  m17n_object_unref (face_freeserif);
   m17n_object_unref (face_freemono);
   m17n_object_unref (face_default_fontset);
   m17n_object_unref (face_no_ctl_fontset);
   m17n_object_unref (face_input_status);
+  m17n_object_unref (face_default);
+  m17n_object_unref (default_face_list);
   m17n_object_unref (selection);
 
-  M17N_FINI ();
-
-  free (fontset_name);
-  free (filename);
-  free (input_method_table);
-  free (InputMethodMenus);
-
   XFreeGC (display, mono_gc);
   XFreeGC (display, mono_gc_inv);
   XFreeGC (display, gc_inv);
@@ -2742,6 +2975,13 @@ main (int argc, char **argv)
   XtDestroyWidget (ShellWidget);
   XtDestroyApplicationContext (context);
 
+  M17N_FINI ();
+
+  free (fontset_name);
+  free (filename);
+  free (input_method_table);
+  free (InputMethodMenus);
+
   exit (0);
 }
 #endif /* not FOR_DOXYGEN */
