@@ -1,4 +1,4 @@
-/* imx-ispell.c -- Input method external module for Ispell.
+/* imx-ispell.c -- Ispell input method external module.	-*- coding: euc-jp; -*-
    Copyright (C) 2003, 2004
      National Institute of Advanced Industrial Science and Technology (AIST)
      Registration Number H15PRO112
@@ -21,7 +21,7 @@
    02111-1307, USA.  */
 
 /***en
-    @page mimx-ispell external module for the input method <en, ispell>
+    @enpage mimx-ispell external module for the input method <en, ispell>
 
     @section mimx-ispell-description DESCRIPTION
 
@@ -50,6 +50,36 @@
     @section mimx-ispell-seealso See also
     @ref mdbIM
 */
+/***ja
+    @japage mimx-ispell ϥ᥽å <en, ispell> ѳ⥸塼
+
+    @section mimx-ispell-description DESCRIPTION
+
+    ͭ饤֥ mimx-ispell.so ϥ᥽å <en, ispell> Ѥ 
+    볰⥸塼Ǥꡢʲδؿ export Ƥ롣
+
+    <ul>
+    <li> init
+
+    饤֥ν
+
+    <li> fini
+
+    饤֥νλ 
+
+    <li> ispell_word
+
+    ߤ preedit ƥ (ʸ) ֤Ĵ١ְäƤиΥ 
+    Ȥ֤
+
+    </ul>
+
+    Υץm17n ϥ᥽åѳ⥸塼ν򼨤 
+    ΤΤǤꡢºݤѤտޤΤǤϤʤ
+
+    @section mimx-ispell-seealso 
+    @ref mdbIM
+*/
 
 #ifndef FOR_DOXYGEN
 
@@ -61,6 +91,7 @@
 #ifdef HAVE_ISPELL
 
 static int initialized = 0;
+static int face_available;
 static MFace *mface_overstrike = NULL;
 
 static MPlist *
@@ -83,11 +114,16 @@ init (MPlist *args)
     {
       MFaceHLineProp hline;
 
-      hline.type = MFACE_HLINE_STRIKE_THROUGH;
-      hline.width = 1;
-      hline.color = msymbol ("black");
-      mface_overstrike = mface ();
-      mface_put_prop (mface_overstrike, Mhline, &hline);
+      face_available = 0;
+      if (m17n_status () == M17N_GUI_INITIALIZED)
+	{
+	  face_available = 1;
+	  hline.type = MFACE_HLINE_STRIKE_THROUGH;
+	  hline.width = 1;
+	  hline.color = msymbol ("black");
+	  mface_overstrike = mface ();
+	  mface_put_prop (mface_overstrike, Mhline, &hline);
+	}
     }
   return NULL;
 }
@@ -95,7 +131,9 @@ init (MPlist *args)
 MPlist *
 fini (MPlist *args)
 {
-  if (! --initialized)
+  if (initialized != 0
+      && --initialized == 0
+      && face_available)
     m17n_object_unref (mface_overstrike);
   return NULL;
 }
@@ -148,7 +186,8 @@ ispell_word (MPlist *args)
   if (*p == '#')
     {
       mt = mtext_dup (ic->preedit);
-      mtext_push_prop (mt, 0, mtext_len (mt), Mface, mface_overstrike);
+      if (face_available)
+	mtext_push_prop (mt, 0, mtext_len (mt), Mface, mface_overstrike);
       mplist_add (actions, Mtext, mt);
       add_action (actions, msymbol ("shift"), Msymbol, init_state);
       m17n_object_unref (mt);
@@ -182,7 +221,8 @@ ispell_word (MPlist *args)
       m17n_object_unref (mt);
     }
   mt = mtext_dup (ic->preedit);
-  mtext_push_prop (mt, 0, mtext_len (mt), Mface, mface_overstrike);
+  if (face_available)
+    mtext_push_prop (mt, 0, mtext_len (mt), Mface, mface_overstrike);
   mplist_add (candidates, Mtext, mt);
   m17n_object_unref (mt);
   plist = mplist_add (mplist (), Mplist, candidates);
