@@ -42,20 +42,20 @@
 
 /***ja
     @addtogroup m17nCharacter
-    @brief ʸ֥ȤȤ˴ؤ API
+    @brief ʸ֥ȤȤ˴ؤ API.
 
     m17n 饤֥ @e ʸ ʸɡˤɽ롣Ǿʸ
     ɤ @c 0 ǡʸɤϥޥ #MCHAR_MAX ˤä
-    Ƥ롣#MCHAR_MAX  @c 0x3FFFFF22ӥåȡ˰ʾǤ
+    Ƥ롣#MCHAR_MAX  @c 0x3FFFFF22ӥåȡ ʾǤ
     ȤݾڤƤ롣
 
     @c 0  @c 0x10FFFF ޤǤʸϡƱͤ Unicode 
     ʸ˳ƤƤ롣
 
-    ʸ @e ʸץѥƥ ȸƤ֥ץѥƥ0İʾĤȤǤ
-    롣ʸץѥƥ @e   @e  ʤ롣ϥܥǤ
+    ʸ @e ʸץѥƥ ȸƤ֥ץѥƥ 0 İʾĤȤ
+    롣ʸץѥƥ @e   @e  ʤ롣ϥܥ
     ꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤 
-    ʸ C ʸץѥƥΤ @c K ǤΡפñ
+    ʸ C ʸץѥƥΤ K ǤΡפñ
     ʸ C  K ץѥƥפȸƤ֤Ȥ롣  */
 /*=*/
 
@@ -151,7 +151,7 @@ mchar__fini (void)
     The macro #MCHAR_MAX gives the maximum character code.  */
 
 /***ja
-    @brief ʸɤκ
+    @brief ʸɤκ.
 
     ޥ #MCHAR_MAX ʸɤκͤͿ롣  */
 
@@ -183,7 +183,7 @@ mchar__fini (void)
     the <em>Unicode Technical Report #24</em>.  */
 
 /***ja
-    @brief ץȤɽ魯
+    @brief ץȤɽ魯.
 
     ܥ #Mscript  <tt>"script"</tt> Ȥ̾ʸ
     ѥƥΥȤƻȤ롣Υץѥƥͤϡʸ°
@@ -204,7 +204,7 @@ MSymbol Mscript;
     C-string representing the name of the character.  */
 
 /***ja
-    @brief ̾ɽ魯
+    @brief ̾ɽ魯.
 
     ܥ #Mname  <tt>"name"</tt> Ȥ̾ʸץ
     ƥΥȤƻȤ롣Υץѥƥͤ C-string Ǥꡢ
@@ -227,7 +227,7 @@ MSymbol Mname;
     Unicode.  */
 
 /***ja
-    @brief ̥ƥɽ魯
+    @brief ̥ƥɽ魯.
 
     ܥ #Mcategory  <tt>"category"</tt> Ȥ̾ʸ
     ץѥƥΥȤƻȤ롣Υץѥƥͤϡб
@@ -252,7 +252,7 @@ MSymbol Mcategory;
     class is identical to the one defined in Unicode.  */
 
 /***ja
-    @brief ɸ祯饹ɽ魯
+    @brief ɸ祯饹ɽ魯.
 
     ܥ #Mcombining_class  <tt>"combining-class"</tt> Ȥ
     ̾ʸץѥƥΥȤƻȤ롣Υץѥƥ
@@ -277,7 +277,7 @@ MSymbol Mcombining_class;
     Unicode.  */
 
 /***ja
-    @brief ƥɽ魯
+    @brief ƥɽ魯.
 
     ܥ #Mbidi_category  <tt>"bidi-category"</tt> Ȥ̾
     ʸץѥƥΥȤƻȤ롣Υץѥƥͤϡ
@@ -304,20 +304,21 @@ MSymbol Mbidi_category;
     another property whose key is #Mcomplicated_case_folding.  */
 
 /***ja
-    @brief б뾮ʸʸɽ魯
+    @brief б뾮ʸʸɽ魯.
 
     ܥ #Msimple_case_folding  <tt>"simple-case-folding"</tt> 
     Ȥ̾ʸץѥƥΥȤƻȤ롣Υץѥƥ
     ͤϡб뾮ʸʸǤꡢʸʸζ̵̤뤷
     ʸӤκݤ˻Ȥ롣
 
-    ⤷Τ褦Ӥ˻Ѥñξʸ¸ߤʤ硢Υ
-    ѥƥͤ 0xFFFF ˤʤ롣ξ礽ʸϡ
-    #Mcomplicated_case_folding Ȥʸץѥƥġ  */
+    ʣˡɬפȤʸǤä̤ΰʸбդ
+    ȤˤäӤǤʤˡΥץѥƥͤ @c 0xFFFF 
+    ʤ롣ξ礽ʸϡ#Mcomplicated_case_folding Ȥ
+    ʸץѥƥġ  */
 
 MSymbol Msimple_case_folding;
 /***en
-    @brief Key for corresponding multiple lowercase characters
+    @brief Key for corresponding multiple lowercase characters.
 
     The symbol #Mcomplicated_case_folding has the name
     <tt>"complicated-case-folding"</tt> and is used as the key of a
@@ -326,7 +327,7 @@ MSymbol Msimple_case_folding;
     characters to be used for comparing M-texts ignoring case.  */
 
 /***ja
-    @brief б뾮ʸɽ魯
+    @brief б뾮ʸɽ魯.
 
     ܥ #Mcomplicated_case_folding  
     <tt>"complicated-case-folding"</tt> Ȥ̾ʸץѥƥ
@@ -353,7 +354,7 @@ MSymbol Mcomplicated_case_folding;
     $SYM.  Otherwise it returns #Mnil.  */
 
 /***ja
-    @brief ʸץѥƥ
+    @brief ʸץѥƥ.
 
     ؿ mchar_define_property () ϡ \<#Mchar_table, $TYPE, $SYM \>
     Ȥäǡ١ m17n ١õ  
@@ -361,7 +362,7 @@ MSymbol Mcomplicated_case_folding;
     #Mtext, #Msymbol, #Minteger, #Mplist Τ줫ǤʤФʤʤ
 
     @return
-     $SYM ֤
+     mchar_define_property () $SYM ֤
     Ԥ #Mnil ֤  */
 
 /***
@@ -372,7 +373,7 @@ MSymbol Mcomplicated_case_folding;
     mchar_get_prop (), mchar_put_prop ()  */
 
 MSymbol
-mchar_define_property (char *name, MSymbol type)
+mchar_define_property (const char *name, MSymbol type)
 {
   MSymbol key = msymbol (name);
   MCharPropRecord *record;
@@ -422,18 +423,17 @@ mchar_define_property (char *name, MSymbol type)
     NULL.  */
 
 /***ja
-    @brief ʸץѥƥͤ
+    @brief ʸץѥƥͤ.
 
-    ؿ mchar_get_prop () ϡʸ $C ʸץѥƥ桢 $KEY 
-    ǤΤõ
+    ؿ mchar_get_prop () ϡʸ $C ʸץѥƥΤ 
+    $KEY ǤΤõ
 
     @return
      mchar_get_prop () ϸĤäץѥƥͤ
-    Ԥ #Mnil ֤
+    Ԥ @c NULL ֤
 
     @latexonly \IPAlabel{mchar_get_prop} @endlatexonly
 */
-
 /***
     @errors
     @c MERROR_SYMBOL, @c MERROR_DB
@@ -471,17 +471,15 @@ mchar_get_prop (int c, MSymbol key)
     @return
     If the operation was successful, mchar_put_prop () returns 0.
     Otherwise, it returns -1.  */
-
 /***ja
-    @brief ʸץѥƥͤꤹ
+    @brief ʸץѥƥͤꤹ.
 
-    ؿ mchar_put_prop () ʸ $C ʸץѥƥ桢 $KEY 
+    ؿ mchar_put_prop () ϡʸ $C ʸץѥƥΤ $KEY 
     ΤõͤȤ $VAL ꤹ롣
 
     @return
      mchar_put_prop () 0֤Ԥ-1
       */
-
 /***
     @errors
     @c MERROR_SYMBOL, @c MERROR_DB
@@ -502,7 +500,6 @@ mchar_put_prop (int c, MSymbol key, void *val)
       record->table = (*mdatabase__loader) (record->mdb);
       if (! record->table)
 	MERROR (MERROR_DB, -1);
-      M17N_OBJECT_REF (record->table);
       record->mdb = NULL;
     }
   return mchartable_set (record->table, c, val);
