@@ -19,7 +19,6 @@
    License along with the m17n library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307, USA.  */
-
 /***en
     @addtogroup m17nCharset
     @brief Charset objects and API for them.
@@ -53,30 +52,33 @@
 
 /***ja
     @addtogroup m17nCharset
-    @brief ʸåȥ֥ȤȤ˴ؤ API
+    @brief ʸåȥ֥ȤȤ˴ؤ API.
 
     m17n 饤֥ϡ沽ʸ (CCS)  @e ʸå ȸƤ֥
-    ֥Ȥɽ롣m17n 饤֥¿沽ʸͽ
-    ݡȤƤ뤬ץꥱץबȼʸåȤ
+    ֥Ȥɽ롣m17n 饤֥¿沽ʸ򤢤餫
+    ݡȤƤ뤷ץꥱץबȼʸåȤ
     ɲä뤳ȤǽǤ롣ĤʸʣʸåȤ°Ƥ
     
 
-    m17n 饤֥ˤϡʲΰۤʤ복ǰ:
+    m17n 饤֥ϡʲγǰ̤Ƥ:
 
     @li @e ɥݥ ȤϡCCS θġʸФ
-    ͤǤ롣ɥݥȤϢ³ƤȤϸ¤ʤ
+    ͤǤ롣ɥݥȤϢ³ƤȤϸ¤ʤɥݥȤ
+    @c unsigned ˤäɽ롣̵ʥɥݥȤϥޥ
+    @c MCHAR_INVALID_CODE ɽ롣
 
     @li @e ʸǥå ȤϡCCS ǳʸ˳Ƥ
-    줿ǥåǤ롣ʸǥåNʸϡCCS ʸ
-    ɥݥȤǥȤȤNܤ롣
+    줿ǥåǤ롣ʸǥå N ʸϡCCS 
+    ʸ򥳡ɥݥȽ¤٤Ȥ N ܤ˸롣CCS 
+    ʸǥåϢ³Ƥꡢ0 Ϥޤ롣
 
-    @li @e ʸɤȤϡm17n 饤֥ˤʸɽǤ
+    @li @e ʸ Ȥϡm17n 饤֥ˤʸɽǤ
     ꡢ21 ӥåȰʾĹդǤ롣
 
-    ʸåȥ֥Ȥϡ°ʸΥɥݥȤʸ
-    ɤȤѴꤹ롣ɥݥȤʸɤؤѴ
-     @e ǥ ȸƤӡʸɤ饳ɥݥȤؤѴ @e 
-    󥳡 ȸƤ֡  */
+    ʸåȥ֥ȤϡʸåȤ°ʸΥɥݥ
+    ȤʸɤȤδ֤Ѵꤹ롣ɥݥȤʸ
+    ɤؤѴ @e ǥ ȸƤӡʸɤ饳ɥݥȤؤ
+    Ѵ @e 󥳡 ȸƤ֡  */
 
 /*=*/
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
@@ -618,9 +620,6 @@ mcharset__load_from_database ()
       p = mplist__from_plist (pl);
       mchar_define_charset (MSYMBOL_NAME (name), p);
       M17N_OBJECT_UNREF (p);
-      if ((pl = mplist_find_by_value (pl, Mdefine_coding))
-	  && (MSymbol) MPLIST_VAL (MPLIST_NEXT (pl)) == Mt)
-	mconv__register_charset_coding (name);
     }
 
   M17N_OBJECT_UNREF (def_list);
@@ -646,7 +645,7 @@ mcharset__load_from_database ()
     The macro #MCHAR_INVALID_CODE gives the invalid code-point.  */
 
 /***ja
-    @brief ̵ʥɥݥ
+    @brief ̵ʥɥݥ.
 
     ޥ #MCHAR_INVALID_CODE ̵ʥɥݥȤͿ롣  */
 
@@ -661,11 +660,11 @@ mcharset__load_from_database ()
     <tt>"charset"</tt>.  */
 
 /***ja
-    @brief ܥ @c Mcharset
+    @brief ܥ @c Mcharset.
 
     ǥɤ줿 M-text ϡ @c Mcharset Ǥ褦ʥƥ
     ץѥƥġܥ @c Mcharset  <tt>"charset"</tt> Ȥ
-    ̾Ǥ餫Ƥ롣  */
+    ̾ġ  */
 
 MSymbol Mcharset;
 /*=*/
@@ -676,11 +675,9 @@ MSymbol Mcharset;
     Each of the following symbols represents a predefined charset.  */
 
 /***ja
-    @name ѿ: ʸåȤɽ魯Ѥߥܥ
+    @name ѿ: ʸåȤɽѤߥܥ
 
-    ʲγƥܥϡ @c Mcharset Ǥꡢͤбʸ
-    ȥ֥ȡ @c MCharset ˤؤΥݥ󥿤Ǥ륷ܥץ
-    ƥġ  */
+    ʲγƥܥϡѤʸåȤɽ롣  */
 /*=*/
 /*** @{ */
 /*=*/
@@ -690,11 +687,11 @@ MSymbol Mcharset;
     The symbol #Mcharset_ascii has name <tt>"ascii"</tt> and represents
     the charset ISO 646, USA Version X3.4-1968 (ISO-IR-6).  */
 /***ja
-    @brief ISO 646, USA Version бʸåȤΥܥ
+    @brief ASCII ʸåȤɽ륷ܥ.
 
     ܥ #Mcharset_ascii  <tt>"ascii"</tt> Ȥ̾ 
-    ISO 646, USA Version X3.4-1968 (ISO-IR-6) бʸåȤ
-    ꤹ뤿˻Ȥ롣  */
+    ISO 646, USA Version X3.4-1968 (ISO-IR-6) ʸåȤɽ롣
+     */
 
 MSymbol Mcharset_ascii;
 
@@ -705,11 +702,11 @@ MSymbol Mcharset_ascii;
     The symbol #Mcharset_iso_8859_1 has name <tt>"iso-8859-1"</tt>
     and represents the charset ISO/IEC 8859-1:1998.  */
 /***ja
-    @brief ISO/IEC 8859-1:1998 бʸåȤΥܥ
+    @brief ISO/IEC 8859-1:1998 ʸåȤɽ륷ܥ.
 
     ܥ #Mcharset_iso_8859_1  <tt>"iso-8859-1"</tt> Ȥ̾
-    ISO/IEC 8859-1:1998 бʸåȤꤹ뤿
-    Ȥ롣 */
+    ISO/IEC 8859-1:1998 ʸåȤɽ롣
+    */
 
 MSymbol Mcharset_iso_8859_1;
 
@@ -719,10 +716,10 @@ MSymbol Mcharset_iso_8859_1;
     The symbol #Mcharset_unicode has name <tt>"unicode"</tt> and
     represents the charset Unicode.  */
 /***ja
-    @brief Unicode бʸåȤΥܥ
+    @brief Unicode ʸåȤɽ륷ܥ.
 
     ܥ #Mcharset_unicode  <tt>"unicode"</tt> Ȥ̾
-    Unicode бʸåȤꤹ뤿˻Ȥ롣 */
+    Unicode ʸåȤɽ롣 */
 
 MSymbol Mcharset_unicode;
 
@@ -734,11 +731,10 @@ MSymbol Mcharset_unicode;
     represents the charset that contains all characters supported by
     the m17n library.  */ 
 /***ja
-    @brief ʸޤʸåȤΥܥ
+    @brief ʸޤʸåȤɽ륷ܥ.
 
     ܥ #Mcharset_m17n  <tt>"m17n"</tt> Ȥ̾
-    m17n 饤֥꤬ƤʸޤʸåȤꤹ뤿˻
-    롣 */
+    m17n 饤֥꤬ƤʸޤʸåȤɽ롣 */
 
 MSymbol Mcharset_m17n;
 
@@ -749,8 +745,16 @@ MSymbol Mcharset_m17n;
     The symbol #Mcharset_binary has name <tt>"binary"</tt> and
     represents the fake charset which the decoding functions put to an
     M-text as a text property when they encounter an invalid byte
-    (sequence).  See @ref m17nConv @latexonly
-    (P.\pageref{group__m17nConv}) @endlatexonly for more detail.  */
+    (sequence).  See @ref m17nConv for more details.  */
+
+/***ja
+    @brief ǥɤǤʤʸʸåȤɽ륷ܥ.
+
+    ܥ #Mcharset_binary  <tt>"binary"</tt> Ȥ̾
+     (fake) ʸåȤɽ롣ǥɴؿϡM-text Υƥ
+    ȥץѥƥȤơ̵ʥХȡʥ󥹡ˤ֤ղä롣
+
+     ܺ٤ @ref m17nConv ȤΤȡ */
 
 MSymbol Mcharset_binary;
 
@@ -775,6 +779,9 @@ MSymbol Mcharset_binary;
 /***en
     Parameter key for mchar_define_charset () (which see). */ 
 
+/***ja
+    ؿ mchar_define_charset () ѤΥѥ᡼. */ 
+
 MSymbol Mmethod;
 MSymbol Mdimension;
 MSymbol Mmin_range;
@@ -808,8 +815,9 @@ MSymbol Maliases;
 /***ja
     @name ѿ: ʸåȤΥ᥽åɻ˻Ȥ륷ܥ
 
-    ϡʸåȤ @e ᥽å ꤹ뤿ΥܥǤꡢ
-    ؿ mchar_define_charset () ΰȤƻȤ롣
+    ϡʸåȤ @e ᥽å ꤹ뤿Ѥߥ
+    ǤꡢʸåȤ #Mmethod ѥ᡼ͤȤʤ뤳ȤǤ롣
+    ͤϴؿ mchar_define_charset () ΰȤƻȤ롣
 
     ᥽åɤȤϡɥݥȤʸɤѴݤΤ
     ȤǤ롣ܤϴؿ mchar_define_charset () β򻲾ȤΤȡ  */
@@ -831,28 +839,38 @@ CHARACTER-CODE = CODE-POINT - MIN-CODE + MIN-CHAR
     and MIN-CHAR is a value of #Mmin_char parameter.  */
 
 /***ja
-    @brief եåȷΥ᥽åɤ򼨤ܥ
+    @brief եåȷΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Moffset  <tt>"offset"</tt> Ȥ̾
-    mchar_define_charset () ǥեåȷΥ᥽åɤꤹΰ
-    ȤѤ롣*/
+    ܥ #Moffset  <tt>"offset"</tt> Ȥ̾ʸ
+    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥ
+    ȤʸåȤʸɤδ֤Ѵʲμ˽äƹԤ뤳
+    ̣롣
+
+@verbatim
+ʸ = ɥݥ - MIN-CODE + MIN-CHAR
+@endverbatim
+
+    ǡMIN-CODE ʸåȤ #Mmin_code ѥ᡼ͤǤꡢMIN-CHAR 
+    #Mmin_char ѥ᡼ͤǤ롣 */
 
 MSymbol Moffset;
 /*=*/
 
 /***en @brief Symbol for the map type method of charset.
 
-    The symbol #Mmap has the name <tt>"map"</tt> and, when use as a
+    The symbol #Mmap has the name <tt>"map"</tt> and, when used as a
     value of #Mmethod parameter of a charset, it means that the
     conversion of code-points and character codes of the charset is
     done by map looking up.  The map must be given by #Mmapfile
     parameter.  */
 
-/***ja @brief ޥå׷Υ᥽åɤ򼨤ܥ
+/***ja @brief ޥå׷Υ᥽åɤ򼨤ܥ.
 
-    ܥ #Mmap  <tt>"map"</tt> Ȥ̾
-    mchar_define_charset () ǥޥå׷Υ᥽åɤꤹΰ
-    Ѥ롣*/
+    ܥ #Mmap  <tt>"map"</tt> Ȥ̾ʸåȤ 
+    #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥȤ
+    ʸåȤʸɤδ֤Ѵޥåפ򻲾Ȥ뤳ȤˤäƹԤ
+    뤳Ȥ̣롣ޥåפ #Mmapfile ѥ᡼ȤͿʤ
+    Фʤʤ */
 
 MSymbol Mmap;
 /*=*/
@@ -864,7 +882,8 @@ MSymbol Mmap;
     conversion of code-points and character codes of the charset is
     done by map looking up and offsetting.  The map must be given by
     #Mmapfile parameter.  For this kind of charset, a unique
-    consequent character code space for all characters is assigned.
+    continuous character code space for all characters is assigned.
+
     If the map has an entry for a code-point, the conversion is done
     by looking up the map.  Otherwise, the conversion is done by this
     calculation:
@@ -877,11 +896,26 @@ CHARACTER-CODE = CODE-POINT - MIN-CODE + LOWEST-CHAR-CODE
     and LOWEST-CHAR-CODE is the lowest character code of the assigned
     code space.  */
 
-/***ja @brief ³Υ᥽åɤ򼨤ܥ
+/***ja @brief ˥եΥ᥽åɤ򼨤ܥ.
 
-    ܥ #Minherit  <tt>"inherit"</tt> Ȥ̾
-    mchar_define_charset () ³Υ᥽åɤꤹΰȤ
-    Ѥ롣*/
+    ܥ #Minherit  <tt>"unify"</tt> Ȥ̾ʸ
+    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡɥݥ
+    ȤʸåȤʸɤδ֤ѴޥåפλȤȥեåȤ
+    Ȥ߹碌ˤäƹԤ뤳Ȥ̣롣ޥåפ #Mmapfile ѥ
+    ᡼ȤͿʤФʤʤμγʸåȤˤϡʸ
+    ФϢ³륳ɥڡ줾Ƥ롣
+
+    ɥݥȤޥåפ˴ޤޤƤСѴϥޥå׻Ȥˤäƹ
+    롣ǤʤСʲμ˽
+
+@verbatim
+CHARACTER-CODE = CODE-POINT - MIN-CODE + LOWEST-CHAR-CODE
+@endverbatim
+    
+    ǡMIN-CODE ʸåȤ #Mmin_code ѥ᡼ͤǤꡢ
+    LOWEST-CHAR-CODE ϳƤ줿ɥڡκǤ⾮ʸ
+    ɤǤ롣
+    */
 
 MSymbol Munify;
 /*=*/
@@ -904,6 +938,24 @@ CHARACTER-CODE = PARENT-CODE (CODE-POINT) + SUBSET-OFFSET
     code of CODE-POINT in the parent charset, and SUBSET-OFFSET is a
     value given by #Msubset_offset parameter.  */
 
+/***ja @brief ֥åȷΥ᥽åɤ򼨤ܥ.
+
+    ܥ #Msubset  <tt>"subset"</tt> Ȥ̾ʸ
+    Ȥ #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸ
+    Ȥ̤ʸåȡʿʸåȡˤʬǤ뤳Ȥ̣롣
+    ʸåȤ #Mparents ѥ᡼ˤäͿʤƤϤʤʤ
+    ɥݥȤʸåȤʸɤδ֤ѴϡǰŪˤ
+    ʲμ˽
+
+@verbatim
+CHARACTER-CODE = PARENT-CODE (CODE-POINT) + SUBSET-OFFSET
+@endverbatim
+
+     PARENT-CODE  CODE-POINT οʸåǤʸɤ
+    ֤ؿǤꡢSUBSET-OFFSET  #Msubset_offset ѥ᡼Ϳ
+    ͤǤ롣
+    */
+
 MSymbol Msubset;
 /*=*/
 
@@ -915,6 +967,16 @@ MSymbol Msubset;
     the charset is a superset of parent charsets.  The parent charsets
     must be given by #Mparents parameter.  */
 
+/***ja
+    @brief ѡåȷΥ᥽åɤ򼨤ܥ.
+
+    ܥ #Msuperset  <tt>"superset"</tt> Ȥ̾ʸ
+    åȤ #Mmethod ѥ᡼ͤȤѤ줿ˤϡʸ
+    åȤ̤ʸåȡʿʸåȡˤξ̽Ǥ뤳Ȥ̣
+    롣ʸåȤ #Mparents ѥ᡼ˤäͿʤƤϤ
+    ʤ
+    */
+
 MSymbol Msuperset;
 /*=*/
 /*** @}  */ 
@@ -978,7 +1040,7 @@ MSymbol Msuperset;
     <li> Key is #Mrevision, value is an integer
 
     The value specifies the @e revision @e number of the charset
-    registered in The International Registry.  it must be 0..127.  If
+    registered in The International Registry.  It must be 0..127.  If
     the charset is not in The International Registry, the value is
     ignored.  The value 0 means that the charset has no revision
     number.
@@ -1012,8 +1074,8 @@ MSymbol Msuperset;
     <li> Key is #Mdefine_coding, value is a symbol
 
     If the dimension of the charset is 1, the value specifies whether
-    or not to define a coding system of the same name whose method is
-    @c charset.
+    or not to define a coding system of the same name whose type is
+    #Mcharset.  A coding system is defined if the value is not #Mnil.
 
     Otherwise, this parameter is ignored.
 
@@ -1028,68 +1090,107 @@ MSymbol Msuperset;
     @brief ʸåȤ.
 
     ؿ mchar_define_charset () ϿʸåȤ 
-    $NAME Ȥ̾ΥܥͳǥǤ褦ˤ롣$METHOD 
-    ϤʸåȤˤ륳ɥݥȤΥǥɡ󥳡ɥ᥽
-    ɤꤹ륷ܥǤꡢ#Moffset, #Mmap, #Munify,
-    #Msubset, #Msuperset Τ줫ͤȤ롣
+    $NAME Ȥ̾ĥܥͳǥǤ褦ˤ롣
+    $PLIST ʸåȤΥѥ᡼ʲΤ褦˻ꤹ롣
 
-    $DIMENSION ϤʸåȤΥɥݥȤμǤꡢ1, 2, 3,
-    4Τ줫ͤȤ롣
+    <ul>
 
-    $CODE_RANGE 礭8ХȤǤꡢʸåȤ
-    ɥݥȶ֤ɽ魯1ХȤ2ХȤͤϥɥݥ
-    ȤκǽμǤκǾХȤͤǤ롣3ХȤ4Х
-    ϡ2ܤμκǾͤǤꡢ ʲƱͤ³Ūˡ
-    (2N-1)ХȤ(2N)ХȤNܤμκǾͤȤʤ (N =
-    1, 2, 3, 4)ɥݥȤ @e ʸǥå Ϥͤ
-    ׻롣
+    <li>  #Mmethod ͤܥλ
 
-    $MIN_CODE  $MAX_CODE ϡ줾줳ʸåȤκǾӺ
-    ɥݥȤɽ魯0ꤵ줿 $CODE_RANGE ͤ
+    ͤϡ#Moffset, #Mmap (ǥե), #Munify, #Msubset,
+    #Msuperset Τ줫ǤꡢʸåȤΥɥݥȤǥɡ
+    󥳡ɤݤΥ᥽åɤꤹ롣
+
+    <li>  #Mdimension ͤͤλ
+
+    ͤϡ1 (ǥե), 2, 3, 4 Τ줫ǤꡢʸåȤΥ
+    ɥݥȤμǤ롣
+
+    <li>  #Mmin_range ͤͤλ
+
+    ͤϥɥݥȤκǾͤǤ롣ʤͤ N ܤΥ
+    ȤϤʸåȤΥɥݥȤ N ܤΥХȤκǾΤ
+    Ȥʤ롣ǥեͤ 0 
+
+    <li>  #Mmax_range ͤͤλ
+
+    ͤϥɥݥȤκͤǤ롣ʤͤ N ܤΥ
+    ȤϤʸåȤΥɥݥȤ N ܤΥХȤκΤ
+    Ȥʤ롣ǥեͤϡɥݥȤμ 1, 2, 3, 4 λ
+    줾 0xFF, 0xFFFF, 0xFFFFFF, 0xFFFFFFFF 
+
+    <li>  #Mmin_code ͤͤλ
+
+    ͤϤʸåȤκǾΥɥݥȤǤ롣ǥեͤ 
+    #Mmin_range ͡
+
+    <li>  #Mmax_code ͤͤλ
+
+    ͤϤʸåȤκΥɥݥȤǤ롣ǥեͤ 
+    #Mmax_range ͡
+
+    <li>   #Mascii_compatible ͤܥλ
+
+    ͤϤʸåȤ ASCII ߴǤ뤫ɤ򼨤ǥեͤ
+    #Mnil ǤиߴǤϤʤʳξϸߴǤ롣
+
+    <li>   #Mfinal_byte ͤͤλ
+
+    ͤϤʸåȤ The International Registry ϿƤ 
+    @e üХ Ǥꡢ0 (ǥե) Ǥ뤫 32..127 Ǥ롣0 
+    ϿƤʤȤ̣롣
+
+    <li>   #Mrevision ͤͤλ
+
+    ͤ The International Registry ϿƤ @e revision @e
+    number Ǥꡢ0..127 Ǥ롣ʸåȤϿƤʤˤ
+    ̵ͤ뤵롣 0  revision number ¸ߤʤȤ̣
     롣
 
-    $FINAL_BYTE  The International Registry ϿƤ
-    @e üХ Ǥ롣ϿƤʤ CCS ξˤ -1 ǤʤƤ
-    ʤʤ
+    <li>   #Mmin_char ͤͤλ
 
-    $REVISION ϡThe International Registry ϿƤ@e revision
-    @e number Ǥ롣⤷ revision number ¸ߤʤʤ -1 Ǥʤ
-    ƤϤʤʤ
+    ͤϤʸåȤκǾʸɤǤ롣ǥեͤ 0 
 
-    @par ᥽åɤ Moffset ξ
+    <li>  #Mmapfile ͤ M-text λ
 
-    $MIN_CHAR ˤϺǾΥɥݥȤбʸɤͿ롣
-    $NPARENTS, $PARENTS,  $SUBSET_OFFSET ̵뤵롣
+    ᥽åɤ #Mmap  #Munify λؿ mdatabase_define () 򤳤
+    ͤ $EXTRA_INFO ȤƸƤ֤Ȥˤäơޥåԥ󥰤˴ؤǡ
+     m17n ǡ١ɲä롣ʤͤϥǡե
+    ̾Ǥ롣
 
-    @par ᥽åɤ Mmap ξ
+    ǤʤСΥѥ᡼̵뤵롣
 
-    m17n ١ \<#Mcharset, $NAME\> Ȥդ
-    ޥåԥ󥰥ơ֥򡢥ǥɤӥ󥳡ɤѤ롣$MIN_CHAR,
-    $NPARENTS, $PARENTS,  $SUBSET_OFFSET ̵뤵롣
+    <li>  #Mparents ͤ plist λ
 
-    @par ᥽åɤ Msubset ξ
+    ᥽åɤ #Msubset ʤСͤĹ 1  plist ǤꡢͤϤ
+    ʸåȤξ̽ȤʤʸåȤ򼨤ܥǤ롣
 
-    $NPARENTS 1ǤʤФʤʤޤ $PARENTS ϷѾθȤʤʸ
-    åȤɽ魯ܥؤΥݥ󥿤Ǥ롣ʸåȤΥ
-    ݥȤ $SUBSET_OFFSET äΤʸåǤΥ
-    ɥݥȤˤʤ롣$MIN_CHAR ̵뤵롣
+    ᥽åɤ #Msuperset ʤСͤĹ 8 ʲ plist Ǥꡢ
+    ͤϤʸåȤβ̽ǤʸåȤ򼨤ܥǤ
+    롣
 
-    @par ᥽åɤ Msuperset ξ
+    ǤʤСΥѥ᡼̵뤵롣
 
-    $NPARENTS ϿƤȤʤʸåȤο$PARENTS ϿʸåȤΥ
-    ܥɽ魯$MIN_CHAR  $SUBSET_OFFSET ̵뤵롣
+    <li>   #Mdefine_coding ͤܥλ
 
-    @return
-    Сmchar_define_charset ()  $NAME Ȥ̾Υ
-    ܥ֤Ǥʤ #Mnil ֤ѿ @c
-    merror_code ˥顼ɤꤹ롣 */
+    ʸåȤμ 1 ʤСͤ #Mcharset פƱ̾Υ
+    ɷϤ뤫ɤꤹ롣ͤ #Mnil ʳξ롣
+
+    ǤʤСΥѥ᡼̵뤵롣
+
+    </ul>
+
+    @return 
+    Сmchar_define_charset()  $NAME Ȥ̾
+    Υ ܥ֤Ǥʤ #Mnil ֤ѿ 
+    #merror_code ˥顼ɤꤹ롣*/
 
 /***
     @errors
     @c MERROR_CHARSET  */
 
 MSymbol
-mchar_define_charset (char *name, MPlist *plist)
+mchar_define_charset (const char *name, MPlist *plist)
 {
   MSymbol sym = msymbol (name);
   MCharset *charset;
@@ -1202,6 +1303,16 @@ mchar_define_charset (char *name, MPlist *plist)
     charset name, and if the canonicalized name represents a charset,
     return it.  Otherwise, return #Mnil.  */
 
+/***ja
+    @brief ʸå̾褹.
+
+    ؿ mchar_resolve_charset ()  $SYMBOL ʸåȤ򼨤Ƥ
+    Ф֤
+
+    ǤʤС$SYMBOL ʸå̾Ȥ줬ʸ
+    Ȥ򼨤ƤƤΤ֤ǤʤС#Mnil 
+    ֤ */
+
 MSymbol
 mchar_resolve_charset (MSymbol symbol)
 {
@@ -1219,12 +1330,19 @@ mchar_resolve_charset (MSymbol symbol)
 /*=*/
 
 /***en
-    @brief List symbols representing a charset.
+    @brief List symbols representing charsets.
 
     The mchar_list_charsets () function makes an array of symbols
     representing a charset, stores the pointer to the array in a place
     pointed to by $SYMBOLS, and returns the length of the array.  */
 
+/***ja
+    @brief ʸåȤɽ魯ܥ󤹤.
+
+    ؿ mchar_list_charsets () ϡʸåȤ򼨤ܥ¤٤
+    ꡢ$SYMBOLS ǥݥȤ줿ˤؤΥݥ󥿤
+    Ĺ֤ */
+
 int
 mchar_list_charset (MSymbol **symbols)
 {
@@ -1250,7 +1368,7 @@ mchar_list_charset (MSymbol **symbols)
     character code.  Otherwise it returns -1.  */
 
 /***ja
-    @brief ɥݥȤǥɤ
+    @brief ɥݥȤǥɤ.
 
     ؿ mchar_decode () ϡܥ $CHARSET_NAME Ǽʸ
      $CODE ȤɥݥȤǥɤʸɤ롣
@@ -1286,7 +1404,7 @@ mchar_decode (MSymbol charset_name, unsigned code)
     code-point.  Otherwise it returns #MCHAR_INVALID_CODE.  */
 
 /***ja
-    @brief ʸɤ򥨥󥳡ɤ
+    @brief ʸɤ򥨥󥳡ɤ.
 
     ؿ mchar_encode () ϡʸ $C 򥨥󥳡ɤƥܥ
     $CHARSET_NAME Ǽʸåˤ륳ɥݥȤ롣
@@ -1329,7 +1447,7 @@ mchar_encode (MSymbol charset_name, int c)
     variable #merror_code.  */
 
 /***ja
-    @brief ꤷʸåȤΤ٤ƤʸФƴؿƤ
+    @brief ꤷʸåȤΤ٤ƤʸФƴؿƤ.
 
     ؿ mcharset_map_chars ()  $CHARSET_NAME Ȥ̾ʸ
     Τ٤ƤʸФ $FUNC Ƥ֡ƤӽФϰʸǤϤ
