@@ -44,18 +44,22 @@
 /***ja
     @addtogroup m17nChartable ʸơ֥
 
-    @brief ʸơ֥Ȥ˴ؤ API
+    @brief ʸơ֥Ȥ˴ؤ API.
 
     m17n 饤֥꤬ʸζ֤ϹǤ뤿ᡢʸξñ
     ˳Ǽ褦ȤȡϵˤʤꤹŪ
     롣̾ɬפȤʤʸˤĤƤξϡϰϤʸ
     ˤΤդƤ뤳Ȥ¿ʸ˴ؤƾ󤬤ˤ⡢Ϣ
-    ³ʸɤʸƱĤȤ¿ǡ
-    褦ʾˤƸΨŪ˾ǼǤΤȤơm17n 饤
-     @e ʸơ֥ (chartable) ȸƤ֥֥ȤѤ롣
-    ץꥱץϡʸơ֥ΰȸʤȤ
-    Ǥ롣ʸˤĤƤξϡξʸơ֥
-    򤽤ʸΥɤǰȤ롣  */
+    ³ʸɤʸƱĤȤ¿
+
+    Τ褦ʷѤʸȤղþΨŪ˳Ǽ뤿ˡ
+    m17n 饤֥ @e ʸơ֥ (chartable) ȸƤ֥֥Ȥ
+    Ѥ롣ʸơ֥ǤϤʤץꥱץ
+    ʸơ֥ΰȤưȤǤ롣ʸˤĤƤ
+    ξϡξʸơ֥򤽤ʸΥɤǰ
+    롣  
+
+    ʸơ֥ϴ֥ȤǤ롣 */
 
 /*=*/
 #if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
@@ -631,10 +635,10 @@ mchartable__lookup (MCharTable *table, int c, int *next_c, int default_p)
     The symbol @c Mchar_table has the name <tt>"char-table"</tt>.  */
 
 /***ja
-    @brief "char-table" Ȥ̾ĥܥ
+    @brief "char-table" Ȥ̾ĥܥ.
 
-    ѿ @c Mchar_table ̾ <tt>"char-table"</tt> Ѥߥ
-    ܥǤ롣  */
+    ܥ @c Mchar_table ̾ <tt>"char-table"</tt> ġ
+    */
 
 MSymbol Mchar_table;
 
@@ -651,19 +655,19 @@ MSymbol Mchar_table;
     @return
     If the operation was successful, mchartable () returns a pointer
     to the created chartable.  Otherwise it returns @c NULL and
-    assigns an error code to the external variable @c merror_code.  */
+    assigns an error code to the external variable #merror_code.  */
 
 /***ja
-    @brief ʸơ֥
+    @brief ʸơ֥.
 
     ؿ mchartable () ϥ $KEY ǤΥǥեͤ 
     $DEFAULT_VALUE Ǥ뿷ʸơ֥롣⤷ $KEY 
     ǤСΥơ֥Ǥϡʥǥեͤޤơ˴֥
-    Ȥ NULL Ǥ롣
+    Ȥ NULL Τ줫Ǥ롣
 
     @return
      mchartable () Ϻ줿ʸơ֥ؤΥݥ
-    ֤Ԥ @c NULL ֤ѿ @c merror_code ˥
+    ֤Ԥ @c NULL ֤ѿ #merror_code ˥
     顼ɤꤹ롣  */
 
 MCharTable *
@@ -693,15 +697,15 @@ mchartable (MSymbol key, void *default_value)
     character $C in chartable $TABLE.  If no value has been set for $C
     explicitly, the default value of $TABLE is returned.  If $C is not
     a valid character, mchartable_lookup () returns @c NULL and
-    assigns an error code to the external variable @c merror_code.  */
+    assigns an error code to the external variable #merror_code.  */
 
 /***ja
-    @brief ʸơ֥ʸ˳Ƥ줿֤ͤ
+    @brief ʸơ֥ʸ˳Ƥ줿֤ͤ.
 
     ؿ mchartable_lookup () ʸơ֥ $TABLE ʸ $C ˳
     Ƥ줿֤ͤ$C ФŪͤʤС$TABLE Υǥե
     ֤ͤ$C ʸǤʤСmchartable_lookup ()  
-    @c NULL ֤ѿ @c merror_code ˥顼ɤꤹ롣  */
+    @c NULL ֤ѿ #merror_code ˥顼ɤꤹ롣  */
 
 /***
     @errors
@@ -731,17 +735,17 @@ mchartable_lookup (MCharTable *table, int c)
     @return
     If the operation was successful, mchartable_set () returns 0.
     Otherwise it returns -1 and assigns an error code to the external
-    variable @c merror_code.  */
+    variable #merror_code.  */
 
 /***ja
-    @brief ʸơ֥Ǥʸͤꤹ
+    @brief ʸơ֥Ǥʸͤꤹ.
 
     ؿ mchartable_set () ϡʸơ֥ $TABLE ʸ $C 
      $VAL Ƥ롣
 
     @return
     Сmchartable_set ()  0 ֤Ǥʤ -1 
-    ֤ѿ @c merror_code ˥顼ɤꤹ롣  */
+    ֤ѿ #merror_code ˥顼ɤꤹ롣  */
 
 /***
     @errors
@@ -803,18 +807,18 @@ mchartable_set (MCharTable *table, int c, void *val)
     @return
     If the operation was successful, mchartable_set_range () returns
     0.  Otherwise it returns -1 and assigns an error code to the
-    external variable @c merror_code.  If $FROM is greater than $TO,
+    external variable #merror_code.  If $FROM is greater than $TO,
     mchartable_set_range () returns immediately without an error.  */
 
 /***ja
-    @brief ϰϤʸͤꤹ
+    @brief ϰϤʸͤꤹ.
 
     ؿ mchartable_set_range () ϡʸơ֥ $TABLE  $FROM 
      $TO ޤǡξüޤˤʸˡͤȤ $VAL ꤹ롣
 
     @return
      mchartable_set_range ()  0 ֤Ǥʤ 
-    -1 ֤ѿ @c merror_code ˥顼ɤꤹ롣$FROM  
+    -1 ֤ѿ #merror_code ˥顼ɤꤹ롣$FROM  
     $TO 礭Ȥˤϡ mchartable_set_range () ϲ⤻顼
     ⵯʤ  */
 
@@ -860,10 +864,10 @@ mchartable_set_range (MCharTable *table, int from, int to, void *val)
     to -1.  */
 
 /***ja
-    @brief ͤǥեȤȰۤʤʸõ
+    @brief ͤǥեȤȰۤʤʸõ.
 
     ؿ mchartable_range () ʸơ֥ $TABLE ǡ$TABLE Υǥե
-    ͰʳͤĺǽȺǸʸĴ١줾 $FROM  
+    ͰʳͤĺǽȺǸʸõ줾 $FROM  
     $TO ꤹ롣٤ƤʸǥեͤͤȤƻäƤС
     $FROM  $TO  -1ꤹ롣  */
 
@@ -904,12 +908,12 @@ mchartable_range (MCharTable *table, int *from, int *to)
     This function always returns 0.  */
 
 /***ja
-    @brief ʸơ֥ʸФƻδؿƤ
+    @brief ʸơ֥ʸФƻδؿƤ.
 
     ؿ mchartable_map () ϡʸơ֥ $TABLE ʸФƴ
-     $FUNC Ƥ֡ͤ $IGNORE ǤʸˤĤƤϴؿƤӽ
-    Ԥʤʤ$IGNORE ʸͤӤ @c == ǹԤʤΤǡʸ
-    ƥݥ󥿤Ȥݤˤդפ롣
+     $FUNC Ƥ֡$TABLE Ǥͤ $IGNORE ǤʸˤĤ
+    ƤϴؿƤӽФԤʤʤ$IGNORE ʸͤӤ @c == 
+    ԤʤΤǡʸƥݥ󥿤Ȥݤˤդפ롣
 
     mchartable_map () ϡʸȤ $FUNC Ƥ֤ΤǤϤʤؿ
     ӽФβŬ褦Ȥ롣ʤϢ³ʸƱͤ
@@ -950,6 +954,15 @@ mchartable_map (MCharTable *table, void *ignore,
     @return
     This function returns $TABLE.  */
 
+/***ja
+    @brief ʸơ֥פ.
+
+    ؿ mdebug_dump_chartab () ʸơ֥ $TABLE  stderr ˿ʹ
+    ˲ɤʷǰ롣$INDENT ϣܰʹߤΥǥȤꤹ롣
+
+    @return
+    δؿ $TABLE ֤  */
+
 MCharTable *
 mdebug_dump_chartab (MCharTable *table, int indent)
 {
