@@ -22,39 +22,38 @@
 
 /***en
     @addtogroup m17nDatabase
-    @brief The m17n database and API for it
+    @brief The m17n database and API for it.
 
     The m17n library dynamically acquires various kinds of information
     in need from data in the <i> m17n database</i>.  Application
     programs can also add/load their original data to/from the m17n
     database.  The m17n database contains multiple heterogeneous data,
-    and each data is identified by four tags; TAG1, TAG2, TAG3, TAG4.
+    and each data is identified by four tags; TAG0, TAG1, TAG2, TAG3.
     Each tag must be a symbol.
 
-    TAG1 specifies the type of data stored in the database as below.
+    TAG0 specifies the type of data stored in the database as below.
 
-    <ul>
-
-    <li> If TAG1 is #Mchar_table, the data is of the @e chartable @e
+    @li
+    If TAG0 is #Mchar_table, the data is of the @e chartable @e
     type and provides information about each character.  In this case,
-    TAG2 specifies the type of the information and must be #Msymbol,
-    #Minteger, #Mstring, #Mtext, or #Mplist.  TAG3 and TAG4 can be any
+    TAG1 specifies the type of the information and must be #Msymbol,
+    #Minteger, #Mstring, #Mtext, or #Mplist.  TAG2 and TAG3 can be any
     symbols.
 
-    <li> If TAG1 is #Mcharset, the data is of the @e charset @e type
+    @li
+    If TAG0 is #Mcharset, the data is of the @e charset @e type
     and provides a decode/encode mapping table for a charset.  In this
-    case, TAG2 must be a symbol representing a charset.  TAG3 and TAG4
+    case, TAG1 must be a symbol representing a charset.  TAG2 and TAG3
     can be any symbols.
 
-    <li> If TAG1 is neither #Mchar_table nor #Mcharset, the data is of
-    the @e plist @e type.  See the documentation of the mdatabase_load
-    () function for the details.  In this case, TAG2, TAG3, and TAG4
-    can be any symbols.
+    @li 
+    If TAG0 is neither #Mchar_table nor #Mcharset, the data is of
+    the @e plist @e type.  See the documentation of the 
+    mdatabase_load () function for the details.  
+    In this case, TAG1, TAG2, and TAG3 can be any symbols.
 
-    </ul>
-
-    Below, the notation \<TAG1, TAG2, TAG3, TAG4\> means a data with
-    those tags.
+    The notation \<TAG0, TAG1, TAG2, TAG3\> means a data with those
+    tags.
 
     Application programs first calls the mdatabase_find () function to
     get a pointer to an object of the type #MDatabase.  That object
@@ -66,35 +65,34 @@
 
 /***ja
     @addtogroup m17nDatabase
-    @brief ١ˤȤ˴ؤ API
+    @brief m17n ǡ١ˤȤ˴ؤ API.
 
-    m17n 饤֥ɬפ˱ưŪ m17n ١ǡ
-    롣ޤץꥱץȼΥǡ m17n 
-    ١ɲäưŪ˼뤳ȤǤ롣m17n 
-    ١ʣΥǡ١ʤꡢƥǡ١ϣĤΥ
-     TAG1, TAG2, TAG3, TAG4ʤ٤ƥܥˤˤäƼ̤롣
+    m17n 饤֥ɬפ˱ưŪ @e m17n @e ǡ١ 
+    롣ޤץꥱץȼΥǡ 
+    m17n ǡ١ɲäưŪ˼뤳ȤǤ롣m17n 
+    ǡ١ˤʣ¿ͤʥǡޤޤƤꡢƥǡϣĤ
+     TAG0, TAG1, TAG2, TAG3ʤ٤ƥܥˤˤäƼ̤롣
 
-    TAG1 ϥǡ١ΥǡΥפ򼨤
+    TAG0 ϥǡ١ΥǡΥפʲΤ褦˻ꤹ롣
 
-    TAG1  #Mchar_table Υǡ١ @e chartable ȸƤФ졢
-    ʸ˴ؤ󶡤롣ξ TAG2 Ͼμꤹ
-    ܥǤꡢʲΤ줫ǤʤФʤʤ
+    @li 
+    TAG0  #Mchar_table Ǥǡ @e chartable ȸƤФ졢
+    ʸ˴ؤ󶡤롣ξ TAG1 Ͼμꤹ
+    ܥǤꡢ#Msymbol, #Minteger, #Mstring, #Mtext, #Mplist 
+    줫Ǥ롣TAG2  TAG3 ǤդΥܥǤ褤
 
-    @li #Msymbol @li #Minteger @li #Mstring @li #Mtext @li #Mplist
-
-    TAG3  TAG4 ǤդΥܥǤ褤
-
-    TAG1  #Mcharset Υǡ١ @e charset ȸƤФ졢ʸ
-    åѤΥǥɡ󥳡ɥޥåפ󶡤롣ξ TAG2 
-    ʸåȤΥܥǤʤФʤʤTAG3  TAG4 ǤդΥ
+    @li 
+    TAG0  #Mcharset Ǥǡ @e charset ȸƤФ졢ʸ
+    åѤΥǥɡ󥳡ɥޥåפ󶡤롣ξ TAG1 
+    ʸåȤΥܥǤʤФʤʤTAG2  TAG3 ǤդΥ
     ܥǤ褤
 
-    TAG1  #Mchar_table Ǥ #Mcharset Ǥʤ硢Υǡ١
-     @e plist  ȸƤФ롣ܺ٤˴ؤƤϴؿ mdatabase_load
-    () 򻲾ȤΤȡξ TAG2TAG3TAG4 ǤդΥܥ
-    Ǥ褤
+    @li
+    TAG0  #Mchar_table Ǥ #Mcharset Ǥʤ硢Υǡ @e
+    plist  Ǥ롣ܺ٤˴ؤƤϴؿ mdatabase_load () 
+    ȤΤȡξ TAG1TAG2TAG3 ǤդΥܥǤ褤
 
-    ʲΥĥǡ١ \<TAG1, TAG2, TAG3, TAG4\> Ȥ
+    Υĥǡ١ \<TAG0, TAG1, TAG2, TAG3\> Ȥ
     ɽ魯
 
     ץꥱץϡޤؿ mdatabase_find () Ȥäƥǡ
@@ -608,6 +606,17 @@ mdatabase__fini (void)
     definitions in the format described in @ref mdbDir "mdbDir(5)".
 
     The default value is NULL.  */
+/***ja
+    @brief ץꥱͭΥǡѥǥ쥯ȥ.
+
+    ץꥱץबΥץͭΥǡ m17n ǡ
+    ١񤭤ǡ󶡤ˤϡޥ M17N_INIT () 
+    Ƥˤѿǡեޤǥ쥯ȥ̾˥åȤ
+    ƤϤʤʤǥ쥯ȥˤ "mdb.dir" ե򤪤Ȥ
+    롣"mdb.dir"եˤϡ @ref mdbDir "mdbDir(5)" 
+    ƤեޥåȤǥǡΥꥹȤ򵭽Ҥ롣
+
+    ǥեȤͤ NULL Ǥ롣  */
 
 char *mdatabase_dir;
 
@@ -616,16 +625,15 @@ char *mdatabase_dir;
     @brief Look for a data in the database.
 
     The mdatabase_find () function searches the m17n database for a
-    data who has tags $TAG1 through $TAG4, and returns a pointer to
-    the data.  If such a database is not found, it returns @c
-    NULL.  */
+    data who has tags $TAG0 through $TAG3, and returns a pointer to
+    the data.  If such a data is not found, it returns @c NULL.  */
 
 /***ja
-    @brief ǡ١õ
+    @brief ǡ١Υǡõ.
 
-    ؿ mdatabase_find () ϡ m17n ١ $TAG1  
-    $TAG4 ޤǤΥĥǡ١õؤΥݥ󥿤֤
-    Τ褦ʥǡ١ʤ @c NULL ֤
+    ؿ mdatabase_find () ϡ m17n ١ $TAG0  
+    $TAG3 ޤǤΥĥǡõؤΥݥ󥿤֤Τ
+    ʥǡʤ @c NULL ֤
 
     @latexonly \IPAlabel{mdatabase_find} @endlatexonly  */
 
@@ -652,10 +660,19 @@ mdatabase_find (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
     @brief Return a data list of the m17n database.
 
     The mdatabase_list () function searches the m17n database for data
-    who have tags $TAG1 through $TAG4, and returns their list by a
+    who have tags $TAG0 through $TAG3, and returns their list by a
     plist.  The value #Mnil in $TAGn means a wild card that matches
     any tag.  Each element of the plist has key #Mt and value a
     pointer to type #MDatabase.  */
+/***ja
+    @brief m17n ǡ١ΥǡꥹȤ֤.
+
+    ؿ mdatabase_list ()  m17n ǡ١椫 $TAG0 $TAG3 
+    ޤǤΥĥǡõΥꥹȤplist Ȥ֤ $TAGn 
+     #Mnil ǤäˤϡǤդΥ˥ޥå磻ɥɤ
+    Ƽ갷롣֤ plist γǤϥ Ȥ #Mt 
+    Ȥ #MDatabase ؤΥݥ󥿤ġ  */
+
 
 MPlist *
 mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
@@ -687,12 +704,12 @@ mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
     @brief Define a data of the m17n database.
 
     The mdatabase_define () function defines a data that has tags
-    $TAG1 through $TAG4 and additional information $EXTRA_INFO.
+    $TAG0 through $TAG3 and additional information $EXTRA_INFO.
 
     $LOADER is a pointer to a function that loads the data from the
     database.  This function is called from the mdatabase_load ()
     function with the two arguments $TAGS and $EXTRA_INFO.  Here,
-    $TAGS is the array of $TAG1 through $TAG4.
+    $TAGS is the array of $TAG0 through $TAG3.
 
     If $LOADER is @c NULL, the default loader of the m17n library is
     used.  In this case, $EXTRA_INFO must be a string specifying a
@@ -704,23 +721,24 @@ mdatabase_list (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3)
     mdatabase_load ().  Otherwise, it returns @c NULL.  */
 
 /***ja
-    @brief ǡ١
+    @brief m17n ǡ١Υǡ.
 
-    ؿ mdatabase_define ()  $TAG1  $TAG4 ޤǤΥղ
-     $EXTRA_INFO ĥǡ١롣
+    ؿ mdatabase_define ()  $TAG0  $TAG3 ޤǤΥղ
+     $EXTRA_INFO ĥǡ롣
 
-    $LOADER ϤΥǡ١ΥɤѤؿؤΥݥ󥿤Ǥ
-    롣δؿ mdatabase_load ()  $MDB  $EXTRA_INFO Ȥ2 
-    ĤΰդǸƤӽФ롣 $MDB  mdatabase_load () 
-    줿Ǥ롣
+    $LOADER ϤΥǡΥɤѤؿؤΥݥ󥿤Ǥ롣
+    δؿ mdatabase_load ()  $TAGS  $EXTRA_INFO Ȥ2 Ĥ
+    դǸƤӽФ롣 $TAGS  $TAG0  $TAG3 ޤǤ
+    Ǥ롣
 
     ⤷ $LOADER  @c NULL ʤ顢m17n 饤֥ɸΥȤ
-    롣Υϥǡ١ $EXTRA_INFO ˻ꤵ줿̾Υե
-    뤫ɤ롣
+    롣ξˤ $EXTRA_INFO ϥǡޤե̾ǤʤƤϤ
+    ʤ
 
     @return
-     mdatabase_define () 줿ǡ١ؤ
-    ݥ󥿤֤Ǥʤ @c NULL ֤
+      mdatabase_define () 줿ǡ١
+    ؤΥݥ󥿤֤Υݥ󥿤ϴؿ mdatabase_load () ΰ
+    ȤѤ뤳ȤǤ롣Ǥʤ @c NULL ֤
 
     @latexonly \IPAlabel{mdatabase_define} @endlatexonly  */
 
@@ -765,17 +783,13 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
 
     If the database is of the @e chartable type, it returns a
     chartable.  The default value of the chartable is set according to
-    the second tag of the database as below:
+    the second tag of the data as below:
 
-    <ul>
+    @li If the tag is #Msymbol, the default value is #Mnil.
+    @li If the tag is #Minteger, the default value is -1.
+    @li Otherwise, the default value is @c NULL.
 
-    <li> If the tag is #Msymbol, the default value is #Mnil.
-    <li> If the tag is #Minteger, the default value is -1.
-    <li> Otherwise, the default value is @c NULL.
-
-    </ul>
-
-    If the data is of the charset type, it returns a plist of length 2
+    If the data is of the @e charset type, it returns a plist of length 2
     (keys are both #Mt).  The value of the first element is an array
     of integers that maps code points to the corresponding character
     codes.  The value of the second element is a chartable of integers
@@ -784,24 +798,24 @@ mdatabase_define (MSymbol tag0, MSymbol tag1, MSymbol tag2, MSymbol tag3,
 
 
 /***ja
-    @brief ǡ١ɤ
+    @brief ǡ١ǡɤ.
 
-    ؿ mdatabase_load ()  $MDB ؤǡ١ɤ
-    Ȥ֤֤Τϥǡ١Υפˤäưۤʤ롣
+    ؿ mdatabase_load ()  $MDB ؤǡɤ
+    Ȥ֤֤ΤϥǡΥפˤäưۤʤ롣
 
-    ǡ١ plist פʤС @e plist ֤
+    ǡ @e plist פʤС @e plist ؤΥݥ󥿤֤
 
-    ǡ١ chartable פʤʸơ֥֤ʸ
-    ơ֥Υǥեͤϡ2ˤäưʲΤ褦˷ޤ롣
+    ǡ @e chartable פʤʸơ֥֤ʸơ֥
+    Υǥեͤϡǡ2ˤäưʲΤ褦˷ޤ롣
 
     @li  #Msymbol ʤ顢ǥեͤ #Mnil
     @li  #Minteger ʤ顢ǥեͤ -1
     @li ʳʤ顢ǥեͤ @c NULL
 
-    ǡ١ charset פʤĹ 2  @c plist ֤ʥ
-    ϶ #Mt ˡǽǤͤϥɥݥȤʸɤ˥ޥ
-    פǤ롣ܤǤͤϤεդΥޥåפ򤹤ʸ
-    ơ֥Ǥ롣ʸåȤͽƤʤФʤʤ
+    ǡ @e charset פʤĹ 2  plist ֤ʥ϶ 
+    #Mt ˡǽǤͤϥɥݥȤбʸɤ˥ޥå
+    Ǥ롣ܤǤͤϵդΥޥåפ򤹤ʸơ
+    Ǥ롣ʸåȤͽƤʤФʤʤ
 
     @latexonly \IPAlabel{mdatabase_load} @endlatexonly
   */
@@ -830,10 +844,10 @@ mdatabase_load (MDatabase *mdb)
     four.  */
 
 /***ja
-    @brief ǡ١Υ
+    @brief ǡΥ.
 
-    ؿ mdatabase_tag () ϡǡ١ $MDB Υʥܥˤ
-    ֤Ĺ 4 Ǥ롣
+    ؿ mdatabase_tag () ϡǡ $MDB Υʥܥˤ
+    Ĺ 4 Ǥ롣
 
     @latexonly \IPAlabel{mdatabase_tag} @endlatexonly  */
 
