@@ -28,21 +28,29 @@
     M-text, a fontset provides rules to select a font for each
     character in the M-text according to the following information.
 
-    - The script character property of a character.
-    - The language text property of a character.
-    - The charset text property of a character.
+    @li The script character property of a character.
+    @li The language text property of a character.
+    @li The charset text property of a character.
 
     The documentation of mdraw_text () describes how that information is
     used.  */
 
-/***ja
-    @addtogroup m17nFontset
-    @brief եȥåȤϰΥͭեȤνǤ
+/***ja @addtogroup m17nFontset 
 
-    @e եȥå  @c MFontset Υ֥ȤǤꡢ¿ʸ
-    ɽκݡλեȤ򽸤ưӤƼ갷˻
-    롣եȥåȤϤޤ졢ʸåȡʸͿ줿
-    ŬڤʥեȤ򤹤뤿ξݻƤ롣  */
+    @brief եȥåȤʸեȤؤбդԤ֥ȤǤ.
+
+    @e եȥå  @c MFontset Υ֥ȤǤ롣M-text 
+    ɽݡեȥåȤ M-text θġʸФƤɤΥե
+    ȤѤ뤫ε§򡢰ʲξ˽äͿ롣
+
+    @li ʸʸץѥƥ "ץ"
+    @li ʸΥƥȥץѥƥ ""
+    @li ʸΥƥȥץѥƥ "ʸå"
+
+    ξ󤬤ɤΤ褦Ѥ뤫 mdraw_text () 򻲾
+    Τȡ
+
+    */
 
 /*=*/
 
@@ -55,6 +63,7 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "config.h"
 #include "m17n-gui.h"
 #include "m17n-misc.h"
 #include "internal.h"
@@ -325,6 +334,96 @@ realize_font_group (MFrame *frame, MFont *request, MPlist *font_group,
     }
 }
 
+static void
+realize_fontset_elements (MFrame *frame, MRealizedFontset *realized,
+			  MFontset *fontset, MFont *request)
+{
+  MPlist *per_script, *per_lang, *per_charset, *font_group;
+  MPlist *plist, *pl;
+
+  realized->fontset = fontset;
+  realized->tick = fontset->tick;
+  realized->spec = *request;
+  realized->frame = frame;
+  realized->per_script = per_script = mplist ();
+  MPLIST_DO (plist, fontset->per_script)
+    {
+      per_lang = mplist ();
+      per_script = mplist_add (per_script, MPLIST_KEY (plist), per_lang);
+      MPLIST_DO (pl, MPLIST_PLIST (plist))
+	{
+	  font_group = mplist ();
+	  mplist_add (font_group, Mplist, MPLIST_VAL (pl));
+	  per_lang = mplist_add (per_lang, MPLIST_KEY (pl), font_group);
+	}
+    }
+
+  realized->per_charset = per_charset = mplist ();
+  MPLIST_DO (plist, fontset->per_charset)
+    {
+      font_group = mplist ();
+      mplist_add (font_group, Mplist, MPLIST_VAL (plist));
+      per_charset = mplist_add (per_charset, MPLIST_KEY (plist), font_group);
+    }
+
+  realized->fallback = mplist ();
+  mplist_add (realized->fallback, Mplist, fontset->fallback);
+
+}
+
+static void
+free_realized_fontset_elements (MRealizedFontset *realized)
+{
+  MPlist *plist, *pl, *p;
+  MRealizedFont *rfont;
+
+  if (realized->per_script)
+    {
+      MPLIST_DO (plist, realized->per_script)
+	{
+	  MPLIST_DO (pl, MPLIST_PLIST (plist))
+	    {
+	      MPLIST_DO (p, MPLIST_PLIST (pl))
+		if ((rfont = MPLIST_VAL (p)) && ! rfont->frame)
+		  free (rfont);
+	      p = MPLIST_PLIST (pl);
+	      M17N_OBJECT_UNREF (p);
+	    }
+	  pl = MPLIST_PLIST (plist);
+	  M17N_OBJECT_UNREF (pl);
+	}
+      M17N_OBJECT_UNREF (realized->per_script);
+    }
+  if (realized->per_charset)
+    {
+      MPLIST_DO (plist, realized->per_charset)
+	{
+	  MPLIST_DO (pl, MPLIST_PLIST (plist))
+	    if ((rfont = MPLIST_VAL (pl)) && ! rfont->frame)
+	      free (rfont);
+	  pl = MPLIST_PLIST (plist);
+	  M17N_OBJECT_UNREF (pl);
+	}
+      M17N_OBJECT_UNREF (realized->per_charset);
+    }
+  if (realized->fallback)
+    {
+      MPLIST_DO (plist, realized->fallback)
+	if ((rfont = MPLIST_VAL (plist)) && ! rfont->frame)
+	  free (rfont);
+      M17N_OBJECT_UNREF (realized->fallback);
+    }
+}
+
+static void
+update_fontset_elements (MRealizedFontset *realized)
+{
+  free_realized_fontset_elements (realized);
+  realize_fontset_elements (realized->frame, realized, realized->fontset,
+			    &realized->spec);
+}
+
+
 
 
 /* Internal API */
@@ -367,8 +466,7 @@ mfont__realize_fontset (MFrame *frame, MFontset *fontset, MFace *face)
 {
   MRealizedFontset *realized;
   MFont request;
-  MPlist *per_script, *per_lang, *per_charset, *font_group;
-  MPlist *plist, *pl, *p;
+  MPlist *plist;
 
   if (fontset->mdb)
     load_fontset_contents (fontset);
@@ -379,43 +477,16 @@ mfont__realize_fontset (MFrame *frame, MFontset *fontset, MFace *face)
       mdebug_hook ();
       request.property[MFONT_SIZE] = 120;
     }
-  MPLIST_DO (p, frame->realized_fontset_list)
+  MPLIST_DO (plist, frame->realized_fontset_list)
     {
-      realized = (MRealizedFontset *) MPLIST_VAL (p);
-      if (fontset->name == MPLIST_KEY (p)
+      realized = (MRealizedFontset *) MPLIST_VAL (plist);
+      if (fontset->name == MPLIST_KEY (plist)
 	  && ! memcmp (&request, &realized->spec, sizeof (request)))
 	return realized;
     }
 
   MSTRUCT_MALLOC (realized, MERROR_FONTSET);
-  realized->fontset = fontset;
-  realized->tick = fontset->tick;
-  realized->spec = request;
-  realized->frame = frame;
-  realized->per_script = per_script = mplist ();
-  MPLIST_DO (plist, fontset->per_script)
-    {
-      per_lang = mplist ();
-      per_script = mplist_add (per_script, MPLIST_KEY (plist), per_lang);
-      MPLIST_DO (pl, MPLIST_PLIST (plist))
-	{
-	  font_group = mplist ();
-	  mplist_add (font_group, Mplist, MPLIST_VAL (pl));
-	  per_lang = mplist_add (per_lang, MPLIST_KEY (pl), font_group);
-	}
-    }
-
-  realized->per_charset = per_charset = mplist ();
-  MPLIST_DO (plist, fontset->per_charset)
-    {
-      font_group = mplist ();
-      mplist_add (font_group, Mplist, MPLIST_VAL (plist));
-      per_charset = mplist_add (per_charset, MPLIST_KEY (plist), font_group);
-    }
-
-  realized->fallback = mplist ();
-  mplist_add (realized->fallback, Mplist, fontset->fallback);
-
+  realize_fontset_elements (frame, realized, fontset, &request);
   mplist_add (frame->realized_fontset_list, fontset->name, realized);
   return realized;
 }
@@ -424,155 +495,163 @@ mfont__realize_fontset (MFrame *frame, MFontset *fontset, MFace *face)
 void
 mfont__free_realized_fontset (MRealizedFontset *realized)
 {
-  MPlist *plist, *pl, *p;
-  MRealizedFont *rfont;
-
-  if (realized->per_script)
-    {
-      MPLIST_DO (plist, realized->per_script)
-	{
-	  MPLIST_DO (pl, MPLIST_PLIST (plist))
-	    {
-	      MPLIST_DO (p, MPLIST_PLIST (pl))
-		if ((rfont = MPLIST_VAL (p)) && ! rfont->frame)
-		  free (rfont);
-	      p = MPLIST_PLIST (pl);
-	      M17N_OBJECT_UNREF (p);
-	    }
-	  pl = MPLIST_PLIST (plist);
-	  M17N_OBJECT_UNREF (pl);
-	}
-      M17N_OBJECT_UNREF (realized->per_script);
-    }
-  if (realized->per_charset)
-    {
-      MPLIST_DO (plist, realized->per_charset)
-	{
-	  MPLIST_DO (pl, MPLIST_PLIST (plist))
-	    if ((rfont = MPLIST_VAL (pl)) && ! rfont->frame)
-	      free (rfont);
-	  pl = MPLIST_PLIST (plist);
-	  M17N_OBJECT_UNREF (pl);
-	}
-      M17N_OBJECT_UNREF (realized->per_charset);
-    }
-  if (realized->fallback)
-    {
-      MPLIST_DO (plist, realized->fallback)
-	if ((rfont = MPLIST_VAL (plist)) && ! rfont->frame)
-	  free (rfont);
-      M17N_OBJECT_UNREF (realized->fallback);
-    }
-
+  free_realized_fontset_elements (realized);
   free (realized);
 }
 
 
+static MRealizedFont *
+try_font_group (MRealizedFontset *realized,
+		MPlist *font_group, MGlyph *g, int *num, int size)
+{
+  MFrame *frame = realized->frame;
+  MRealizedFont *rfont;
+  MPlist *plist;
+  int i;
+
+  if (MPLIST_PLIST_P (font_group))
+    realize_font_group (frame, &realized->spec, font_group, size);
+
+  MPLIST_DO (plist, font_group)
+    {
+      rfont = (MRealizedFont *) MPLIST_VAL (plist);
+      if (rfont->status < 0)
+	continue;
+      /* Check if this font can display all glyphs.  */
+      for (i = 0; i < *num; i++)
+	{
+	  g[i].code = mfont__encode_char (rfont,
+					  g[i].type == GLYPH_CHAR ? g[i].c
+					  : ' ');
+	  if (g[i].code == MCHAR_INVALID_CODE)
+	    break;
+	}
+      if (i == *num)
+	{
+	  if (rfont->status > 0
+	      || mfont__open (rfont) == 0)
+	    /* We found a font that can display all glyphs.  */
+	    return rfont;
+	}
+    }
+
+  /* We couldn't find a font that can display all glyphs.  Find one
+     that can display at least the first glyph.  */
+  MPLIST_DO (plist, font_group)
+    {
+      rfont = (MRealizedFont *) MPLIST_VAL (plist);
+      if (rfont->status < 0)
+	continue;
+      g->code = mfont__encode_char (rfont,
+				    g->type == GLYPH_CHAR ? g->c : ' ');
+      if (g->code != MCHAR_INVALID_CODE)
+	{
+	  if (rfont->status > 0
+	      || mfont__open (rfont) == 0)
+	    {
+	      /* Ok, let's use this font.  Check how many more
+		 characters it supports.  */
+	      int i;
+
+	      for (i = 1; i < *num; i++)
+		{
+		  g[i].code = mfont__encode_char (rfont,
+						  g[i].type == GLYPH_CHAR
+						  ? g[i].c : ' ');
+		  if (g[i].code == MCHAR_INVALID_CODE)
+		    break;
+		}
+	      *num = i;
+	      return rfont;
+	    }
+	}
+    }
+
+  return NULL;
+}
+
 MRealizedFont *
 mfont__lookup_fontset (MRealizedFontset *realized, MGlyph *g, int *num,
 		       MSymbol script, MSymbol language, MSymbol charset,
 		       int size)
 {
-  MFrame *frame = realized->frame;
   MCharset *preferred_charset = (charset == Mnil ? NULL : MCHARSET (charset));
-  MPlist *per_charset, *per_script, *per_lang, *font_group;
-  MPlist *font_groups[256], *plist;
-  int n_font_group = 0;
-  MRealizedFont *first = NULL, *rfont;
-  int first_len;
-  int i;
+  MPlist *per_charset, *per_script, *per_lang;
+  MPlist *plist;
+  MRealizedFont *rfont = NULL;
+
+  if (realized->tick != realized->fontset->tick)
+    update_fontset_elements (realized);
 
   if (preferred_charset
-      && (per_charset = mplist_get (realized->per_charset, charset)) != NULL)
-    font_groups[n_font_group++] = per_charset;
+      && (per_charset = mplist_get (realized->per_charset, charset)) != NULL
+      && (rfont = try_font_group (realized, per_charset, g, num, size)))
+    return rfont;
+
   if (script != Mnil
-      && ((per_script = mplist_find_by_key (realized->per_script, script))
-	  != NULL))
+      && (per_script = mplist_get (realized->per_script, script)))
     {
-      /* The first loop is for matching language (if any), and the second
-	 loop is for non-matching languages.  */
+      /* We prefer font groups in this order:
+	  (1) group matching with LANGUAGE if LANGUAGE is not Mnil
+	  (2) group for generic language
+	  (3) group not matching with LANGUAGE  */
       if (language == Mnil)
 	language = Mt;
-      for (i = 0; i < 2; i++)
+      if ((per_lang = mplist_get (per_script, language))
+	  && (rfont = try_font_group (realized, per_lang, g, num, size)))
+	return rfont;
+
+      if (language == Mt)
 	{
-	  MPLIST_DO (per_lang, MPLIST_PLIST (per_script))
-	    if ((MPLIST_KEY (per_lang) == language) != i)
-	      font_groups[n_font_group++] = MPLIST_PLIST (per_lang);
+	  /* Try the above (3) */
+	  MPLIST_DO (plist, per_script)
+	    if (MPLIST_KEY (plist) != language
+		&& (rfont = try_font_group (realized, MPLIST_PLIST (plist),
+					    g, num, size)))
+	      return rfont;
 	}
-    }
-  font_groups[n_font_group++] = realized->fallback;
-
-  if (n_font_group == 1)
-    {
-      /* As we only have a fallback font group, try all the other
-	 fonts too.  */
-      MPLIST_DO (per_script, realized->per_script)
-	MPLIST_DO (per_lang, MPLIST_PLIST (per_script))
-	  font_groups[n_font_group++] = MPLIST_PLIST (per_lang);
-      MPLIST_DO (per_charset, realized->per_charset)
-	font_groups[n_font_group++] = MPLIST_PLIST (per_charset);
-    }
-
-  for (i = 0; i < n_font_group; i++)
-    {
-      int j;
-      
-      if (MPLIST_PLIST_P (font_groups[i]))
-	realize_font_group (frame, &realized->spec, font_groups[i], size);
-
-      MPLIST_DO (plist, font_groups[i])
-        {
-	  rfont = (MRealizedFont *) MPLIST_VAL (plist);
-	  g->code = mfont__encode_char (rfont, g->c);
-	  if (g->code != MCHAR_INVALID_CODE)
-	    break;
-	}
-      if (MPLIST_TAIL_P (plist))
-	continue;
-      for (j = 1; j < *num; j++)
+      else
 	{
-	  g[j].code = mfont__encode_char (rfont, g[j].c);
-	  if (g[j].code == MCHAR_INVALID_CODE)
-	    break;
-	}
-      if (! first)
-	first = rfont, first_len = j;
-      if (j == *num)
-	/* We found a font that can display all requested
-	   characters.  */
-	break;
+	  /* At first try the above (2) */
+	  if ((per_lang = mplist_get (per_script, Mt))
+	      && (rfont = try_font_group (realized, per_lang, g, num, size)))
+	    return rfont;
 
-      MPLIST_DO (plist, MPLIST_NEXT (plist))
-	{
-	  rfont = (MRealizedFont *) MPLIST_VAL (plist);
-	  for (j = 0; j < *num; j++)
-	    {
-	      g[j].code = mfont__encode_char (rfont, g[j].c);
-	      if (g[j].code == MCHAR_INVALID_CODE)
-		break;
-	    }
-	  if (j == *num)
-	    break;
+	  /* Then try the above (3) */
+	  MPLIST_DO (plist, per_script)
+	    if (MPLIST_KEY (plist) != language
+		&& MPLIST_KEY (plist) != Mt
+		&& (rfont = try_font_group (realized, MPLIST_PLIST (plist),
+					    g, num, size)))
+	      return rfont;
 	}
-      if (! MPLIST_TAIL_P (plist))
-	break;
     }
 
-  if (i == n_font_group) 
+  if (language != Mnil)
+    /* Find a font group for this language from all scripts.  */
+    MPLIST_DO (plist, realized->per_script)
+      if ((per_lang = mplist_get (MPLIST_PLIST (plist), language))
+	  && (rfont = try_font_group (realized, per_lang, g, num, size)))
+	return rfont;
+
+  /* Try fallback fonts.  */
+  if ((rfont = try_font_group (realized, realized->fallback, g, num, size)))
+    return rfont;
+
+  /* At last try all fonts.  */
+  MPLIST_DO (per_script, realized->per_script)
     {
-      if (! first)
-	return NULL;
-      rfont = first, *num = first_len;
-      for (i = 0; i < *num; i++)
-	g[i].code = mfont__encode_char (rfont, g[i].c);
+      MPLIST_DO (per_lang, MPLIST_PLIST (per_script))
+	if ((rfont = try_font_group (realized, MPLIST_PLIST (per_lang),
+				     g, num, size)))
+	  return rfont;
     }
-  if (! rfont->status
-      && mfont__open (rfont) < 0)
-    {
-      MPLIST_VAL (font_group) = NULL;
-      return NULL;
-    }
-  return rfont;
+  MPLIST_DO (per_charset, realized->per_charset)
+    if ((rfont = try_font_group (realized, MPLIST_PLIST (per_charset),
+				 g, num, size)))
+      return rfont;
+
+  return NULL;
 }
 
 /*** @} */
@@ -599,12 +678,31 @@ mfont__lookup_fontset (MRealizedFontset *realized, MGlyph *g, int *num,
     vacant.
 
     The macro M17N_INIT () creates the default fontset.  An
-    application program can modify it before the first call of mframe
-    ().
+    application program can modify it before the first call of 
+    mframe ().
 
     @return
     This function returns a pointer to the found or newly created
     fontset.  */
+/***ja 
+    @brief եȥåȤ֤.
+
+    ؿ mfontset () ̾ $NAME ĥեȥåȥ֥Ȥؤ
+    ݥ󥿤֤ $NAME  @c NULL ʤСǥեȥեȥå
+    ؤΥݥ󥿤֤
+
+    $NAME Ȥ̾ĥեȥåȤʤСΤ
+    롣κݡm17n ǡ١ \<@c fontset, $NAME\> Ȥǡ
+    СեȥåȤϤΥǡ˱äƽ롣ʤС
+    Τޤޤˤ롣
+
+    ޥ M17N_INIT () ϥǥեȤΥեȥåȤ롣ץꥱ
+    ץ mframe () ƸƤ֤ޤǤϥǥեȥե
+    åȤѹ뤳ȤǤ롣
+
+    @return
+    δؿϸĤä뤤ϺäեȥåȤؤΥݥ󥿤֤
+     */
 
 MFontset *
 mfontset (char *name)
@@ -613,21 +711,26 @@ mfontset (char *name)
   MFontset *fontset;
 
   if (! name)
-    return default_fontset;
-  sym = msymbol (name);
-  fontset = mplist_get (fontset_list, sym);
-  if (fontset)
-    return fontset;
-  M17N_OBJECT (fontset, free_fontset, MERROR_FONTSET);
-  fontset->name = sym;
-  fontset->mdb = mdatabase_find (Mfontset, sym, Mnil, Mnil);
-  if (! fontset->mdb)
+    fontset = default_fontset;
+  else
     {
-      fontset->per_script = mplist ();
-      fontset->per_charset = mplist ();
-      fontset->fallback = mplist ();
+      sym = msymbol (name);
+      fontset = mplist_get (fontset_list, sym);
+      if (! fontset)
+	{
+	  M17N_OBJECT (fontset, free_fontset, MERROR_FONTSET);
+	  fontset->name = sym;
+	  fontset->mdb = mdatabase_find (Mfontset, sym, Mnil, Mnil);
+	  if (! fontset->mdb)
+	    {
+	      fontset->per_script = mplist ();
+	      fontset->per_charset = mplist ();
+	      fontset->fallback = mplist ();
+	      fontset->font_spec_list = mplist ();
+	    }
+	  mplist_put (fontset_list, sym, fontset);
+	}
     }
-  mplist_put (fontset_list, sym, fontset);
   M17N_OBJECT_REF (fontset);
   return fontset;
 }
@@ -635,9 +738,13 @@ mfontset (char *name)
 /*=*/
 
 /***en
-    @brief Return the name of a fontset
+    @brief Return the name of a fontset.
 
-    The mfontset_name () function returns the name of $FONTSET.  */
+    The mfontset_name () function returns the name of fontset $FONTSET.  */
+/***ja
+    @brief եȥåȤ֤̾.
+
+    ؿ mfontset_name () ϥեȥå $FONTSET ֤̾  */
 MSymbol
 mfontset_name (MFontset *fontset)
 {
@@ -647,12 +754,19 @@ mfontset_name (MFontset *fontset)
 /*=*/
 
 /***en
-    @brief Make a copy a fontset
+    @brief Make a copy of a fontset.
 
-    The mfontset_copy () function makes a copy of $FONTSET, gives it a
+    The mfontset_copy () function makes a copy of fontset $FONTSET, gives it a
     name $NAME, and returns a pointer to the created copy.  $NAME must
-    not be a name of existing fontset.  Otherwise, this function
+    not be a name of existing fontset.  In such case, this function
     returns NULL without making a copy.  */
+/***ja
+    @brief եȥåȤΥԡ.
+
+    ؿ mfontset_copy () ϥեȥå $FONTSET Υԡäơ
+    ̾ $NAME ͿΥԡؤΥݥ󥿤֤$NAME ϴ¸
+    եȥåȤ̾ǤäƤϤʤʤξˤϥԡ餺
+    NULL ֤  */
 
 MFontset *
 mfontset_copy (MFontset *fontset, char *name)
@@ -666,6 +780,9 @@ mfontset_copy (MFontset *fontset, char *name)
   M17N_OBJECT (copy, free_fontset, MERROR_FONTSET);
   copy->name = sym;
 
+  if (fontset->mdb)
+    load_fontset_contents (fontset);
+
   if (fontset->per_script)
     {
       copy->per_script = mplist ();
@@ -701,8 +818,9 @@ mfontset_copy (MFontset *fontset, char *name)
 /***en
     @brief Modify the contents of a fontset.
 
-    The mfontset_modify_entry () function associates, in $FONTSET, a
-    copy of $FONT with the $SCRIPT / $LANGUAGE pair or with $CHARSET.
+    The mfontset_modify_entry () function associates, in fontset
+    $FONTSET, a copy of $FONT with the $SCRIPT / $LANGUAGE pair or
+    with $CHARSET.
 
     Each font in a fontset is associated with a particular
     script/language pair, with a particular charset, or with the
@@ -719,12 +837,11 @@ mfontset_copy (MFontset *fontset, char *name)
     charset.
 
     If both $SCRIPT and $CHARSET are not @c Mnil, two copies of $FONT
-    are created.  Then one is associated with the script/language pair
-    and the other with the charset.
+    are created.  Then one is associated with the $SCRIPT / $LANGUAGE
+    pair and the other with that charset.
 
-    If both $SCRIPT and $CHARSET are @c Mnil, $FONT is associated
-    with @c Mnil.  This kind of fonts are called <i>fallback
-    fonts</i>.
+    If both $SCRIPT and $CHARSET are @c Mnil, $FONT is associated with
+    @c Mnil.  This kind of fonts are called @e fallback @e fonts.
 
     The argument $HOW specifies the priority of $FONT.  If $HOW is
     positive, $FONT has the highest priority in the group of fonts
@@ -741,24 +858,47 @@ mfontset_copy (MFontset *fontset, char *name)
     @return
     If the operation was successful, mfontset_modify_entry () returns 0.
     Otherwise it returns -1 and assigns an error code to the external
-    variable @c merror_code.  */
+    variable #merror_code.  */
 
 /***ja
-    @brief ȥץȤȤ߹碌˥եȤϢդ
+    @brief եȥåȤƤѹ.
 
-    ؿ mfontset_set_language_script () ϡ$LANGUAGE  $SCRIPT 
-    ߹碌Фƥե $FONT Ȥ褦եȥå $FONTSET 
-    ꤹ롣$LANGUAGE  $SCRIPT ϡ줾ȥץȤ
-    ꤹ륷ܥǤ롣
+    ؿ mfontset_modify_entry () ϡ$LANGUAGE  $SCRIPT Ȥ߹
+    ޤ $CHARSET Ф $FONT ΥԡȤ褦ˡեȥ
+     $FONTSET ꤹ롣
 
-    ȥץȤȤ߹碌ĤФʣΥեȤꤹ뤳
-    ȤǤ롣$PREPEND_P  0 ʳʤС$FONT ϤȤ߹碌
-    ƺͥѤΤȤʤ롣ǤʤСͥٺΥե
-    ȤȤϿ롣
+    եȥåȤγƥեȤϡΥץȤȸΥڥ
+    ʸåȡܥ @c Mnil Τ줫ȴϢդƤ롣Ʊ
+    ΤȴϢդ줿եȤϥ롼פ롣
 
-    @return
-    Ȥmfontset_set_language_script ()  0 ֤
-    ԤȤ -1 ֤ѿ @c merror_code ˥顼ɤ
+    $SCRIPT  @c Mnil Ǥ뤫ץȤꤹ륷ܥǤ롣
+    ܥǤˤϡ$LANGUAGE ϸꤹ륷ܥ뤫 @c
+    Mnil Ǥꡢ$FONT the $SCRIPT / $LANGUAGE ڥ˴Ϣդ롣
+
+    $CHARSET  @c Mnil Ǥ뤫ʸåȥ֥Ȥɽܥ
+    Ǥ롣ܥǤˤ $FONT ϤʸåȤȴϢդ
+    롣
+
+    $SCRIPT  $CHARSET  @c Mnil Ǥʤˤ $FONT Υԡ
+    ĺ졢줾 $SCRIPT / $LANGUAGE ڥʸåȤ˴Ϣ
+    դ롣
+
+    $SCRIPT  $CHARSET  @c Mnil ʤС $FONT  @c Mnil 
+    Ϣդ롣μΥեȤ @e fallback @e font ȸƤФ롣
+
+     $HOW  $FONT ͥ٤ꤹ롣$HOW ʤС$FONT Ʊ
+    ΤȴϢդ줿롼Ǻǹͥ٤ġ$HOW 
+    Сͥ٤ġ$HOW  0 ʤС$FONT ϴϢդ줿
+    ΤФͣѲǽʥեȤȤʤꡢ¾ΥեȤϥ롼
+    롣
+
+    $LAYOUTER_NAME  @c Mnil Ǥ뤫@ref flt 򼨤ܥǤ롣
+    ܥǤС$FONT ѤM-text ɽݤˤϡ FONT
+    LAYOUT TABLE Ȥäʸ󤫤饰ե롣
+
+    @return 
+    Ȥmfontset_modify_entry ()  0 ֤
+    ԤȤ -1 ֤ѿ #merror_code ˥顼ɤ
     ꤹ롣  */
 
 /***
@@ -778,14 +918,17 @@ mfontset_modify_entry (MFontset *fontset,
   if (fontset->mdb)
     load_fontset_contents (fontset);
 
-  MPLIST_DO (pl, fontset->font_spec_list)
-    {
-      if (! memcmp (MPLIST_VAL (pl), spec, sizeof (MFont)))
-	{
-	  font = MPLIST_VAL (pl);
-	  break;
-	}
-    }
+  if (! fontset->font_spec_list)
+    fontset->font_spec_list = mplist ();
+  else
+    MPLIST_DO (pl, fontset->font_spec_list)
+      {
+	if (! memcmp (MPLIST_VAL (pl), spec, sizeof (MFont)))
+	  {
+	    font = MPLIST_VAL (pl);
+	    break;
+	  }
+      }
   if (! font)
     {
       font = mfont ();
@@ -822,9 +965,9 @@ mfontset_modify_entry (MFontset *fontset,
     layouter_name = Mt;
   for (i--; i >= 0; i--)
     {
-      if (how == -1)
+      if (how == 1)
 	mplist_push (plist[i], layouter_name, font);
-      else if (how == 1)
+      else if (how == -1)
 	mplist_add (plist[i], layouter_name, font);
       else
 	{
@@ -833,9 +976,110 @@ mfontset_modify_entry (MFontset *fontset,
 	}
     }
 
+  fontset->tick++;
   return 0;
 }
 
+/*=*/
+
+/***en
+    @brief Lookup a fontset.
+
+    The mfontset_lookup () function lookups $FONTSET and returns a
+    plist that describes the contents of $FONTSET corresponding to the
+    specified script, language, and charset.
+
+    If $SCRIPT is @c Mt, keys of the returned plist are script name
+    symbols for which some fonts are specified and values are NULL.
+
+    If $SCIRPT is a script symbol, the returned plist is decided by
+    $LANGUAGE.
+
+    If $LANGUAGE is @c Mt, keys of the plist are language name symbols
+    for which some fonts are specified and values are NULL.  A key may
+    be @c Mt which means some fallback fonts are specified for the
+    script.
+
+    If $LANGUAGE is a language name symbol, the plist is a @c
+    FONT-GROUP for the specified script and langauge.
+
+    If $LANGAUGE is @c Mt, the plist is fallback @c FONT-GROUP for the
+    script.
+
+    If $SCRIPT is @c Mnil, the returned plist is decided as below.
+
+    If $CHARSET is @c Mt, keys of the returned plist are charset name
+    symbols for which some fonts are specified and values are NULL.
+
+    If $CHARSET is a charset symbol, the plist is a @c FONT-GROUP for
+    the charset.
+
+    If $CHARSET is @c Mnil, the plist is a fallback @c FONT-GROUP.
+
+    @c FONT-GROUP is a plist whose keys are FLT name symbols (@c Mt if
+    no FLT is associated with the font) and values are pointers to
+    #MFont.
+
+    @return
+    It returns a plist describing the contents of a fontset.  The
+    plist should be freed by m17n_object_unref ().  */
+
+MPlist *
+mfontset_lookup (MFontset *fontset,
+		 MSymbol script, MSymbol language, MSymbol charset)
+{
+  MPlist *plist = mplist (), *pl, *p;
+
+  if (fontset->mdb)
+    load_fontset_contents (fontset);
+  if (script == Mt)
+    {
+      if (! fontset->per_script)
+	return plist;
+      p = plist;
+      MPLIST_DO (pl, fontset->per_script)
+	p = mplist_add (p, MPLIST_KEY (pl), NULL);
+      return plist;
+    }
+  if (script != Mnil)
+    {
+      if (! fontset->per_script)
+	return plist;
+      pl = mplist_get (fontset->per_script, script);
+      if (! pl)
+	return plist;
+      if (language == Mt)
+	{
+	  p = plist;
+	  MPLIST_DO (pl, pl)
+	    p = mplist_add (p, MPLIST_KEY (pl), NULL);
+	  return plist;
+	}
+      if (language == Mnil)
+	language = Mt;
+      pl = mplist_get (pl, language);
+    }
+  else if (charset != Mnil)
+    {
+      if (! fontset->per_charset)
+	return plist;
+      if (charset == Mt)
+	{
+	  p = plist;
+	  MPLIST_DO (pl, fontset->per_charset)
+	    p = mplist_add (p, MPLIST_KEY (pl), NULL);
+	  return plist;
+	}
+      pl = mplist_get (fontset->per_charset, charset);
+    }
+  else
+    pl = fontset->fallback;
+  if (! pl)
+    return plist;
+  return mplist_copy (pl);
+}
+
+
 /*** @} */
 
 /*** @addtogroup m17nDebug */
@@ -843,14 +1087,23 @@ mfontset_modify_entry (MFontset *fontset,
 /*** @{  */
 
 /***en
-    @brief Dump a fontset
+    @brief Dump a fontset.
 
-    The mdebug_dump_fontset () function prints $FONTSET in a human readable
+    The mdebug_dump_fontset () function prints fontset $FONTSET in a human readable
     way to the stderr.  $INDENT specifies how many columns to indent
     the lines but the first one.
 
     @return
     This function returns $FONTSET.  */
+/***ja
+    @brief եȥåȤפ.
+
+    ؿ mdebug_dump_face () ϥեȥå $FONTSET  stderr ˿
+    ֤˲ɤʷǰ롣 $INDENT ϣܰʹߤΥǥȤꤹ
+    롣
+
+    @return
+    δؿ $FONTSET ֤  */
 
 MFontset *
 mdebug_dump_fontset (MFontset *fontset, int indent)
