@@ -27,29 +27,29 @@
     The input driver @c minput_gui_driver is provided for internal
     input methods that is useful on window systems.  It displays
     preedit text and status text at the inputting spot.  See the
-    documentation of @c minput_gui_driver for more detail.
+    documentation of @c minput_gui_driver for more details.
 
     In the m17n-X library, the foreign input method of name @c Mxim is
     provided.  It uses XIM (X Input Method) as a background input
     engine.  The symbol @c Mxim has a property @c Minput_driver whose
     value is a pointer to the input driver @c minput_xim_driver.  See
-    the documentation of @c minput_xim_driver for more detail.  */
+    the documentation of @c minput_xim_driver for more details.  */
 
 /***ja
     @addtogroup m17nInputMethodWin
-    @brief ɥƥϥ᥽åɤΥݡ
+    @brief ɥƥϥ᥽åɤΥݡ.
 
-    ϥɥ饤 @c minput_gui_driver ϡɥƥ
-    Ѥϥ᥽åɤΤΤΤǤ롣Υɥ饤Фϥ
-    ݥåȤ preedit ƥȤ status ƥȤɽ롣ܺ٤ˤĤ
-    Ƥ @c minput_gui_driver ΥɥȤ򻲾ȤΤȡ
+    ϥɥ饤 @c minput_gui_driver ϡɥƥѤ
+    ϥ᥽åѤǤ롣Υɥ饤ФϥݥåȤ preedit 
+    ƥȤ status ƥȤɽ롣ܺ٤ˤĤƤ @c
+    minput_gui_driver 򻲾ȤΤȡ
 
     m17n-X 饤֥ϡ@c Mxim ȸ̾ĳϥ᥽åɤ
     Ƥ롣 XIM (X Input Method) Хå饦ɤϥ
     󥸥ȤѤ롣ܥ @c Mxim  @c Minput_driver Ȥ
     ץѥƥäƤꡢͤϥɥ饤 @c minput_xim_driver 
-    ؤΥݥ󥿤Ǥ롣 ܺ٤ˤĤƤ @c minput_xim_driver Υɥ
-    Ȥ򻲾ȤΤȡ  */
+    ؤΥݥ󥿤Ǥ롣 ܺ٤ˤĤƤ @c minput_xim_driver 
+    ȤΤȡ  */
 
 /*=*/
 
@@ -60,6 +60,7 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "config.h"
 #include "m17n-gui.h"
 #include "m17n-misc.h"
 #include "internal.h"
@@ -105,13 +106,13 @@ win_create_ic (MInputContext *ic)
   win_ic_info->ic_info = (MInputContextInfo *) ic->info;
   win_ic_info->frame = frame;
   win_ic_info->client.win = win_info->client;
-  mwin__window_geometry (frame, win_info->client, win_info->client,
+  (*frame->driver->window_geometry) (frame, win_info->client, win_info->client,
 			 &win_ic_info->client.geometry);
   win_ic_info->focus.win = win_info->focus;
-  mwin__window_geometry (frame, win_info->focus, win_info->client,
+  (*frame->driver->window_geometry) (frame, win_info->focus, win_info->client,
 			 &win_ic_info->focus.geometry);
 
-  win_ic_info->preedit.win = mwin__create_window (frame, win_info->client);
+  win_ic_info->preedit.win = (*frame->driver->create_window) (frame, win_info->client);
   win_ic_info->preedit.control.two_dimensional = 1;
   win_ic_info->preedit.control.as_image = 0;
   win_ic_info->preedit.control.with_cursor = 1;
@@ -120,11 +121,11 @@ win_create_ic (MInputContext *ic)
   win_ic_info->preedit.geometry.x = -1;
   win_ic_info->preedit.geometry.y = -1;
 
-  win_ic_info->status.win = mwin__create_window (frame, win_info->client);
+  win_ic_info->status.win = (*frame->driver->create_window) (frame, win_info->client);
   win_ic_info->status.control.as_image = 1;
   win_ic_info->status.control.enable_bidi = 1;
 
-  win_ic_info->candidates.win = mwin__create_window (frame, win_info->client);
+  win_ic_info->candidates.win = (*frame->driver->create_window) (frame, win_info->client);
   win_ic_info->candidates.control.as_image = 1;
 
   ic->info = win_ic_info;
@@ -137,10 +138,11 @@ win_destroy_ic (MInputContext *ic)
 {
   MInputGUIContextInfo *win_ic_info = (MInputGUIContextInfo *) ic->info;
   MInputContextInfo *ic_info = (MInputContextInfo *) win_ic_info->ic_info;
+  MFrame *frame = win_ic_info->frame;
 
-  mwin__destroy_window (win_ic_info->frame, win_ic_info->preedit.win);
-  mwin__destroy_window (win_ic_info->frame, win_ic_info->status.win);
-  mwin__destroy_window (win_ic_info->frame, win_ic_info->candidates.win);
+  (*frame->driver->destroy_window) (frame, win_ic_info->preedit.win);
+  (*frame->driver->destroy_window) (frame, win_ic_info->status.win);
+  (*frame->driver->destroy_window) (frame, win_ic_info->candidates.win);
   ic->info = ic_info;
   (*minput_default_driver.destroy_ic) (ic);
   free (win_ic_info);
@@ -281,7 +283,7 @@ adjust_window_and_draw (MFrame *frame, MInputContext *ic, MText *mt, int type)
 	}
     }
 
-  mwin__adjust_window (frame, win, geometry, &physical);
+  (*frame->driver->adjust_window) (frame, win, geometry, &physical);
   mdraw_text_with_control (frame, win, -x0, -y0, mt, 0, len, control);
 }
 
@@ -298,7 +300,7 @@ win_callback (MInputContext *ic, MSymbol command)
 
       if (! win_ic_info->preedit.mapped)
 	{
-	  mwin__map_window (frame, win_ic_info->preedit.win);
+	  (*frame->driver->map_window) (frame, win_ic_info->preedit.win);
 	  win_ic_info->preedit.mapped = 1;
 	}
       win_ic_info->preedit.control.cursor_pos = ic->cursor_pos;
@@ -352,7 +354,7 @@ win_callback (MInputContext *ic, MSymbol command)
 	{
 	  if (win_ic_info->candidates.mapped)
 	    {
-	      mwin__unmap_window (frame, win_ic_info->candidates.win);
+	      (*frame->driver->unmap_window) (frame, win_ic_info->candidates.win);
 	      win_ic_info->candidates.mapped = 0;
 	    }
 	  return;
@@ -360,7 +362,7 @@ win_callback (MInputContext *ic, MSymbol command)
 
       if (! win_ic_info->candidates.mapped)
 	{
-	  mwin__map_window (frame, win_ic_info->candidates.win);
+	  (*frame->driver->map_window) (frame, win_ic_info->candidates.win);
 	  win_ic_info->candidates.mapped = 1;
 	}
 
@@ -446,7 +448,7 @@ win_callback (MInputContext *ic, MSymbol command)
     {
       if (win_ic_info->preedit.mapped)
 	{
-	  mwin__unmap_window (frame, win_ic_info->preedit.win);
+	  (*frame->driver->unmap_window) (frame, win_ic_info->preedit.win);
 	  win_ic_info->preedit.mapped = 0;
 	}
     }
@@ -454,7 +456,7 @@ win_callback (MInputContext *ic, MSymbol command)
     {
       if (! win_ic_info->status.mapped)
 	{
-	  mwin__map_window (frame, win_ic_info->status.win);
+	  (*frame->driver->map_window) (frame, win_ic_info->status.win);
 	  win_ic_info->status.mapped = 1;
 	}
     }
@@ -462,7 +464,7 @@ win_callback (MInputContext *ic, MSymbol command)
     {
       if (win_ic_info->status.mapped)
 	{
-	  mwin__unmap_window (frame, win_ic_info->status.win);
+	  (*frame->driver->unmap_window) (frame, win_ic_info->status.win);
 	  win_ic_info->status.mapped = 0;
 	}
     }
@@ -470,7 +472,7 @@ win_callback (MInputContext *ic, MSymbol command)
     {
       if (! win_ic_info->candidates.mapped)
 	{
-	  mwin__map_window (frame, win_ic_info->candidates.win);
+	  (*frame->driver->map_window) (frame, win_ic_info->candidates.win);
 	  win_ic_info->candidates.mapped = 1;
 	}
     }
@@ -478,10 +480,32 @@ win_callback (MInputContext *ic, MSymbol command)
     {
       if (win_ic_info->candidates.mapped)
 	{
-	  mwin__unmap_window (frame, win_ic_info->candidates.win);
+	  (*frame->driver->unmap_window) (frame, win_ic_info->candidates.win);
 	  win_ic_info->candidates.mapped = 0;
 	}
     }
+  else if (command == Minput_reset)
+    {
+      MInputCallbackFunc func;
+
+      if (minput_default_driver.callback_list
+	  && (func = ((MInputCallbackFunc)
+		      mplist_get (minput_default_driver.callback_list,
+				  Minput_reset))))
+	{
+	  MInputContextInfo *ic_info
+	    = (MInputContextInfo *) win_ic_info->ic_info;
+	  ic->info = ic_info;
+	  (func) (ic, Minput_reset);
+	  ic->info = win_ic_info;
+	}
+      if (ic->preedit_changed)
+	minput__callback (ic, Minput_preedit_draw);
+      if (ic->status_changed)
+	minput__callback (ic, Minput_status_draw);
+      if (ic->candidates_changed)
+	minput__callback (ic, Minput_candidates_draw);
+    }
 }
 
 static int
@@ -523,6 +547,7 @@ minput__win_init ()
     plist = mplist_add (plist, Minput_candidates_done, (void *) win_callback);
     plist = mplist_add (plist, Minput_set_spot, (void *) win_callback);
     plist = mplist_add (plist, Minput_toggle, (void *) win_callback);
+    plist = mplist_add (plist, Minput_reset, (void *) win_callback);
   }
   minput_driver = &minput_gui_driver;
 
@@ -569,9 +594,8 @@ minput__win_fini ()
     displays them at the input spot set by the function
     minput_set_spot ().
 
-    The function m17n_initialize_win () set the variable @c
-    minput_driver to the pointer to this driver so that all internal
-    input methods use it.
+    The macro M17N_INIT () set the variable @c minput_driver to the
+    pointer to this driver so that all internal input methods use it.
 
     Therefore, unless @c minput_driver is changed from the default,
     the driver dependent arguments to the functions whose name begin
@@ -581,7 +605,7 @@ minput__win_fini ()
 
     The argument $ARG of the function minput_create_ic () must be a
     pointer to the structure @c MInputGUIArgIC.  See the documentation
-    of @c MInputGUIArgIC for more detail.
+    of @c MInputGUIArgIC for more details.
 
     If the argument $KEY is @c Mnil, the argument $ARG of the
     function minput_filter () must be a pointer to the object of type
@@ -591,40 +615,55 @@ minput__win_fini ()
     same one as that of the function minput_filter (). */
 
 /***ja
-    @brief ɥƥϥ᥽åϥɥ饤
+    @brief ɥƥϥ᥽åϥɥ饤.
 
     ϥɥ饤 @c minput_gui_driver ϡɥƥѤ
-    ϥ᥽åѤΤΤǤ롣
+    ϥ᥽åѤǤ롣
 
     Υɥ饤Фϡؿ minput_set_spot () ˤäꤵ줿ϥݥ
     Ȥ preedit ƥѤΥ֥ɥ status ƥѤΥ
     ɥꡢ줾ɽ롣
 
-    ؿ m17n_initialize_win () ѿ @c minput_driver 򤳤Υɥ饤
+    ޥ M17N_INIT () ѿ @c minput_driver 򤳤Υɥ饤
     ؤΥݥ󥿤ꤷƤϥ᥽åɤΥɥ饤ФȤ
     ˤ롣
 
     äơ@c minput_driver ǥեͤΤޤޤǤСminput_ 
-    ǻϤޤ̾İʲδؿΥɥ饤Ф˰¸ϼΤ褦
+    ǻϤޤ̾ĴؿΰΤɥ饤а¸ΤΤϰʲΤ褦
     ʤ롣
 
     ؿ minput_open_im () ΰ $ARG ̵뤵롣
 
     ؿ minput_create_ic () ΰ $ARG Ϲ¤ @c MInputGUIArgIC 
     Υݥ󥿤ǤʤƤϤʤʤܺ٤ˤĤƤ @c MInputGUIArgIC 
-    ɥȤ򻲾ȤΤȡ
+    򻲾ȤΤȡ
 
-    ؿ minput_filter () ΰ $ARG  @c Mnil ξ硢 $ARG  
-    XEvent Υ֥ȤؤΥݥ󥿤ǤʤƤϤʤʤξ $KEY 
-     $ARG 롣
+    ؿ minput_filter () ΰ $ARG  @c Mnil ξ硢 $ARG  @c
+    XEvent Υ֥ȤؤΥݥ󥿤ǤʤƤϤʤʤξ 
+    $KEY  $ARG 롣
 
-    ؿ minput_lookup () ΰ $ARG ϴؿ minput_filter ()  
+    ؿ minput_lookup () ΰ $ARG ϴؿ minput_filter () ΰ 
     $ARG ƱǤʤƤϤʤʤ */
 
 MInputDriver minput_gui_driver;
 
 /*=*/
 
+/***en
+    @brief Symbol of the name "xim".
+
+    The variable Mxim is a symbol of name "xim".  It is a name of the
+    input method driver #minput_xim_driver.  */ 
+/***ja
+    @brief "xim"̾Ȥƻĥܥ .
+
+    ѿ Mxim "xim"̾ȤƻĥܥǤ롣"xim" ϥ᥽
+    ɥɥ饤 #minput_xim_driver ̾Ǥ롣  */ 
+
+MSymbol Mxim;
+
+/*=*/
+
 /***en
     @brief Convert an event to an input key.
 
@@ -632,13 +671,11 @@ MInputDriver minput_gui_driver;
     corresponding to event $EVENT on $FRAME by a window system
     dependent manner.
 
-    In the m17n-X library, $EVENT must be a pointer to the struct @c
-    XKeyEvent, and it is handled as below.
+    In the m17n-X library, $EVENT must be a pointer to the structure
+    @c XKeyEvent, and it is handled as below.
 
     At first, the keysym name of $EVENT is acquired by the function @c
-    XKeysymToString.
-
-    Then, the name is modified as below.
+    XKeysymToString.  Then, the name is modified as below.
 
     If the name is one of "a" .. "z" and $EVENT has a Shift modifier,
     the name is converted to "A" .. "Z" respectively, and the Shift
@@ -649,7 +686,7 @@ MInputDriver minput_gui_driver;
     cleared.
 
     If $EVENT still has Shift, Control, Meta, Alt, Super, and/or Hyper
-    modifiers, the name is prepended by "S-", "C-", "M-", "A-", "s-",
+    modifiers, the name is preceded by "S-", "C-", "M-", "A-", "s-",
     and/or "H-" respectively in this order.
 
     For instance, if the keysym name is "a" and the event has Shift,
@@ -658,55 +695,44 @@ MInputDriver minput_gui_driver;
     At last, a symbol who has the name is returned.  */
 
 /***ja
-    @brief ̾ΤϥѴ
+    @brief ٥ȤϥѴ.
 
-    ؿ minput_name_to_key () ϡ̾ $NAME бϥ֤
+    ؿ minput_name_to_key () ϡ$FRAME Υ٥ $EVENT б
+    ϥ֤ǤΡбפϥɥƥ¸Ǥ롣
 
-    $NAME ϼηȤ롣
+    m17n-X 饤֥ξˤϡ$EVENT  ¤ @c XKeyEvent ؤΥ
+    󥿤ǤꡢΤ褦˽롣
 
-	[ MODIFIER-MNEMONIC '-' ] * KEY-NAME
+    ޤؿ @c XKeysymToString ˤäơ$EVENT  keysym ̾
+    ǰʲѹä롣
 
-     MODIFIER-MNEMONIC  'S', 'C', 'M', 'A', 's', 'H' Τ
-    Ǥꡢ줾 Shift, Control, Meta, Alt, Super, Hyper γƥ
-    ǥե򼨤KEY-NAME ϥΥܥå̾ɽʸ
+    ̾ "a" .. "z" Τ줫Ǥä $EVENT  Shift ǥե
+    С̾Ϥ줾 "A" .. "Z" Ѵ졢Shift ǥե
+    ϼ롣
 
-    ϥβ 16 ӥåȤ "keysym bits" ȤФ졢KEY-NAME б
-    륳ɤɽ7 bits (Ĥޤ 17 ӥåܤ 23 ӥåܤޤ) 
-    "modifier bits" ȸƤФ졢MODIFIER-MNEMONIC ɽƤ롣
-
-    KEY-NAME  1 ХĹǤ硢ϥϼΤ褦˹롣
-
-    ޤkeysym bits ˤϤΥХȥɤΤΤꤵ롣
-
-    ǡShift, Control, Meta γƥǥեϰʲμ³
-    keysym bits ȿǤ롣
-
-    (1) Shift ǥեꡢkeysym bits ʸǤ(Ĥ
-     'a'  'z' Ǥ), ʸ (Ĥޤ 'A' through 'Z') Ѵ
-    롣Shift ǥե modifier bits 롣
-
-    (2) Control ǥե, keysym bits ϥӥåñ̤ 
-    0x1F  and 黻ԤControl ǥե modifier bits 
+    ̾ХĹ $EVENT  Control ǥեС̾
+     0x1F ӥåñ and 黻롣Control ǥեϼ
     롣
 
-    (3) Meta ǥե, keysym bits ϥӥåñ̤ 0x80 
-     or 黻Ԥ Meta ǥե modifier bits 
-    롣
+    $EVENT ˤޤ Shift, Control, Meta, Alt, Super, Hyper ʤɤΥǥ
+    եС̾ˤ줾"S-", "C-", "M-", "A-", "s-",
+    "H-" դ
+    
+    ȤСkeysym ̾ "a" ǥ٥Ȥ Shift, Meta, and Hyper ǥ
+    եƤС̾ "H-M-A" Ǥ롣
 
-    ȤС"S-a"  "A" Ϥɤ 65 "C-a"  "C-A" Ϥɤ
-    1 ֤"M-a"  225 , "C-M-a"  129 ֤
+    Ǹˤ̾ĥܥ֤*/
 
-    KEY-NAME  2 ХȰʾǤˤϡбkeysym bits @c
-    m17n @c 饤֥ Υơ֥ˤäǤ롣ΤȤ
-    ơ֥ KEY-NAME ʤСδؿ -1 ֤  */
 
 MSymbol
 minput_event_to_key (MFrame *frame, void *event)
 {
   int modifiers;
-  MSymbol key = mwin__parse_event (frame, event, &modifiers);
+  MSymbol key;
   char *name, *str;
 
+  M_CHECK_READABLE (frame, MERROR_IM, Mnil);
+  key = (*frame->driver->parse_event) (frame, event, &modifiers);
   if (! modifiers)
     return key;
 
