@@ -22,17 +22,16 @@
 
 /***en
     @addtogroup m17nInputMethod
-    @brief API for Input method
+    @brief API for Input method.
 
     An input method is an object to enable inputting various
     characters.  An input method is identified by a pair of symbols,
-    LANGUAGE and NAME.  This pair decides an input driver of the input
-    method.  An input driver is a set of functions for handling the
-    input method.  There are two kinds of input methods; internal one
-    and foreign one.
+    LANGUAGE and NAME.  This pair decides a input method driver of the
+    input method.  An input method driver is a set of functions for
+    handling the input method.  There are two kinds of input methods;
+    internal one and foreign one.
 
     <ul>
-
     <li> Internal Input Method
 
     An internal input method has non @c Mnil LANGUAGE, and the body is
@@ -42,12 +41,12 @@
     GUI use.  Those driver utilize the input processing engine
     provided by the m17n library itself.  The m17n database may
     provides an input method that is not only for a specific language.
-    The database uses @c Mt as the language of such an input method.
+    The database uses @c Mt as LANGUAGE of such an input method.
 
     An internal input method accepts an input key which is a symbol
     associated with an input event.  As there is no way for the @c
     m17n @c library to know how input events are represented in an
-    application program, a application programmer have to convert an
+    application program, an application programmer have to convert an
     input event to an input key by himself.  See the documentation of
     the function minput_event_to_key () for the detail.
 
@@ -57,72 +56,82 @@
     defined in an external resources (e.g. XIM of X Window System).
     For this kind of input methods, the symbol NAME must have a
     property of key @c Minput_driver, and the value must be a pointer
-    to an input driver.  So, by preparing a proper input 
+    to an input method driver.  Therefore, by preparing a proper
     driver, any kind of input method can be treated in the framework
     of the @c m17n @c library.
 
-    For convenience, the m17n-X library provides an input driver that
-    enables the input style of OverTheSpot for XIM, and stores @c
-    Minput_driver property of the symbol @c Mxim with a pointer to
-    that driver.  See the documentation of m17n GUI API for the
-    detail.
+    For convenience, the m17n-X library provides an input method
+    driver that enables the input style of OverTheSpot for XIM, and
+    stores @c Minput_driver property of the symbol @c Mxim with a
+    pointer to the driver.  See the documentation of m17n GUI API for
+    the detail.
 
     </ul>
 
     PROCESSING FLOW
 
-    The typical processing flow of handling an input method is: open
-    an input method, create an input context for the input method,
-    filter an input key, and looking up a produced text in the input
-    context.  */
+    The typical processing flow of handling an input method is: 
+
+     @li open an input method
+     @li create an input context for the input method
+     @li filter an input key
+     @li look up a produced text in the input context  */
 
 /*=*/
-
 /***ja
     @addtogroup m17nInputMethod
-    @brief ϥ᥽åAPI
+    @brief ϥ᥽åAPI.
 
     ϥ᥽åɤ¿ͤʸϤ뤿Υ֥ȤǤ롣ϥ
     åɤϥܥ LANGUAGE  NAME ȤˤäƼ̤졢Ȥˤ
-    ϥɥ饤Фޤ롣ϥɥ饤ФȤϻϥ᥽åɤ򰷤
-    δؿνޤǤ롣
+    ϥ᥽åɥɥ饤Фޤ롣ϥ᥽åɥɥ饤ФȤϤϥ
+    åɤ򰷤δؿνޤǤ롣 ϥ᥽åɤˤ᥽
+    ɤȳ᥽åɤ̤꤬롣
 
-    ϥ᥽åɤˤ᥽åɤȳ᥽åɤ̤꤬롣
-
-    ϥ᥽å
+    <ul> 
+    <li> ϥ᥽å
 
     ϥ᥽åɤ LANGUAGE  @c Mnil ʳΤΤǤꡢΤ
-    m17n ١ <Minput_method, LANGUAGE, NAME>Ȥ
-    Ƥ롣μϥ᥽åɤФơm17n饤֥
-    CUIѤGUIѤ줾ϥɥ饤Ф餫Ƥ롣
-    Υɥ饤Фm17n饤֥꼫ȤϽ󥸥Ѥ롣
+    m17n ǡ١ <Minput_method, LANGUAGE, NAME>Ȥդ
+    Ƥ롣μϥ᥽åɤФơm17n 饤֥
+    CUIѤGUIѤ줾ϥ᥽åɥɥ饤Ф餫
+    롣Υɥ饤Фm17n饤֥꼫ȤϽ󥸥
+    롣m17n ǡ١ˤϡθѤǤʤϥ᥽åɤ
+    뤳ȤǤΤ褦ϥ᥽åɤ LANGUAGE  @c Mt Ǥ
+    롣
 
-    ϥ᥽åɤϡ桼ϥ٥Ȥбϥ
-    롣ϥϥܥǤ롣@c m17n @c 饤֥ ϥ٥
-    ץꥱ󡦥ץǤɤΤ褦ɽƤ뤫Τ
-    ʤΤǡϥ٥ȤϥؤѴϥץޤǤ
-    ǹԤʤƤϤʤʤܺ٤ˤĤƤϴؿ minput_event_to_key () 
-    ΥɥȤ򻲾ȤΤȡ
+    ϥ᥽åɤϡ桼ϥ٥ȤбܥǤ
+    ϥ롣@c m17n @c 饤֥ ϥ٥Ȥץꥱ
+    ץǤɤɽƤ뤫ΤѤʤΤǡ
+    ٥ȤϥؤѴϥץꥱץޤǤǹ
+    ʤƤϤʤʤܺ٤ˤĤƤϴؿ minput_event_to_key () 
+    򻲾ȡ
 
-    ϥ᥽å
+    <li> ϥ᥽å
 
     ϥ᥽åɤ LANGUAGE  @c Mnil ΤΤǤꡢΤϳ
-    Υ꥽Ȥ롣ʤȤX Window System XIM ʤɡ) 
-    μϥ᥽åɤǤϥܥ NAME @c Minput_driver 򥭡
-    ץѥƥͤϥɥ饤ФؤΥݥ󥿤ǤʤƤϤ
-    ʤäơŬڤϥɥ饤Ф뤳Ȥˤäơ
-    ʤϥ᥽åɤ @c m17n @c 饤֥ȤǰǤ롣
+    ꥽Ȥ롣ʤȤX Window System XIM ʤɡ) 
+    μϥ᥽åɤǤϡܥ NAME @c Minput_driver 򥭡
+    Ȥץѥƥͤϥ᥽åɥɥ饤ФؤΥݥ󥿤
+    ʤƤϤʤʤäơŬڤʥɥ饤Ф뤳Ȥˤäơ
+    ʤϥ᥽åɤ @c m17n @c 饤֥ Ȥǰ
+    Ǥ롣
 
-    ñΤᡢm17n X 饤֥XIM  OverTheSpot ϥ
-    ¸ϥɥ饤Ф󶡤Ƥ롣ޤܥ @c Mxim  @c
+    Τᡢm17n X 饤֥ XIM  OverTheSpot ϥ
+    ¸ϥ᥽åɥɥ饤Ф󶡤ޤܥ @c Mxim  @c
     Minput_driver ץѥƥͤȤƤΥɥ饤ФؤΥݥ󥿤ݻ
-    Ƥ롣ܺ٤ˤĤƤm17n-win API ΥɥȤ򻲾ȤΤȡ
+    Ƥ롣ܺ٤ˤĤƤ m17n GUI API ΥɥȤ򻲾ȤΤȡ
+
+    </ul> 
 
     ή
 
-    ϥ᥽åɽŵŪʽϰʲΤ褦ˤʤ롣ϥ᥽åɤΥ
-    ץ󡢤ϥ᥽åɤϥƥȤϥ٥ȤΥե
-    륿ϥƥȤǤƥȤθ     */
+    ϥ᥽åɽŵŪʽϰʲΤ褦ˤʤ롣
+    
+    @li ϥ᥽åɤΥץ
+    @li ϥ᥽åɤϥƥȤ
+    @li ϥ٥ȤΥե륿
+    @li ϥƥȤǤƥȤθ     */
 
 /*=*/
 
@@ -132,9 +141,13 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <dlfcn.h>
 
 #include "config.h"
+
+#ifdef HAVE_DLFCN_H
+#include <dlfcn.h>
+#endif
+
 #include "m17n-gui.h"
 #include "m17n-misc.h"
 #include "internal.h"
@@ -143,6 +156,8 @@
 #include "symbol.h"
 #include "plist.h"
 
+static int mdebug_mask = MDEBUG_INPUT;
+
 static MSymbol Minput_method;
 
 /** Symbols to load an input method data.  */
@@ -224,7 +239,7 @@ integer_value (MInputContext *ic, MPlist *arg)
   MInputContextInfo *ic_info = (MInputContextInfo *) ic->info;
   int code;
   MText *preedit = ic->preedit;
-  int len = mtext_nbytes (preedit);
+  int len = mtext_nchars (preedit);
 
   if (MPLIST_INTEGER_P (arg))
     return MPLIST_INTEGER (arg);
@@ -241,7 +256,7 @@ integer_value (MInputContext *ic, MPlist *arg)
     code = ic->cursor_pos + 1;
   else if (code == '<')
     code = 0;
-  else if (code == '<')
+  else if (code == '>')
     code = len;
   return (code >= 0 && code < len ? mtext_ref_char (preedit, code) : -1);
 }
@@ -677,8 +692,6 @@ load_input_method (MSymbol language, MSymbol name, MPlist *plist,
   MPlist *macros = NULL;
   MPlist *elt;
 
-  if (! MPLIST_PLIST_P (plist))
-    MERROR (MERROR_IM, -1);
   for (; MPLIST_PLIST_P (plist); plist = MPLIST_NEXT (plist))
     {
       elt = MPLIST_PLIST (plist);
@@ -738,9 +751,12 @@ load_input_method (MSymbol language, MSymbol name, MPlist *plist,
 	}
     }
 
-  MPLIST_DO (elt, maps)
-    M17N_OBJECT_UNREF (MPLIST_VAL (elt));
-  M17N_OBJECT_UNREF (maps);
+  if (maps)
+    {
+      MPLIST_DO (elt, maps)
+	M17N_OBJECT_UNREF (MPLIST_VAL (elt));
+      M17N_OBJECT_UNREF (maps);
+    }
   if (! title)
     title = mtext_from_data (MSYMBOL_NAME (name), MSYMBOL_NAMELEN (name),
 			     MTEXT_FORMAT_US_ASCII);
@@ -798,7 +814,7 @@ shift_state (MInputContext *ic, MSymbol state_name)
 {
   MInputMethodInfo *im_info = (MInputMethodInfo *) ic->im->info;
   MInputContextInfo *ic_info = (MInputContextInfo *) ic->info;
-  MIMState *state = ic_info->state;
+  MIMState *state;
 
   /* Find a state to shift to.  If not found, shift to the initial
      state.  */
@@ -806,6 +822,8 @@ shift_state (MInputContext *ic, MSymbol state_name)
   if (! state)
     state = (MIMState *) MPLIST_VAL (im_info->states);
 
+  MDEBUG_PRINT1 ("\n[IM] state-shift (%s)", MSYMBOL_NAME (state->name));
+
   /* Enter the new state.  */
   ic_info->state = state;
   ic_info->map = state->map;
@@ -820,6 +838,16 @@ shift_state (MInputContext *ic, MSymbol state_name)
       mtext_put_prop_values (ic->preedit, 0, mtext_nchars (ic->preedit),
 			     Mcandidate_index, NULL, 0);
       mtext_cat (ic->produced, ic->preedit);
+      if ((mdebug__flag & mdebug_mask)
+	  && mtext_nchars (ic->produced) > 0)
+	{
+	  int i;
+
+	  MDEBUG_PRINT (" (produced");
+	    for (i = 0; i < mtext_nchars (ic->produced); i++)
+	      MDEBUG_PRINT1 (" U+%04X", mtext_ref_char (ic->produced, i));
+	  MDEBUG_PRINT (")");
+	}
       mtext_reset (ic->preedit);
       ic->candidate_list = NULL;
       ic->candidate_show = 0;
@@ -843,9 +871,17 @@ shift_state (MInputContext *ic, MSymbol state_name)
   if (ic_info->key_head == ic_info->used
       && ic_info->map == ic_info->state->map
       && ic_info->map->map_actions)
-    take_action_list (ic, ic_info->map->map_actions);
+    {
+      MDEBUG_PRINT (" init-actions:");
+      take_action_list (ic, ic_info->map->map_actions);
+    }
 }
 
+/* Find a candidate group that contains a candidate number INDEX from
+   PLIST.  Set START_INDEX to the first candidate number of the group,
+   END_INDEX to the last candidate number plus 1, GROUP_INDEX to the
+   candidate group number if they are non-NULL.  If INDEX is -1, find
+   the last candidate group.  */
 
 static MPlist *
 find_candidates_group (MPlist *plist, int index,
@@ -859,7 +895,8 @@ find_candidates_group (MPlist *plist, int index,
 	len = mtext_nchars (MPLIST_MTEXT (plist));
       else
 	len = mplist_length (MPLIST_PLIST (plist));
-      if (i + len > index)
+      if (index < 0 ? MPLIST_TAIL_P (MPLIST_NEXT (plist))
+	  : i + len > index)
 	{
 	  if (start_index)
 	    *start_index = i;
@@ -954,7 +991,7 @@ new_index (MInputContext *ic, int current, int limit, MSymbol sym, MText *mt)
 }
 
 static void
-udpate_candidate (MInputContext *ic, MTextProperty *prop, int idx)
+update_candidate (MInputContext *ic, MTextProperty *prop, int idx)
 {
   int from = mtext_property_start (prop);
   int to = mtext_property_end (prop);
@@ -1018,6 +1055,7 @@ take_action_list (MInputContext *ic, MPlist *action_list)
 	  args = MPLIST_NEXT (action);
 	}
 
+      MDEBUG_PRINT1 (" %s", MSYMBOL_NAME (name));
       if (name == Minsert)
 	{
 	  if (MPLIST_MTEXT_P (args))
@@ -1088,9 +1126,14 @@ take_action_list (MInputContext *ic, MPlist *action_list)
 				     end - start - 1, MPLIST_SYMBOL (args),
 				     NULL)
 			: MPLIST_INTEGER (args)));
-	      if (idx < 0
-		  || (idx >= end
-		      && MPLIST_TAIL_P (MPLIST_NEXT (group))))
+	      if (idx < 0)
+		{
+		  find_candidates_group (mtext_property_value (prop), -1,
+					 NULL, &end, NULL);
+		  idx = end - 1;
+		}
+	      else if (idx >= end
+		       && MPLIST_TAIL_P (MPLIST_NEXT (group)))
 		idx = 0;
 	    }
 	  else
@@ -1123,7 +1166,7 @@ take_action_list (MInputContext *ic, MPlist *action_list)
 		ingroup_index = len - 1;
 	      idx += ingroup_index;
 	    }
-	  udpate_candidate (ic, prop, idx);
+	  update_candidate (ic, prop, idx);
 	}
       else if (name == Mshow)
 	ic->candidate_show = 1;
@@ -1276,12 +1319,13 @@ take_action_list (MInputContext *ic, MPlist *action_list)
 	  else
 	    val1 /= val2;
 	  mplist_put (ic_info->vars, sym, (void *) val1);
+	  MDEBUG_PRINT2 ("(%s=%d)", MSYMBOL_NAME (sym), val1);
 	}
       else if (name == Mequal || name == Mless || name == Mgreater)
 	{
 	  int val1, val2;
 	  MPlist *actions1, *actions2;
-	  int ret;
+	  int ret = 0;
 
 	  val1 = integer_value (ic, args);
 	  args = MPLIST_NEXT (args);
@@ -1351,6 +1395,9 @@ handle_key (MInputContext *ic)
   MSymbol key = ic_info->keys[ic_info->key_head];
   int i;
 
+  MDEBUG_PRINT2 ("[IM] handle `%s' in state %s", 
+		 MSYMBOL_NAME (key), MSYMBOL_NAME (ic_info->state->name));
+
   if (map->submaps)
     {
       submap = mplist_get (map->submaps, key);
@@ -1360,12 +1407,14 @@ handle_key (MInputContext *ic)
 
   if (submap)
     {
+      MDEBUG_PRINT (" submap-found");
       mtext_cpy (ic->preedit, ic_info->preedit_saved);
       ic->cursor_pos = ic_info->state_pos;
       ic_info->key_head++;
       ic_info->map = map = submap;
       if (map->map_actions)
 	{
+	  MDEBUG_PRINT (" map-actions:");
 	  if (take_action_list (ic, map->map_actions) < 0)
 	    return -1;
 	}
@@ -1388,6 +1437,7 @@ handle_key (MInputContext *ic)
 	{
 	  if (map->branch_actions)
 	    {
+	      MDEBUG_PRINT (" branch-actions:");
 	      if (take_action_list (ic, map->branch_actions) < 0)
 		return -1;
 	    }
@@ -1396,6 +1446,7 @@ handle_key (MInputContext *ic)
 	  if (ic_info->map != ic_info->state->map)
 	    shift_state (ic, ic_info->state->name);
 	}
+      MDEBUG_PRINT ("\n");
     }
   else
     {
@@ -1404,14 +1455,20 @@ handle_key (MInputContext *ic)
       /* If MAP is the root map of the initial state, it means that
 	 the current input method can not handle KEY.  */
       if (map == ((MIMState *) MPLIST_VAL (im_info->states))->map)
-	return -1;
+	{
+	  MDEBUG_PRINT (" unhandled\n");
+	  return -1;
+	}
 
       if (map != ic_info->state->map)
 	{
 	  /* If MAP is not the root map... */
 	  /* If MAP has branch actions, perform them.  */
 	  if (map->branch_actions)
-	    take_action_list (ic, map->branch_actions);
+	    {
+	      MDEBUG_PRINT (" branch-actions:");
+	      take_action_list (ic, map->branch_actions);
+	    }
 	  /* If MAP is still not the root map, shift to the current
 	     state. */
 	  if (ic_info->map != ic_info->state->map)
@@ -1419,7 +1476,10 @@ handle_key (MInputContext *ic)
 	      shift_state (ic, ic_info->state->name);
 	      /* If MAP has branch_actions, perform them.  */
 	      if (ic_info->map->branch_actions)
-		take_action_list (ic, ic_info->map->branch_actions);
+		{
+		  MDEBUG_PRINT (" init-actions:");
+		  take_action_list (ic, ic_info->map->branch_actions);
+		}
 	    }
 	}
       else
@@ -1427,33 +1487,42 @@ handle_key (MInputContext *ic)
 	  /* MAP is the root map, perform branch actions (if any) or
 	     shift to the initial state.  */
 	  if (map->branch_actions)
-	    take_action_list (ic, map->branch_actions);
+	    {
+	      MDEBUG_PRINT (" branch-actions:");
+	      take_action_list (ic, map->branch_actions);
+	    }
 	  else
 	    shift_state (ic,
 			 ((MIMState *) MPLIST_VAL (im_info->states))->name);
 	}
+      MDEBUG_PRINT ("\n");
     }
   return 0;
 }
 
 static void
-reset_ic (MInputContext *ic)
+reset_ic (MInputContext *ic, MSymbol ignore)
 {
   MInputMethodInfo *im_info = (MInputMethodInfo *) ic->im->info;
   MInputContextInfo *ic_info = (MInputContextInfo *) ic->info;
 
+  if (im_info->states)
+    /* Shift to the initial state.  */
+    shift_state (ic, Mnil);
+  else
+    ic_info->state = NULL;
   MLIST_RESET (ic_info);
-  ic_info->state = (MIMState *) MPLIST_VAL (im_info->states);
-  ic_info->map = ic_info->state->map;
+  ic_info->map = ic_info->state ? ic_info->state->map : NULL;
   ic_info->state_key_head = ic_info->key_head = 0;
+  ic_info->key_unhandled = 0;
   ic->cursor_pos = ic_info->state_pos = 0;
-  ic->status = ic_info->state->title;
+  ic->status = ic_info->state ? ic_info->state->title : NULL;
   if (! ic->status)
     ic->status = im_info->title;
   ic->candidate_list = NULL;
   ic->candidate_show = 0;
   ic->status_changed = ic->preedit_changed = ic->candidates_changed = 1;
-  if (ic_info->map->map_actions)
+  if (ic_info->map && ic_info->map->map_actions)
     take_action_list (ic, ic_info->map->map_actions);
 }
 
@@ -1562,7 +1631,7 @@ create_ic (MInputContext *ic)
 	}
       M17N_OBJECT_UNREF (func_args);
     }
-  reset_ic (ic);
+  reset_ic (ic, Mnil);
   return 0;
 }
 
@@ -1609,6 +1678,11 @@ filter (MInputContext *ic, MSymbol key, void *arg)
   MInputContextInfo *ic_info = (MInputContextInfo *) ic->info;
   int i = 0;
 
+  if (! ic_info->state)
+    {
+      ic_info->key_unhandled = 1;
+      return 0;
+    }
   mtext_reset (ic->produced);
   ic->status_changed = ic->preedit_changed = ic->candidates_changed = 0;
   MLIST_APPEND1 (ic_info, keys, key, MERROR_IM);
@@ -1618,7 +1692,7 @@ filter (MInputContext *ic, MSymbol key, void *arg)
       {
 	/* KEY was not handled.  Reset the status and break the
 	   loop.  */
-	reset_ic (ic);
+	reset_ic (ic, Mnil);
 	/* This forces returning 1.  */
 	ic_info->key_unhandled = 1;
 	break;
@@ -1626,7 +1700,7 @@ filter (MInputContext *ic, MSymbol key, void *arg)
     if (i++ == 100)
       {
 	mdebug_hook ();
-	reset_ic (ic);
+	reset_ic (ic, Mnil);
 	ic_info->key_unhandled = 1;
 	break;
       }
@@ -1775,6 +1849,7 @@ minput__init ()
   Minput_candidates_draw = msymbol ("input-candidates-draw");
   Minput_set_spot = msymbol ("input-set-spot");
   Minput_toggle = msymbol ("input-toggle");
+  Minput_reset = msymbol ("input-reset");
 
   Mcandidate_list = msymbol_as_managing_key ("  candidate-list");
   Mcandidate_index = msymbol ("  candidate-index");
@@ -1820,7 +1895,9 @@ minput__init ()
   minput_default_driver.destroy_ic = destroy_ic;
   minput_default_driver.filter = filter;
   minput_default_driver.lookup = lookup;
-  minput_default_driver.callback_list = NULL;
+  minput_default_driver.callback_list = mplist ();
+  mplist_put (minput_default_driver.callback_list, Minput_reset,
+	      (void *) reset_ic);
   minput_driver = &minput_default_driver;
   return 0;
 }
@@ -1878,7 +1955,13 @@ minput__char_to_key (int c)
 
     These are the predefined symbols that are used as the @c COMMAND
     argument of callback functions of an input method driver (see
-    #MInputDriver::callback_list).  */ 
+    #MInputDriver::callback_list ).  */ 
+/***ja
+    @name ѿ ХåޥѤߥܥ.
+
+    ϥ᥽åɥɥ饤ФΥХåؿˤ @c COMMAND Ȥ
+    ѤѤߥܥ (#MInputDriver::callback_list )
+      */ 
 /*** @{ */ 
 /*=*/
 
@@ -1893,17 +1976,18 @@ MSymbol Minput_candidates_done;
 MSymbol Minput_candidates_draw;
 MSymbol Minput_set_spot;
 MSymbol Minput_toggle;
+MSymbol Minput_reset;
 /*** @} */
 /*=*/
 
 /***en
-    @brief The default input driver for internal input methods.
+    @brief The default driver for internal input methods.
 
     The variable #minput_default_driver is the default driver for
     internal input methods.
 
     The member MInputDriver::open_im () searches the m17n database for
-    an input method that matches the tag \<#Minput_method, $LANGUAGE,
+    an input method that matches the tag \< #Minput_method, $LANGUAGE,
     $NAME\> and loads it.
 
     The member MInputDriver::callback_list () is @c NULL.  Thus, it is
@@ -1919,33 +2003,45 @@ MSymbol Minput_toggle;
     "minput_" are all ignored.  */
 
 /***ja
-    @brief ϥ᥽åѥǥեϥɥ饤
+    @brief ϥ᥽åѥǥեȥɥ饤.
 
-    ϥɥ饤 minput_default_driver ϥ᥽åѤΥǥե
-    Ȥϥɥ饤ФǤ롣
+    ϥɥ饤 #minput_default_driver ϥ᥽åѤΥǥե
+    ȤΥɥ饤ФǤ롣
 
-    Υɥ饤Ф <callback> Ф @c NULL ʤΤǡץ¦
-    Ǥä, ŬڤʥХåؿꤷPreedit ƥ,
-    Status ƥȤ桼ɽǤ褦ˤʤƤϤʤʤ
+     MInputDriver::open_im ()  m17n ǡ١椫饿 
+    \< #Minput_method, $LANGUAGE, $NAME\> ˹פϥ᥽åɤõ
+    ɤ롣
 
-    ؿ M17N_INIT () ѿ @c minput_driver 򤳤Υɥ饤ФؤΥݥ
+     MInputDriver::callback_list ()  @c NULL ʤΤǡץ
+    ¦Ǥä, ŬڤʥХåؿ plist ꤷʤ
+    Ϥʤʤʤȡpreedit ƥȤʤɤΥեɥХå
+    桼ɽʤ
+
+    ޥ M17N_INIT () ѿ #minput_driver 򤳤Υɥ饤ФؤΥݥ
     ꤷƤϥ᥽åɤΥɥ饤ФȤ褦ˤ롣
 
     äơ@c minput_driver ǥեͤΤޤޤǤСminput_ 
-    ǻϤޤʲδؿΥɥ饤Ф˰¸ $ARG Ϥɤ̵뤵
-    롣  */
+    ǻϤޤؿΥɥ饤Ф˰¸ $ARG Ϥ٤̵뤵롣  */
 
 MInputDriver minput_default_driver;
 /*=*/
 
 /***en
-    @brief The input driver for internal input methods.
+    @brief The driver for internal input methods.
 
     The variable #minput_driver is a pointer to the input method
     driver that is used by internal input methods.  The macro
     M17N_INIT () initializes it to a pointer to #minput_default_driver
-    (if <m17n.h> is included) or to #minput_gui_driver (if
-    <m17n-gui.h> is included).  */ 
+    (if <m17n<EM></EM>.h> is included) or to #minput_gui_driver (if
+    <m17n-gui<EM></EM>.h> is included).  */ 
+/***ja
+    @brief ϥ᥽åѥɥ饤.
+
+    ѿ #minput_driver ϥ᥽åɤˤäƻѤƤϥ
+    åɥɥ饤ФؤΥݥ󥿤Ǥ롣ޥ M17N_INIT () ϤΥݥ
+     #minput_default_driver (<m17n<EM></EM>.h> ޤޤ) ޤ
+    #minput_gui_driver ( <m17n-gui<EM></EM>.h> ޤޤ) ˽
+    롣  */ 
 
 MInputDriver *minput_driver;
 
@@ -1960,49 +2056,43 @@ MSymbol Minput_driver;
     language $LANGUAGE and name $NAME, and returns a pointer to the
     input method object newly allocated.
 
-    This function at first decides an input driver for the input
-    method as below.
+    This function at first decides an driver for the input method as
+    below.
 
-    If $LANGUAGE is not #Mnil, an input driver pointed by the variable
+    If $LANGUAGE is not #Mnil, the driver pointed by the variable
     #minput_driver is used.
 
     If $LANGUAGE is #Mnil and $NAME has #Minput_driver property, the
-    input driver pointed to by the property value is used to open the
-    input method.  If $NAME has no such property, @c NULL is returned.
+    driver pointed to by the property value is used to open the input
+    method.  If $NAME has no such property, @c NULL is returned.
 
-    Then, the member MInputDriver::open_im () of the input driver is
+    Then, the member MInputDriver::open_im () of the driver is
     called.  
 
     $ARG is set in the member @c arg of the structure MInputMethod so
-    that the input driver can refer to it.  */
+    that the driver can refer to it.  */
 
 /***ja
-    @brief ϥ᥽åɤ򥪡ץ󤹤     
+    @brief ϥ᥽åɤ򥪡ץ󤹤.
 
-    ؿ mim_open () ϸ $LANGUAGE ̾ $NAME Ŭ礹ϥ᥽
-    ɤ򥪡ץ󤷡ο˳Ƥ줿ϥ᥽åɤؤΥݥ󥿤֤
+    ؿ minput_open_im () ϸ $LANGUAGE ̾ $NAME ˹פ
+    ϥ᥽åɤ򥪡ץ󤷡˳Ƥ줿ϥ᥽åɥ֥
+    ȤؤΥݥ󥿤֤
+    
+    δؿϡޤϥ᥽åѤΥɥ饤ФʲΤ褦ˤƷꤹ롣
 
-    $LANGUAGE  #Mnil ǤʤСm17n ١椫 \<@c
-    Minput_method, $LANGUAGE, $NAME \> ȤŬ礹ϥ᥽
-    ɤõĤäϡѿ #minput_driver ǥݥȤ
-    ϥɥ饤ФѤƤϥ᥽åɤ򥪡ץ󤹤롣Ĥ
-    䥪ץǤʤäˤ @c NULL ֤
+    $LANGUAGE  #Mnil ǤʤСѿ #minput_driver ǻؤƤ
+    ɥ饤ФѤ롣
 
-    ѿ #minput_driver ϡ#minput_default_driver  @c
-    minput_gui_driver Τɤ餫ݥȤƤ롣Ԥ CUI ѤǤ
-    ꡢؿm17n_initialize () Ƥ֤Ȥˤä #minput_driver  
-    #minput_default_driver ݥȤ褦ˤʤ롣Ԥ GUI Ѥ
-    ꡢؿ m17n_initialize_win () ˤäƥݥȤ롣ܺ٤ˤ
-    ƤϤѿΥɥȤ򻲾ȤΤȡ
+    $LANGUAGE  #Mnil Ǥꡢ$NAME  #Minput_driver ץѥƥ
+    ľˤϡΥץѥƥͤǻؤƤϥɥ饤ФѤ
+    ϥ᥽åɤ򥪡ץ󤹤롣$NAME ˤΤ褦ʥץѥƥ̵ä
+     @c NULL ֤
 
-    $LANGUAGE  #Mnil Ǥꡢ$NAME  #Minput_driver 򥭡Ȥ
-    ץѥƥľˤϡΥץѥƥͤǥݥȤƤ
-    ϥɥ饤ФѤϥ᥽åɤ򥪡ץ󤹤롣$NAME ˤΤ褦
-    ץѥƥ̵ä䥪ץǤʤäˤ @c NULL 
-    ֤
+    ǡɥ饤ФΥ MInputDriver::open_im () ƤФ롣
 
-    $ARG ϡϥɥ饤ФȤǤ褦ˡ¤ MInputMethod Υ
-     @c arg ˥åȤ롣
+    $ARG ϡɥ饤ФȤǤ褦ˡ¤ MInputMethod Υ 
+    @c arg ꤵ롣
 
     @latexonly \IPAlabel{minput_open} @endlatexonly
 
@@ -2045,7 +2135,7 @@ minput_open_im (MSymbol language, MSymbol name, void *arg)
     must have been created by minput_open_im ().  */
 
 /***ja
-    @brief ϥ᥽åɤ򥯥
+    @brief ϥ᥽åɤ򥯥.
 
     ؿ minput_close_im () ϡϥ᥽å $IM 򥯥롣
     ϥ᥽å $IM  minput_open_im () ˤäƺ줿ΤǤʤ
@@ -2074,14 +2164,18 @@ minput_close_im (MInputMethod *im)
     returns a pointer to it.  Otherwise it returns @c NULL.  */
 
 /***ja
-    @brief ϥƥȤ
+    @brief ϥƥȤ.
 
     ؿ minput_create_ic () ϥ᥽å $IM бϥƥ
-    ȥ֥Ȥ롣
+    ȥ֥Ȥ #Minput_preedit_start,
+    #Minput_status_start, #Minput_status_draw б륳Хå
+    򤳤ν˸Ƥ֡
 
     @return
-    硢minput_create_ic () ϥƥ
-    ؤΥݥ󥿤֤Ԥ @c NULL ֤  */
+
+    ϥƥȤ줿硢minput_create_ic () Ϥϥ
+    ƥȤؤΥݥ󥿤֤Ԥ @c NULL ֤
+      */
 
 MInputContext *
 minput_create_ic (MInputMethod *im, void *arg)
@@ -2124,14 +2218,17 @@ minput_create_ic (MInputMethod *im, void *arg)
     The minput_destroy_ic () function destroys the input context $IC,
     which must have been created by minput_create_ic ().  It calls
     callback functions corresponding to #Minput_preedit_done,
-    #Minput_status_done, and #Mcandidate_done in this order.  */
+    #Minput_status_done, and #Minput_candidates_done in this order.  */
 
 /***ja
-    @brief ϥƥȤ˲
+    @brief ϥƥȤ˲.
 
     ؿ minput_destroy_ic () ϡϥƥ $IC ˲롣
     ϥƥȤ minput_create_ic () ˤäƺ줿ΤǤ
-    Фʤʤ  */
+    Фʤʤδؿ#Minput_preedit_done,
+    #Minput_status_done, #Minput_candidates_done б륳Хåؿ
+    ν˸Ƥ֡
+  */
 
 void
 minput_destroy_ic (MInputContext *ic)
@@ -2156,26 +2253,29 @@ minput_destroy_ic (MInputContext *ic)
 
     The minput_filter () function filters input key $KEY according to
     input context $IC, and calls callback functions corresponding to
-    #Minput_preedit_draw, #Minput_status_draw, and #Mcandidate_draw if
-    the preedit text, the status, and the current candidate are
-    changed respectively.
+    #Minput_preedit_draw, #Minput_status_draw, and
+    #Minput_candidates_draw if the preedit text, the status, and the
+    current candidate are changed respectively.
 
     @return
     If $KEY is filtered out, this function returns 1.  In that case,
     the caller should discard the key.  Otherwise, it returns 0, and
     the caller should handle the key, for instance, by calling the
-    function minput_lookup () with the same $KEY.  */
+    function minput_lookup () with the same key.  */
 
 /***ja
-    @brief ϥΥե륿󥰤򤹤
+    @brief ϥե륿.
 
     ؿ minput_filter () ϥ $KEY ϥƥ $IC ˱
-    ƥե륿󥰤롣
+    ƥե륿preedit ƥȡơǤθ䤬Ѳ
+    ݤˤϤ줾#Minput_preedit_draw, #Minput_status_draw,
+    #Minput_candidates_draw б륳ХåؿƤ֡
 
-    ϥƥ $IC ϥ᥽åɤϥե륿С
-    δؿ 1 ֤ξƤӽФ¦ϤΥΤƤ٤Ǥ롣
-    Ǥʤ 0 ֤ƤӽФ¦ȤƱ $KEY Ǵؿ 
-    minput_lookup () Ƥ֤ʤɤơΥ롣
+    @return 
+    $KEY ե륿Сδؿ 1 ֤ξƤ
+    Ф¦ϤΥΤƤ٤Ǥ롣Ǥʤ 0 ֤Ƥ
+    Ф¦ϡȤƱǴؿ minput_lookup () Ƥ֤ʤɤơ
+    Υ롣
 
     @latexonly \IPAlabel{minput_filter} @endlatexonly
 */
@@ -2198,7 +2298,6 @@ minput_filter (MInputContext *ic, MSymbol key, void *arg)
 	minput__callback (ic, Minput_status_draw);
       if (ic->candidates_changed)
 	minput__callback (ic, Minput_candidates_draw);
-      ic->preedit_changed = ic->status_changed = ic->candidates_changed = 0;
     }
 
   return ret;
@@ -2216,7 +2315,7 @@ minput_filter (MInputContext *ic, MSymbol key, void *arg)
     If a text was produced by the input method, it is concatenated
     to M-text $MT.
 
-    This function calls #MInputDriver::lookup.
+    This function calls #MInputDriver::lookup .
 
     @return
     If $KEY was correctly handled by the input method, this function
@@ -2224,23 +2323,21 @@ minput_filter (MInputContext *ic, MSymbol key, void *arg)
     may be produced in $MT.  */
 
 /***ja
-    @brief ϥ᥽åɤäƥȤγ
+    @brief ϥƥΥƥȤθ.
 
-    ؿ minput_lookup () ϥƥ $IC ΥƥȤ
+    ؿ minput_lookup () ϥƥ $IC ΥƥȤ򸡺
     롣$KEY ϴؿminput_filter () ؤľθƤӽФѤ줿
     ƱǤʤƤϤʤʤ
 
-    ƥȤϥ᥽åɤˤäƤС$IC->produced 
-    Ƥ롣
+    ƥȤϥ᥽åɤˤäƤСƥȤ M-text
+    $MT Ϣ뤵롣
 
-     δؿϡX ɥ <tt>XLookupString ()</tt> 
-    <tt>XmbLookupString ()</tt> <tt>XwcLookupString ()</tt> б
-    롣
+    δؿϡ#MInputDriver::lookup Ƥ֡
 
-    @return
-    $KEY ϥ᥽åɤˤäŬڤ˽ǤƤСδؿ 0 
-    ֤Ǥʤ -1 ֤ξǤ$IC->produced ˲餫
-    ΥƥȤƤ뤳Ȥ롣
+    @return 
+    $KEY ϥ᥽åɤˤäŬڤ˽ǤСδؿ 0 ֤
+    Ǥʤ -1 ֤ξǤ $MT ˲餫ΥƥȤ
+    Ƥ뤳Ȥ롣
 
     @latexonly \IPAlabel{minput_lookup} @endlatexonly  */
 
@@ -2255,34 +2352,40 @@ minput_lookup (MInputContext *ic, MSymbol key, void *arg, MText *mt)
     @brief Set the spot of the input context.
 
     The minput_set_spot () function set the spot of input context $IC
-    to coordinate ($X, $Y) with the height $ASCENT and $DESCENT.
+    to coordinate ($X, $Y ) with the height $ASCENT and $DESCENT .
+    The semantics of these values depend on the input method driver.
     $FONTSIZE specfies the fontsize of a preedit text in 1/10 point.
-    The semantics of these values depend on the input driver.
 
-    For instance, an input driver designed to work in CUI environment
-    may use $X and $Y as column and row numbers, and ignore $ASCENT
-    and $DESCENT.  An input driver designed to work on a window system
-    may treat $X and $Y as pixel offsets relative to the origin of the
+    For instance, an driver designed to work in CUI environment may
+    use $X and $Y as column and row numbers, and ignore $ASCENT and
+    $DESCENT .  An driver designed to work on a window system may
+    treat $X and $Y as pixel offsets relative to the origin of the
     client window, and treat $ASCENT and $DESCENT as ascent and
-    descent pixels of a line at ($X . $Y).
+    descent pixels of a line at ($X . $Y ).
 
     $MT and $POS is an M-text and a character position at the spot.
-    $MT may be NULL, in which case, the input method can't get
+    $MT may be @c NULL, in which case, the input method cannot get
     information about the text around the spot.  */
 
 /***ja
     @brief ϥƥȤΥݥåȤꤹ
 
     ؿ minput_set_spot () ϡϥƥ $IC ΥݥåȤ򡢺
-    ɸ ($X, $Y) ⤵ $ASCENT$DESCENT ꤹ롣ͤΰ
-    ̣ϥɥ饤Ф˰¸롣
+    ɸ ($X, $Y ) ⤵ $ASCENT $DESCENT ꤹ롣 ͤ
+    ̣ϥ᥽åɥɥ饤Ф˰¸롣$FONTSIZE preedit ƥ
+    Υեȥ 1/10 ݥñ̤ǻꤹ롣
 
-    Ȥ CUI Ķưϥɥ饤Ф $X, $Y 򤽤줾ȹ
-    ֹȤѤ$ASCENT$DESCENT ̵뤹뤫⤷ʤ ޤ
-    ɥƥѤϥɥ饤Ф $X,$Y 򥯥饤ȥ
+    Ȥ CUI Ķưɥ饤Ф $X  $Y 򤽤줾ȹԤ
+    ֹȤѤ$ASCENT  $DESCENT ̵뤹뤫⤷ʤ ޤ
+    ɥƥѤΥɥ饤Ф $X  $Y 򥯥饤ȥɥ
     θΥեåȤԥñ̤ɽΤȤư
-    $ASCENT  $DESCENT  ($X . $Y) ΥȤȥǥȤԥ
-    ñ̤ɽΤȤư⤷ʤ  */
+    $ASCENT  $DESCENT  ($X . $Y ) ΥȤȥǥȤԥ
+    ñ̤ɽΤȤư⤷ʤ
+
+    $MT  $POS ϤΥݥåȤ M-text ʸ֤Ǥ롣$MT  @c
+    NULL Ǥ褯ξˤϥ᥽åɤϥݥåȼդΥƥȤ
+    ؤ뤳ȤǤʤ
+    */
 
 void
 minput_set_spot (MInputContext *ic, int x, int y,
@@ -2306,6 +2409,12 @@ minput_set_spot (MInputContext *ic, int x, int y,
 
     The minput_toggle () function toggles the input method associated
     with the input context $IC.  */
+/***ja
+    @brief ϥ᥽åɤؤ.
+
+    ؿ minput_toggle () ϥƥ $IC бդ줿
+    ᥽åɤȥ뤹롣
+    */
 
 void
 minput_toggle (MInputContext *ic)
@@ -2315,6 +2424,32 @@ minput_toggle (MInputContext *ic)
   ic->active = ! ic->active;
 }
 
+/***en
+    @brief Reset an input context.
+
+    The minput_reset_ic () function resets the input context $IC by
+    calling a callback functions corresponding to #Minput_reset.  It
+    actually shifts the state to the initial one, and thus the current
+    preediting text (if any) is committed.  If necessary, a program
+    can extract that text by calling minput_lookup () just after the
+    call of minput_reset_ic ().  In that case, the arguments @c KEY
+    and @c ARG of minput_lookup () are ignored.  */
+/***ja
+    @brief ϥƥȤꥻåȤ.
+
+    ؿ minput_reset_ic ()  #Minput_reset б륳Хåؿ
+    Ƥ֤Ȥˤäϥƥ $IC ꥻåȤ롣ϼºݤ
+    ϥ᥽åɤ֤˥եȤ롣äơ⤷
+    ƥȤСϥߥåȤ롣ɬפʤ饢ץꥱ
+     minput_lookup () ɤǤΥƥȤФ롣
+    ݡminput_lookup () ΰ @c KEY  @c ARG ̵뤵롣 */
+void
+minput_reset_ic (MInputContext *ic)
+{
+  if (ic->im->driver.callback_list)
+    minput__callback (ic, Minput_reset);
+}
+
 
 /*** @} */
 /*=*/
@@ -2324,7 +2459,7 @@ minput_toggle (MInputContext *ic)
 /*=*/
 
 /***en
-    @brief Dump an input method
+    @brief Dump an input method.
 
     The mdebug_dump_im () function prints the input method $IM in a
     human readable way to the stderr.  $INDENT specifies how many
@@ -2332,6 +2467,14 @@ minput_toggle (MInputContext *ic)
 
     @return
     This function returns $IM.  */
+/***ja
+    @brief ϥ᥽åɤפ.
+
+    ؿ mdebug_dump_im () ϥ᥽å $IM  stderr ˿ʹ֤˲ɤ
+    ǰ롣$INDENT ϣܰʹߤΥǥȤꤹ롣
+
+    @return
+    δؿ $IM ֤  */
 
 MInputMethod *
 mdebug_dump_im (MInputMethod *im, int indent)
