@@ -22,7 +22,7 @@
 
 /***en
     @addtogroup m17nIntro
-    @brief Introduction to the m17n library
+    @brief Introduction to the m17n library.
 
     <em>API LEVELS</em>
 
@@ -31,34 +31,51 @@
     <ol>
     <li> CORE API
 
-    It provides basic modules to handle M-texts.  They don't require
+    It provides basic modules to handle M-texts.  They do not require
     the m17n database.  To use this API, an application program must
-    include <m17n-core.h> and be linked by -lm17n-core.
+    include <m17n-core<EM></EM>.h> and be linked by -lm17n-core.
 
     <li> SHELL API
 
-    It provides modules that utilize the m17n database.  They load
-    various kinds of data from the database on demand.  To use this
-    API, an application program must include <m17n.h> and be linked by
-    -lm17n-core -lm17n.  With that, CORE API is also available.
+    It provides modules that utilize the m17n database (code
+    conversion, character property, etc).  They load various kinds of
+    data from the database on demand.  To use this API, an application
+    program must include <m17n<EM></EM>.h> and be linked by
+    -lm17n-core -lm17n.
+
+    When you use this API, CORE API is also available.
 
     <li> GUI API
 
     It provides GUI modules such as drawing and inputting M-texts on a
-    window system.  The API itself is independent on a window system,
-    but the m17n library must be configured to use a specific window
-    system.  Currently, we support only the X Window System.  To use
-    this API, an application program must include <m17n-gui.h> and
-    <m17n-X.h>, and be linked by -lm17n-core -lm17n -lm17n-X.  With
-    that, CORE and SHELL APIs are also available.
+    graphic device.  The API itself is independent on a graphic device
+    but most functions require an argument MFrame which is created for
+    a specific type of graphic device.  Currently, a null device, X
+    Window System, and an image date (gdImagePtr) of GD library are
+    supported as a graphic device.
+
+    On a frame of the null device, you can't draw text nor use an
+    input method (but, for instance, the funciton mdraw_glyph_list ()
+    is available).
+
+    On a frame of the X Window System, you can use the whole GUI API.
+
+    On a frame of GD library, you can use all drawing API but can't
+    use input method.
+
+    To use this device, an application program must include
+    <m17n-gui<EM></EM>.h> and be linked by -lm17n-core -lm17n
+    -lm17n-gui.
+
+    When you use this API, CORE and SHELL API are also available.
 
     <li> MISC API
 
     It provides miscellaneous functions to support error handling and
-    debugging.  This API can't be used by itself, but with one or more
-    APIs listed above.  To use the API, an application program must
-    include <m17n-misc.h> in addition to one of <m17n-core.h>,
-    <m17n.h>, and <m17n-gui.h>.
+    debugging.  This API cannot be used by itself, but with one or
+    more APIs listed above.  To use the API, an application program
+    must include <m17n-misc<EM></EM>.h> in addition to one of a header
+    file described above.
 
     </ol>
 
@@ -72,12 +89,13 @@
     <li> @c M17NDIR
 
     Name of a directory that contains data of the m17n database.  See
-    @ref m17nDatabase for the detail.
+    @ref m17nDatabase for more details.
 
     <li> @c MDEBUG_XXXX
 
     Environment variables whose name start by "MDEBUG_" controls
-    printing of debug information.  See @ref m17nDebug for the detail.
+    printing of debug information.  See @ref m17nDebug for more
+    details.
 
     </ul>
 
@@ -103,10 +121,10 @@
     <li> symbol variables -- Mname
 
     Variables of type MSymbol start with 'M' followed by their names
-    (e.g. Mlanguage (name is "langauge"), Miso_2022 (name is
+    (e.g. Mlanguage (name is "language"), Miso_2022 (name is
     "iso-2022").
 
-    <li> macross -- MOBJECT_XXX
+    <li> macros -- MOBJECT_XXX
 
     They start by 'M' followed by upper case object names.
 
@@ -121,25 +139,116 @@
 
 /***ja
     @addtogroup m17nIntro
+    @brief m17n 饤֥ ȥ.
 
-    @e m17n饤֥  C ¿饤֥Ǥ롣
-    Υ饤֥ϡ¿ʸ򰷤ɬפʰʲδܵǽ򥢥
-    ꥱץ󶡤롣
+    @em APIΥ٥
 
-    @li ¿ƥѥ֥ȤȤƤι¤ @e M-text
+    m17n 饤֥ API ϰʲΣʬवƤ롣
 
-    @li M-text 򰷤¿δؿޥ
+    <ol>
+    <li>  API
 
-    @li ΥեޥåȤǥ󥳡ɤ줿ƥȤ M-text ֤Ѵ
-    Ԥʤǥ󥳡
+    M-text 򰷤δŪʥ⥸塼󶡤롣ʬ API 
+    m17n ǡ١ɬפȤʤѤ뤿ˤϡץꥱ
+    ץ<m17n-core<EM></EM>.h>  include  -lm17n-core ǥ
+    󥯤ʤƤϤʤʤ
 
-    @li Unicode Ƥʸ˲äƱ Unicode ʸ
-    ʸ
+    <li>  API
 
-    @li ʸξΨɤǼ빽¤ @e ʸơ֥ (CharTable)
+    m17n ǡ١Ѥ⥸塼ʥѴʸץѥƥ
+    ˤ󶡤롣⥸塼ϥǡ١ɬפ˱¿ͤʥǡ
+    ɤ롣Ѥ뤿ˤϡץꥱץ
+    <m17n<EM></EM>.h>  include  -lm17n-core -lm17n ǥ󥯤
+    ƤϤʤʤ
 
-    @li ץM-textΥɥƥǤɽϥƥࡣ
-    ʤΤ API  m17n-win.h ˴ޤޤƤ롣  */
+     API Ѥݤˤϥ API ѤǤ롣
+
+    <li> GUI API
+
+    M-texts 򥰥եåǥХɽϤꤹȤä
+    եå桼󥿥ե⥸塼󶡤롣API Τϥ
+    եåǥХȤΩǤ뤬¿δؿΥեå
+    ǥХ˺줿 MFrame ˼롣Ǥϡ̥ǥХ
+    X ɥƥࡢGD 饤֥Υ᡼gdImagePtrˡ
+    եåǥХȤƥݡȤƤ롣
+
+    ̥ǥХǤɽϤǤʤʤȤ
+    mdraw_glyph_list () ʤɤδؿϻѲǽˡ
+
+    X ɥƥǤϤ٤Ƥ GUI API Ȥ롣
+
+    GD 饤֥Υ᡼ǤϡѤ API Ϥ٤ƻȤ뤬Ϥ
+    Ǥʤ
+
+    ΥǥХѤ뤿ˤϡץꥱץ
+    <m17n-gui<EM></EM>.h>  include -lm17n-core -lm17n -lm17n-gui
+    ǥ󥯤ʤƤϤʤʤ
+
+     API Ѥݤˤϡ API ȥ API ѤǤ롣
+
+    <li> ¾ API
+
+    顼ǥХåѤΤ¾δؿ󶡤롣 API Ϥ
+    ǤϻѤǤ嵭¾ΤΤȶ˻ȤѤ뤿ˤϡ嵭
+    줫 include ե˲äơ <m17n-misc<EM></EM>.h> 
+    include ʤƤϤʤʤ
+
+    </ol>
+
+    @ref m17n-config "m17n-config(1)" ⻲ȡ
+
+    @em Ķѿ
+
+    m17n 饤֥ϼδĶѿθ롣
+
+    <ul>
+    <li> @c M17NDIR
+
+    m17n ǡ١Υǡޤǥ쥯ȥ̾ܺ٤@ref
+    m17nDatabase ȡ
+
+    <li> @c MDEBUG_XXXX
+
+    "MDEBUG_" ǻϤޤ̾ĴĶѿϥǥХå椹롣ܺ
+    @ref m17nDebug ȡ
+
+    </ul>
+
+    @em API @em ̿̾§ ޥ
+
+    饤֥ϡؿѿޥ export 롣'm' 
+     'M' ΤȤ˥֥̾("symbol"  "plist" ʤɡ 
+    "mtext" ֥ȤϤϤ 'm' νʣ򤱤뤿 "text" 
+    롣) ޤϥ⥸塼̾(draw, input ʤ) ³ΤǤ롣
+    
+    <ul>
+
+    <li> ؿ -- mobject () ޤ mobject_xxx ()
+
+    'm' ΤȤ˾ʸǥ֥̾³ȤСmsymbol (),
+     mtext_ref_char (), mdraw_text () ʤɡ
+
+    <li> ܥǤʤѿ -- mobject,  ޤ mobject_xxx
+    
+    ؿƱ̿̾§˽(Ȥ  mface_large)
+
+    <li> ܥѿ -- Mname
+
+    MSymbol ѿϡ'M' θ̾³Ȥ Mlanguage (̾ 
+    "language"), Miso_2022 (̾"iso-2022")ʤɡ
+
+    <li> ޥ -- MOBJECT_XXX
+
+    'M' θʸǥ֥̾³
+
+    <li>  -- MObject ޤ MObjectXxx
+
+    'M' θʸǻϤƥ֥̾³ (Ȥ
+    MConverter, MInputDriver)
+
+    </ul>
+    
+    */
 /*=*/
 /*** @{ */
 #ifdef FOR_DOXYGEN
@@ -185,7 +294,7 @@
 
 /***ja
     ޥ #M17NLIB_VERSION_NAME  m17n 饤֥ΥС̾
-    C-string ηͿ롣  */
+    ʸȤͿ롣  */
 
 #define M17NLIB_VERSION_NAME
 
@@ -194,20 +303,33 @@
 /***en
     @brief Initialize the m17n library.
 
-    The macro M17N_INIT () initializes the m17n library.  This
-    function must be called before any m17n functions are used.
+    The macro M17N_INIT () initializes the m17n library.  This macro
+    must be called before any m17n functions are used.
 
-    If the initialization was successful, the external variable @c
-    merror_code is set to 0.  Otherwise it is set to -1.  */
+    It is safe to call this macro multiple times, but in that case,
+    the macro M17N_FINI () must be called the same times to free the
+    memory.
+
+    If the initialization was successful, the external variable
+    #merror_code is set to 0.  Otherwise it is set to -1.
+
+    @seealso
+    M17N_FINI (), m17n_status ()  */
 
 /***ja
-    @brief m17n 饤֥ν
+    @brief m17n 饤֥.
 
-    ޥ M17N_INIT ()  m17n 饤֥롣m17n 饤֥
-    ȤȤϡǽˤδؿƤФʤƤϤʤʤ
+    ޥ M17N_INIT ()  m17n 饤֥롣m17n δؿ
+    ѤˡΥޥޤƤФʤƤϤʤʤ
+    
+    ΥޥʣƤǤǤ뤬ξ뤿
+    ˥ޥ M17N_FINI () ƱƤɬפ롣
 
-    ѿ #merror_code ϡ 0 ˡǤʤ -1 
-    ꤵ롣  */
+    ѿ #merror_code ϡ 0 ˡǤʤ 
+    -1 ꤵ롣  
+
+    @seealso
+    M17N_FINI (), m17n_status ()  */
 
 #define M17N_INIT()
 
@@ -217,12 +339,27 @@
     @brief Finalize the m17n library.
 
     The macro M17N_FINI () finalizes the m17n library.  It frees all the
-    memory area used by the m17n library.  Once this function is
+    memory area used by the m17n library.  Once this macro is
     called, no m17n functions should be used until the
-    macro M17N_INIT () is called again.  */
+    macro M17N_INIT () is called again.
 
+    If the macro M17N_INIT () was called N times, the Nth call of this
+    macro actually free the memory. 
+
+    @seealso
+    M17N_INIT (), m17n_status ()  */
 /***ja
-    @brief m17n 饤֥νλ  */
+    @brief m17n 饤֥λ. 
+
+    ޥ M17N_FINI ()  m17n 饤֥λ롣m17n 饤֥
+    ȤäƤΥΰϲ롣٤ΥޥƤФ줿顢
+    ޥ M17N_INIT () ٸƤФޤ m17n ؿϻȤ٤Ǥʤ
+
+    ޥ M17N_INIT ()  N ƤФƤˤϡΥޥ N 
+    ƤФƽƥ꤬롣
+
+    @seealso
+    M17N_INIT (), m17n_status ()  */
 
 #define M17N_FINI()
 #endif /* FOR_DOXYGEN */
@@ -245,8 +382,6 @@
 #include "m17n-misc.h"
 #include "internal.h"
 
-static int core_initialized;
-
 static void
 default_error_handler (enum MErrorCode err)
 {
@@ -261,6 +396,10 @@ static int report_header_printed;
 
 /* Internal API */
 
+int m17n__core_initialized;
+int m17n__shell_initialized;
+int m17n__gui_initialized;
+
 void
 mdebug__report_object (char *name, M17NObjectArray *array)
 {
@@ -327,6 +466,33 @@ mdebug__print_time ()
   } while (0)
 
 
+void
+mdebug__register_object (M17NObjectArray *array, void *object)
+{
+  if (array->used == 0)
+    MLIST_INIT1 (array, objects, 256);
+  array->count++;
+  MLIST_APPEND1 (array, objects, object, MERROR_OBJECT);
+}
+
+void
+mdebug__unregister_object (M17NObjectArray *array, void *object)
+{
+  array->count--;
+  if (array->count >= 0)
+    {
+      int i = 0;
+
+      while (i < array->used && array->objects[i] != object) i++;
+      if (i < array->used)
+	array->objects[i] = NULL;
+      else
+	mdebug_hook ();
+    }
+  else									\
+    mdebug_hook ();
+}
+
 
 /* External API */
 
@@ -338,7 +504,7 @@ m17n_init_core (void)
 {
   int mdebug_mask = MDEBUG_INIT;
 
-  if (core_initialized)
+  if (m17n__core_initialized++)
     return;
 
   merror_code = MERROR_NONE;
@@ -374,7 +540,6 @@ m17n_init_core (void)
 
   mdatabase__finder = NULL;
   mdatabase__loader = NULL;
-  core_initialized = 1;
 
  err:
   MDEBUG_POP_TIME ();
@@ -387,25 +552,25 @@ m17n_fini_core (void)
 {
   int mdebug_mask = MDEBUG_FINI;
 
-  if (core_initialized)
-    {
-      MDEBUG_PUSH_TIME ();
-      MDEBUG_PUSH_TIME ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize chartable module."));
-      mchartable__fini ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize textprop module."));
-      mtext__prop_fini ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize mtext module."));
-      mtext__fini ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize symbol module."));
-      msymbol__fini ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize plist module."));
-      mplist__fini ();
-      core_initialized = 0;
-      MDEBUG_POP_TIME ();
-      MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the core modules."));
-      MDEBUG_POP_TIME ();
-    }
+  if (m17n__core_initialized == 0
+      || --m17n__core_initialized > 0)
+    return;
+
+  MDEBUG_PUSH_TIME ();
+  MDEBUG_PUSH_TIME ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize chartable module."));
+  mchartable__fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize textprop module."));
+  mtext__prop_fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize mtext module."));
+  mtext__fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize symbol module."));
+  msymbol__fini ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize plist module."));
+  mplist__fini ();
+  MDEBUG_POP_TIME ();
+  MDEBUG_PRINT_TIME ("FINI", (stderr, " to finalize the core modules."));
+  MDEBUG_POP_TIME ();
   report_header_printed = 0;
 }
 
@@ -413,6 +578,41 @@ m17n_fini_core (void)
 #endif /* !FOR_DOXYGEN || DOXYGEN_INTERNAL_MODULE */
 /*=*/
 
+/*** @addtogroup m17nIntro */
+
+/*** @{  */
+/*=*/
+
+/***en
+    @brief Report which part of the m17n library is initialized.
+
+    The m17n_status () function returns one of these values depending
+    on which part of the m17n library is initialized:
+
+	#M17N_NOT_INITIALIZED, #M17N_CORE_INITIALIZED,
+	#M17N_SHELL_INITIALIZED, #M17N_GUI_INITIALIZED  */
+
+/***ja
+    @brief m17n 饤֥Τɤʬ줿𤹤.
+
+    ؿ m17n_status () m17n 饤֥Τɤʬ줿
+    ơʲͤΤ줫֤
+
+	#M17N_NOT_INITIALIZED, #M17N_CORE_INITIALIZED,
+	#M17N_SHELL_INITIALIZED, #M17N_GUI_INITIALIZED  */
+
+enum M17NStatus
+m17n_status (void)
+{
+  return (m17n__gui_initialized ? M17N_GUI_INITIALIZED
+	  : m17n__shell_initialized ? M17N_SHELL_INITIALIZED
+	  : m17n__core_initialized ? M17N_CORE_INITIALIZED
+	  : M17N_NOT_INITIALIZED);
+}
+
+/*** @} */
+
+/*=*/
 /***en
     @addtogroup m17nObject
     @brief Managed objects are objects managed by the reference count.
@@ -432,7 +632,25 @@ m17n_fini_core (void)
 
     In addition to the predefined managed object types, users can
     define their own managed object types.  See the documentation of
-    the m17n_object () for the details.  */
+    the m17n_object () for more details.  */
+/***ja
+    @addtogroup m17nObject
+    @brief ֥ȤȤϻȿˤäƴƤ륪֥ȤǤ.
+
+    m17n ֥ȤΤ뷿ΤΤϡȿˤäƴƤ롣
+    Υ֥Ȥ @e ֥ ȸƤФ롣
+    Ǥλȿ 0 ˽Ƥ롣ؿ m17n_object_ref () 
+    ϴ֥Ȥλȿ 1 䤷ؿm17n_object_unref () 
+     1 餹ȿ 0 ˤʤä֥ȤϼưŪ˲
+    롣
+
+    ǤץѥƥϡͤȤƴ֥Ȥ
+    롣ؿ msymbol_put ()  mplist_put () ʤɤϤΥץѥƥ
+    ̰롣
+
+    Ѥߴ֥ȥפ¾ˡ桼ɬפʴ֥
+    ȥפʬ뤳ȤǤ롣ܺ٤m17n_object () 
+    򻲾ȡ  */
 
 /*** @{  */
 /*=*/
@@ -455,6 +673,24 @@ m17n_fini_core (void)
     @errors
     This function never fails.  */
 
+/***ja
+    @brief ֥ȤƤ.
+
+    ؿ m17n_object () $SIZE ХȤο֥Ȥ
+    ơλȿ 1 Ȥ롣 $FREER ϻȿ 0 ˤʤäݤˤ
+    ֥Ȥ뤿ѤؿǤ롣$FREER  NULL
+    С֥Ȥϴؿ free () ˤäƲ롣
+
+    Ƥ줿֥ƬΥХȤϡ#M17NObjectHead 롣 
+    ΰ m17n 饤֥꤬ѤΤǡץꥱץ
+    ϿƤϤʤʤ
+
+    @return
+    δؿϿƤ줿֥Ȥ֤
+
+    @errors
+    δؿϼԤʤ    */
+
 #if EXAMPLE_CODE
 typedef struct
 {
@@ -509,6 +745,18 @@ m17n_object (int size, void (*freer) (void *))
 
     @errors
     This function never fails.  */
+/***ja
+    @brief ֥Ȥλȿ 1 䤹.
+
+    ؿ m17n_object_ref ()  $OBJECT ǻؤ֥Ȥ
+    ȿ 1 䤹
+
+    @return 
+    δؿϡ䤷ȿ 16 ӥåȤ̵(ʤ 
+    0x10000 ̤)ˤޤС֤Ǥʤ -1 ֤
+
+    @errors
+    δؿϼԤʤ    */
 
 int
 m17n_object_ref (void *object)
@@ -557,7 +805,20 @@ m17n_object_ref (void *object)
 
     @errors
     This function never fails.  */
+/***ja
+    @brief ֥Ȥλȿ 1 餹.
 
+    ؿ m17n_object_unref ()  $OBJECT ǻؤ֥
+    λȿ 1 餹ȿ 0 ˤʤС֥Ȥϲؿ
+    äƲ롣
+
+    @return 
+    δؿϡ餷ȿ 16 ӥåȤ̵(ʤ 
+    0x10000 ̤)ˤޤС֤Ǥʤ -1 ֤
+    Ĥޤꡢ0 ֤ä褿$OBJECT ϲƤ롣
+
+    @errors
+    δؿϼԤʤ    */
 int
 m17n_object_unref (void *object)
 {
@@ -607,19 +868,19 @@ m17n_object_unref (void *object)
 
     The first type is argument errors.  When a library function is
     called with invalid arguments, it returns a value that indicates
-    error and at the same time sets the external variable @e
-    merror_code to a non-zero integer.
+    error and at the same time sets the external variable #merror_code
+    to a non-zero integer.
 
     The second type is memory allocation errors.  When the required
     amount of memory is not available on the system, m17n library
     functions call a function pointed to by the external variable @c
     m17n_memory_full_handler.  The default value of the variable is a
     pointer to the default_error_handle () function, which just calls
-    exit ().  */
+    <tt> exit ()</tt>.  */
 
 /***ja
     @addtogroup m17nError 顼
-    @brief m17n 饤֥Υ顼
+    @brief m17n 饤֥Υ顼.
 
     m17n 饤֥δؿǤϡĤμΥ顼롣
 
@@ -629,15 +890,17 @@ m17n_object_unref (void *object)
 
     ⤦Ĥμϥƥ顼Ǥ롣ƥबɬפ̤Υ
     Ƥ뤳ȤǤʤ硢饤֥ؿϳѿ @c
-    m17n_memory_full_handler ؤؿƤ֡ǥեȤǤϡñ
-    <tt>exit ()</tt> Ƥ֤ȤˤʤäƤ롣
+    m17n_memory_full_handler ؤؿƤ֡ǥեȤǤϡؿ 
+    default_error_handle () ؤƤꡢδؿñ <tt>exit
+    ()</tt> Ƥ֡
 */
 
 /*** @{ */
 
 /*=*/
 
-/***en @brief External variable to hold error code of the m17n library
+/***en 
+    @brief External variable to hold error code of the m17n library.
 
     The external variable #merror_code holds an error code of the
     m17n library.  When a library function is called with an invalid
@@ -645,19 +908,21 @@ m17n_object_unref (void *object)
 
     This variable initially has the value 0.  */
 
-/***ja @brief m17n 饤֥Υ顼ɤݻ볰ѿ
+/***ja 
+    @brief m17n 饤֥Υ顼ɤݻ볰ѿ.
 
     ѿ #merror_code ϡm17n 饤֥Υ顼ɤݻ롣
-    饤֥ؿǤʤȤȤ˸ƤФ줿ݤˤϡ
-    ѿ @c enum #MErrorCode ΰĤ˥åȤ롣
+    饤֥ؿǤʤȤȤ˸ƤФ줿ݤˤϡѿ 
+    @c enum #MErrorCode ΰĤ˥åȤ롣
 
-    ѿνͤϣǤ롣  */
+    ѿνͤ 0 Ǥ롣  */
 
-enum MErrorCode merror_code;
+int merror_code;
 
 /*=*/
 
-/***en @brief Memory allocation error handler
+/***en 
+    @brief Memory allocation error handler.
 
     The external variable #m17n_memory_full_handler holds a pointer
     to the function to call when a library function failed to allocate
@@ -665,12 +930,13 @@ enum MErrorCode merror_code;
     function the error occurred.
 
     This variable initially points a function that simply calls the
-    exit () function with $ERR as an argument.
+    <tt>exit </tt>() function with $ERR as an argument.
 
     An application program that needs a different error handling can
     change this variable to point a proper function.  */
 
-/***ja @brief ƥ顼ϥɥ
+/***ja 
+    @brief ƥ顼ϥɥ.
 
     ѿ #m17n_memory_full_handler ϡ饤֥ؿ
     ˼Ԥݤ˸Ƥ֤٤ؿؤΥݥ󥿤Ǥ롣$ERR  @c enum
@@ -745,6 +1011,58 @@ void (*m17n_memory_full_handler) (enum MErrorCode err);
 
     </ul>
 */
+/***ja
+    @addtogroup m17nDebug
+    @brief m17n 饤֥桼ΤΥץǥХåݡ.
+
+    m17n 饤֥ϡΥ桼ʬΥץǥХå뤿
+    ˡʲεǽ򥵥ݡȤƤ롣
+
+    <ul>
+
+    <li> ޤޤʾΥץȤ椹Ķѿ
+
+    <ul>
+
+    <li> MDEBUG_INIT -- 1 ʤСM17N_INIT () ƤФ줿ǡ饤
+    ֥ν˴ؤץȤ롣
+
+    <li> MDEBUG_FINI -- 1 ʤСM17N_FINI () ƤФ줿ǡޤ
+    Ƥʤ֥ȤλȿץȤ롣
+
+    <li> MDEBUG_CHARSET -- 1 ʤСm17n ǡ١ɤ줿
+    ʸåȤˤĤƤξץȤ롣
+
+    <li> MDEBUG_CODING --  1 ʤСm17n ǡ١ɤ줿
+    ɷϤˤĤƤξץȤ롣
+
+    <li> MDEBUG_DATABASE -- 1 ʤСm17n ǡ١ɤ
+    ǡˤĤƤξץȤ롣
+
+    <li> MDEBUG_FONT -- 1 ʤС򤵤ƥץ󤵤줿եȤˤ
+    ƤξץȤ롣
+
+    <li> MDEBUG_FONT_FLT -- 1 ʤСFont Layout Table ΤɤΥޥ
+    ¹椫ˤĤƤΤץȤ롣
+
+    <li> MDEBUG_FONT_OTF -- 1 ʤСOpenType Layout Table Τɤ°
+    ¹椫ˤĤƤΤץȤ롣
+
+    <li> MDEBUG_INPUT -- 1 ʤС¹ϥ᥽åɤξ֤դ
+    ξץȤ롣
+
+    <li> MDEBUG_ALL -- 1 ʤС嵭٤Ƥѿ 1 ˤΤƱ
+    ̤ġ
+
+    </ul>
+
+    <li> Υ֥Ȥʹ֤˲ɤʷǥץȤؿܺ٤
+    ؿ mdebug_dump_XXXX () ȡ
+
+    <li> 顼ȯ˸ƤФեåؿmdebug_hook () ȡ
+
+    </ul>
+*/
 
 /*=*/
 /*** @{ */
@@ -754,8 +1072,15 @@ void (*m17n_memory_full_handler) (enum MErrorCode err);
     @brief Hook function called on an error.
 
     The mdebug_hook () function is called when an error happens.  It
-    returns -1q without doing anything.  It is useful to set a break
+    returns -1 without doing anything.  It is useful to set a break
     point on this function in a debugger.  */ 
+/***ja
+    @brief 顼κݤ˸ƤФեåؿ.
+
+    ؿ mdebug_hook () ϥ顼äݤ˸ƤФ졢⤻-1 
+    ֤ǥХåǥ֥졼ݥȤꤹ뤿Ѥ뤳ȤǤ
+    롣
+    */ 
 
 int
 mdebug_hook ()
