@@ -32,6 +32,8 @@ extern "C"
 {
 #endif
 
+#if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
+
 extern void m17n_init_win (void);
 #undef M17N_INIT
 #define M17N_INIT() m17n_init_win ()
@@ -40,6 +42,8 @@ extern void m17n_fini_win (void);
 #undef M17N_FINI
 #define M17N_FINI() m17n_fini_win ()
 
+#endif
+
 /***en @defgroup m17nGUI GUI API */
 /***ja @defgroup m17nGUI GUI API */
 /*=*/
@@ -57,32 +61,42 @@ extern void m17n_fini_win (void);
     various information about the corresponding physical display/input
     device.
 
-    The internal structure of the type #MFrame is concealed from
-    application code, and its contents depend on the window system in
-    use.  In the m17n-X library, it contains the information about @e
-    display and @e screen in the X Window System.  */
+    The internal structure of the type #MFrame is concealed from an
+    application program, and its contents depend on the window system
+    in use.  In the m17n-X library, it contains the information about
+    @e display and @e screen in the X Window System.  */
 
 /***ja
-    @brief ե졼ѹ¤
+    @brief ե졼η.
 
-    #MFrame ϡե졼४֥Ѥι¤ΤǤ롣ġΥե졼
-    ϡ줬бʪǥХγƼݻ롣
+    #MFrame ϡ@e ե졼 ֥ѤηǤ롣ġΥե졼ϡ
+    줬бʪŪɽϥǥХγƼݻ롣
 
-    #MFrame ¤ϡѤ륦ɥƥ˰¸ޤ
-    ץꥱץफϸʤm17n-X 饤֥ˤ
-    ե졼ϡX ɥ display  screen ˴ؤġ
+    #MFrame ¤ϡץꥱץफϸʤ
+    ޤƤϻѤ륦ɥƥ˰¸롣ޤm17n-X 
+    ֥ˤե졼ϡX ɥ @e display  @e screen 
+    ˴ؤġ
       */
 
 typedef struct MFrame MFrame;
 
 /*=*/
 
+extern MSymbol Mdevice;
+
 extern MSymbol Mfont;
 extern MSymbol Mfont_width;
 extern MSymbol Mfont_ascent;
 extern MSymbol Mfont_descent;
 extern MFrame *mframe_default;
 
+extern MSymbol Mdisplay;
+extern MSymbol Mscreen;
+extern MSymbol Mdrawable;
+extern MSymbol Mwidget;
+extern MSymbol Mdepth;
+extern MSymbol Mcolormap;
+
 extern MFrame *mframe (MPlist *plist);
 
 extern void *mframe_get_prop (MFrame *frame, MSymbol key);
@@ -107,17 +121,17 @@ extern void *mframe_get_prop (MFrame *frame, MSymbol key);
     This structure is used both for specifying a font in a fontset
     and for storing information about available system fonts.
 
-    The internal structure is concealed from application code.  */
+    The internal structure is concealed from an application program.  */
 
 /***ja
-    @brief եȤι¤
+    @brief եȤη.
 
-    #MFont ϥեȻѤι¤ΤǤꡢեȤΥץ
-    ѥƥȤ family, weight, style, stretch, adstyle, registry,
-    size, resolution ġ
+    #MFont ϥեȻѤι¤ΤǤꡢեȤΥץѥƥǤ
+     foundry, family, weight, style, stretch, adstyle, registry,
+    size, resolution ˴ؤޤࡣ
 
-    ι¤ΤϥեȥåΥեȤꤹȡѲǽ
-    ƥեȤξǼξѤ롣
+    ι¤ΤϥեȥåΥեȤꤹݤȡѲǽʥ
+    ƥեȤξǼݤξѤ롣
 
     ¤ϥץꥱץफϸʤ  */
 
@@ -129,21 +143,21 @@ typedef struct MFont MFont;
 
 /*=*/
 
-extern MSymbol Mfont;
+extern MSymbol Mx, Mfreetype, Mxft;
 
 extern MPlist *mfont_freetype_path;
 
 extern MFont *mfont ();
 
-extern MFont *mfont_from_name (char *name);
-
 extern MFont *mfont_copy (MFont *font);
 
-extern char *mfont_name (MFont *font);
+extern MFont *mfont_parse_name (char *name, MSymbol format);
 
-extern MFont *mfont_from_spec (char *family, char *weight, char *slant,
-			       char *swidth, char *adstyle, char *registry,
-			       unsigned short point, unsigned short res);
+extern char *mfont_unparse_name (MFont *font, MSymbol format);
+
+/* These two are obsolete (from 1.1.0).  */
+extern char *mfont_name (MFont *font);
+extern MFont *mfont_from_name (char *name);
 
 extern MSymbol Mfoundry;
 extern MSymbol Mfamily;
@@ -155,6 +169,8 @@ extern MSymbol Mregistry;
 extern MSymbol Msize;
 extern MSymbol Mresolution;
 
+extern MSymbol Mfontconfig;
+
 extern void *mfont_get_prop (MFont *font, MSymbol key);
 
 extern int mfont_put_prop (MFont *font, MSymbol key, void *val);
@@ -179,10 +195,18 @@ extern int mfont_set_encoding (MFont *font,
     font not greater than the #Msize property of $SPEC.  */
 
 /***ja
-    @brief եȤõ
+    @brief եȤõ.
 
     ؿ mfont_find () ϡե졼 $FRAME ǥե $SPEC ˤ
-    ȤᤤեȤؤΥݥ󥿤֤  */
+    ȤפѲǽʥեȤؤΥݥ󥿤֤  
+
+    $SCORE  NULL Ǥ뤫ĤäեȤ $SPEC ˤɤۤɹ
+    Ƥ뤫򼨤¸ؤΥݥ󥿤Ǥ롣
+    ۤɤäƤ뤳Ȥ̣롣
+
+    $LIMITED_SIZE  0 ǤʤС$SPEC Υץѥƥ #Msize 礭
+    ʤեȤõ롣
+*/
 
 extern MFont *mfont_find (MFrame *frame, MFont *spec,
 			  int *score, int limited_size);
@@ -191,6 +215,12 @@ extern MSymbol *mfont_selection_priority ();
 
 extern int mfont_set_selection_priority (MSymbol *keys);
 
+extern int mfont_resize_ratio (MFont *font);
+
+extern MPlist *mfont_list (MFrame *frame, MFont *font, MSymbol language,
+			   int maxnum);
+
+
 /* end of font module */
 /*=*/
 
@@ -198,7 +228,8 @@ extern int mfont_set_selection_priority (MSymbol *keys);
 /***en @defgroup m17nFontset Fontset */
 /***ja @defgroup m17nFontset եȥå */
 /*=*/
-
+/*** @addtogroup m17nFontset
+     @{   */
 typedef struct MFontset MFontset;
 
 extern MFontset *mfontset (char *name);
@@ -213,6 +244,9 @@ extern int mfontset_modify_entry (MFontset *fontset,
 				  MFont *spec, MSymbol layouter_name,
 				  int how);
 
+extern MPlist *mfontset_lookup (MFontset *fontset, MSymbol script,
+				MSymbol language, MSymbol charset);
+/*** @}   */
 /* end of fontset module */
 /*=*/
 
@@ -226,10 +260,10 @@ extern int mfontset_modify_entry (MFontset *fontset,
     @brief Type of faces.
 
     The type #MFace is the structure of face objects.  The internal
-    structure is concealed from application code.  */
+    structure is concealed from an application program.  */
 
 /***ja
-    @brief եѹ¤
+    @brief եη.
 
     #MFace ϥե֥ȤΤι¤ΤǤ롣¤
     ץꥱץफϸʤ  */
@@ -293,10 +327,17 @@ extern MFace *mface_from_font (MFont *font);
     The type #MFaceHLineProp is to specify the detail of #Mhline
     property of a face.  The value of the property must be a pointer
     to an object of this type.  */
+/***ja
+    @brief եοʿѷ.
+
+    #MFaceHLineProp ϥե #Mhline ץѥƥξܺ٤ꤹ뷿
+    Ǥ롣ΥץѥƥͤϤηΥ֥ȤǤʤƤϤʤʤ
+      */
 
 typedef struct
 {
   /***en Type of the horizontal line.  */
+  /***ja ʿΥ.  */
   enum MFaceHLineType
     {
       MFACE_HLINE_BOTTOM,      
@@ -307,10 +348,13 @@ typedef struct
     } type;
 
   /***en Width of the line in pixels.  */
+  /***ja ʥԥñ̡.  */
   unsigned width;
 
   /***en Color of the line.  If the value is Mnil, foreground color of
       a merged face is used.  */
+  /***ja ο.  Mnil ʤС礷եʿȤ롣  */
+  
   MSymbol color;
 } MFaceHLineProp;
 /*=*/
@@ -322,10 +366,17 @@ typedef struct
     The type #MFaceBoxProp is to specify the detail of #Mbox property
     of a face.  The value of the property must be a pointer to an
     object of this type.  */
+/***ja
+    @brief եΰϤȻѷ.
+
+    #MFaceBoxProp ϥե #Mbox ץѥƥξܺ٤ꤹ뷿Ǥ
+    롣ΥץѥƥͤϤηΥ֥ȤǤʤƤϤʤʤ
+      */
 
 typedef struct
 {
   /***en Width of the box line in pixels.  */
+  /***ja ʥԥñ̡.  */
   unsigned width;
 
   MSymbol color_top;
@@ -348,7 +399,13 @@ typedef struct
     The type #MFaceHookFunc is to specify the #Mhook property of a
     face.  The value of the property must be function of this
     type.  */
-typedef void *(*MFaceHookFunc) (MFace *face, void *arg, void *info);
+/***ja
+    @brief եΥեåؿη.
+
+    #MFaceHookFunc ϥե #Mhook ץѥƥꤹ뷿Ǥ롣
+    ΥץѥƥͤϡηδؿǤʤƤϤʤʤ
+      */
+typedef void (*MFaceHookFunc) (MFace *face, void *arg, void *info);
 /*=*/
 
 extern void *mface_get_prop (MFace *face, MSymbol key);
@@ -369,16 +426,21 @@ extern void mface_update (MFrame *frame, MFace *face);
 /***en
     @brief Window system dependent type for a window.
 
-    The type MDrawWindow is for a window; a rectangular area that
+    The type #MDrawWindow is for a window; a rectangular area that
     works in several ways like a miniature screen.
 
     What it actually points depends on a window system.  A program
     that uses the m17n-X library must coerce the type @c Drawable to
     this type.  */
+/***ja 
+    @brief ɥƥ˰¸롢ɥη.
 
-/***ja ɥƥ˰¸롢ɥɽ֥Ѥη
+    #MDrawWindow ϥɥʤĤǥ꡼Υߥ˥
+    ȤƯΰѤηǤ롣
 
-       m17n X 饤֥Ǥϡ@c Window Ʊ.  */
+    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 饤֥
+    Ѥץ @c Drawable 򤳤ηѴʤƤϤʤ
+    ʤ */
 
 typedef void *MDrawWindow;
 /*=*/
@@ -387,12 +449,21 @@ typedef void *MDrawWindow;
 /***en
     @brief Window system dependent type for a region.
 
-    The type MDrawRegion is for a region; an arbitrary set of pixels
+    The type #MDrawRegion is for a region; an arbitrary set of pixels
     on the screen (typically a rectangular area).
 
     What it actually points depends on a window system.  A program
     that uses the m17n-X library must coerce the type @c Region to
     this type.  */
+/***ja
+    @brief ɥƥ˰¸롢ΰη.
+
+    #MDrawRegion ΰ衢ʤ꡼ǤդΥԥν
+   ŵŪˤ϶ΰѤηǤ롣
+
+    ºݤ˲ؤϥɥƥ˰¸롣 m17n X 饤֥
+    Ѥץ @c Region 򤳤ηѴʤƤϤʤ
+    ʤ  */
 
 typedef void *MDrawRegion;
 /*=*/
@@ -403,17 +474,29 @@ typedef void *MDrawRegion;
 
     The type #MDrawControl is the structure that controls how to draw
     an M-text.  */
+/***ja
+    @brief ƥɽη.
+
+    #MDrawControl ϡM-text ɤɽ뤫椹빽¤ΤǤ롣
+      */
+
 
 typedef struct
 {
   /***en If nonzero, draw an M-text as image, i.e. with background
       filled with background colors of faces put on the M-text.
       Otherwise, the background is not changed.  */
+  /***ja 0 ǤʤС M-text Ȥơʤطʤ M-text 
+      եǻꤵƤطʿɽ롣Ǥʤ
+      ʤѤʤ  */
   unsigned as_image : 1;
 
   /***en If nonzero and the first glyph of each line has negative
       lbearing, shift glyphs horizontally to right so that no pixel is
       drawn to the left of the specified position.  */
+  /***ja 0 ǤʤƹԤκǽΥդlbearing ʤСդ
+      ʿ˱ˤ餷ơꤷ֤꺸˥ԥ뤬ʤ褦
+      롣  */
   unsigned align_head : 1;
 
   /***en If nonzero, draw an M-text two-dimensionally, i.e., newlines
@@ -421,46 +504,72 @@ typedef struct
       the next line.  If <format> is non-NULL, and the function
       returns nonzero line width, a line longer than that width is
       also broken.  */
+  /***ja 0 ǤʤСM-text 򣲼ŪˡʤM-text  
+      newline ǲԤ³ʸϼιԤɽ롣⤷ <format>  
+      NULL Ǥʤδؿ 0 Ǥʤ֤СĹԤ
+      Ԥ롣  */
   unsigned two_dimensional : 1;
 
   /***en If nonzero, draw an M-text to the right of a specified
       position.  */
+  /***ja 0 ǤʤСM-text ꤷ֤αɽ롣  */
   unsigned orientation_reversed : 1;
 
-  /***en If nonzero, reorder glyphs correctly for bidi text.  */
+  /***en If nonzero, reorder glyphs correctly for bidi text.  */ 
+  /***ja 0 ʤСbidi ƥѤ˥դ󤹤롣  */
   unsigned enable_bidi : 1;
 
   /***en If nonzero, don't draw characters whose general category (in
       Unicode) is Cf (Other, format).  */
+  /***ja 0 ǤʤС˥ɤ֤̥ƥ꤬ Cf (Other,
+      format) Ǥʸɽʤ  */
   unsigned ignore_formatting_char : 1;
 
   /***en If nonzero, draw glyphs suitable for a terminal.  Not yet
       implemented.  */
+  /***ja 0 ǤʤСüѤΥդɽ롣̤  */
   unsigned fixed_width : 1;
 
   /***en If nonzero, draw glyphs with anti-aliasing if a backend font
       driver supports it.  */
+  /***ja 0 ǤʤСꥢǥդɽ롣ʥХå
+      ɤΥեȥɥ饤Фꥢǽ򥵥ݡȤƤ
+      Τߡ */
   unsigned anti_alias : 1;
 
+  /***en If nonzero, disable the adjustment of glyph positions to
+      avoid horizontal overlapping at font boundary.  */
+  /***ja 0 ǤʤСեȶǤοʿΥդνŤʤ
+      뤿Υհ֤Ĵ̵ˤ롣  */
+  unsigned disable_overlapping_adjustment : 1;
+
   /***en If nonzero, the values are minimum line ascent and descent
       pixels.  */
+  /***ja 0 ǤʤСͤϹԤ ascent  descent κǾͤ򼨤  */
   unsigned int min_line_ascent;
   unsigned int min_line_descent;
 
   /***en If nonzero, the values are maximum line ascent and descent
       pixels.  */
+  /***ja 0 ǤʤСͤϹԤ ascent  descent κͤ򼨤  */
   unsigned int max_line_ascent;
   unsigned int max_line_descent;
 
   /***en If nonzero, the value specifies how many pixels each line can
       occupy on the display.  The value zero means that there is no
       limit.  It is ignored if <format> is non-NULL.  */
+  /***ja 0 ǤʤСͤϤΥǥץ쥤ǳƹԤ뤳ȤΤǤ
+      ԥ򼨤 0 ϸꤵʤȤ̣롣<format> 
+      NULL Ǥʤ̵뤵롣   */
   unsigned int max_line_width;
 
   /***en If nonzero, the value specifies the distance between tab
       stops in columns (the width of one column is the width of a
       space in the default font of the frame).  The value zero means
       8.  */
+  /***ja 0 ǤʤСͤϥ֥ȥå״֤εΥ򥳥ñ̡ʥ
+      ե졼ΥǥեȥեȤˤʸǤˤǼ 
+      0  8 ̣롣 */
   unsigned int tab_width;
 
   /***en If non-NULL, the value is a function that calculates the
@@ -483,13 +592,28 @@ typedef struct
       because of the width limit.
 
       This has an effect only when <two_dimensional> is nonzero.  */
+  /***ja 0 ǤʤСͤϴؿǤꡢδؿϹֹ LINE Ⱥɸ Y 
+      ˴ŤƳƹԤΥǥȤȺ׻줾INDENT 
+      WIDTH ǻؤ¸롣
+
+      ǥȤϡƹԤκǽΥդʥ 
+      <orientation_reversed>  0 λˤ뤤Ϻʤʳλˤ˲
+      ԥ뤺餹ꤹ롣ͤʤеˤ餹
+
+      ϡƹԤǥץ쥤뤳ȤΤǤԥκ
+      ͤǤ롣ͤ 0 ξ¤ʤȤ̣롣
+
+      LINE  Y ϲʸˤäƹԤޤäݤˤ 0 ˥ꥻåȤ졢
+      ˤäƹԤޤäˤ 1 Ť䤵롣
+
+       <two_dimensional>  0 ǤʤˤΤͭǤ롣  */
   void (*format) (int line, int y, int *indent, int *width);
 
   /***en If non-NULL, the value is a function that calculates a line
       breaking position when a line is too long to fit within the
-      width limit.  POS is a position of the character next to the
-      last one that fits within the limit.  FROM is a position of the
-      first character of the line, and TO is a position of the last
+      width limit.  POS is the position of the character next to the
+      last one that fits within the limit.  FROM is the position of the
+      first character of the line, and TO is the position of the last
       character displayed on the line if there were not width limit.
       LINE and Y are the same as the arguments to <format>.
 
@@ -500,6 +624,17 @@ typedef struct
 
       The mdraw_default_line_break () function is useful for such a
       script that uses SPACE as a word separator.  */
+  /***ja NULL ǤʤСͤϹԤ˼ޤʤ˹Ԥ
+      ֤׻ؿǤ롣POS Ϻ˼ޤǸʸμʸ
+      ΰ֤Ǥ롣FROM ϹԤκǽʸΰ֡TO Ϻꤵ
+      ƤʤФιԤɽǸʸΰ֤Ǥ롣LINE  
+      Y  <format> ΰƱǤ롣
+
+      δؿϹԤʸ֤֤ʤƤϤʤʤޤMT 
+      ѹƤϤʤʤ
+
+      ؿ mdraw_default_line_break () ϡζڤȤѤ
+      륹ץѤȤѤ뤳ȤǤ롣  */
   int (*line_break) (MText *mt, int pos, int from, int to, int line, int y);
 
   int with_cursor;
@@ -509,19 +644,28 @@ typedef struct
       displayed next to the last character of an M-text.  If the value
       is negative, even if <cursor_width> is nonzero, cursor is not
       displayed.  */
+  /***ja ɽʸ֤򼨤ʸ֤礭
+      СM-text κǸʸ٤ɽ롣ʤС
+      <cursor_width>  0 ǤʤƤ⥫ɽʤ
+        */
   int cursor_pos;
 
   /***en If nonzero, display a cursor at the character position
       <cursor_pos>.  If the value is positive, it is the pixel width
       of the cursor.  If the value is negative, the cursor width is
       the same as the underlining glyph(s).  */
+  /***ja 0 ǤʤС<cursor_pos> ˥ɽ롣ͤʤС
+      Ϥ͡ʥԥñ̡ˤǤ롣ʤС
+      륰դƱǤ롣  */
   int cursor_width;
 
   /***en If nonzero and <cursor_width> is also nonzero, display double
       bar cursors; at the character position <cursor_pos> and at the
       logically previous character.  Both cursors have one pixel width
-      with horizontal fringes at upper or lower positions.  HOW TO
-      EXPLAIN THE DOUBLE CURSORS?  */
+      with horizontal fringes at upper or lower positions.  */
+  /***ja If 0 Ǥʤ <cursor_width>  0 ǤʤСС
+      ʸ<cursor_pos> ŪˤˤʸΣɽ
+      롣Ȥ⣱ԥǡ岼˿ʿξ꤬Ĥ*/
   int cursor_bidi;
 
   /***en If nonzero, on drawing partial text, pixels of surrounding
@@ -530,13 +674,23 @@ typedef struct
       upper vowel) is drawn so that V is placed over the middle of two
       Cs.  If this CVC sequence is already drawn and only the last C
       is drawn again (for instance by updating cursor position), the
-      left half of V is erased if this member is zero.  By setting
+      right half of V is erased if this member is zero.  By setting
       this member to nonzero, even with such a drawing, we can keep
       this CVC sequence correctly displayed.  */
+  /***ja 0 ǤʤСƥȤΰʬɽݤˡΥƥ
+      Τɽΰ˿ʬɽ롣ȤСʸ
+      λҲ-첻-ҲȤ󥹤ΤĤϡ첻ĤλҲ
+      δ֤˾ˤΤ褦롣Τ褦ʥ󥹤Ǥ
+      ƤꡢǸλҲľʤȤС֤
+      ݤʤɡˤΥФ 0 ǤС첻αȾʬä
+      ޤ 0 ʳˤ뤳ȤˤäơΤ褦ʺݤˤҲ-
+      -Ҳ󥹤ɽ³뤳ȤǤ롣  */
   int partial_update;
 
   /***en If nonzero, don't cache the result of any drawing information
       of an M-text.  */
+  /***ja 0 ǤʤСM-text ɽ˴ؤ򥭥å夷ʤ
+       */
   int disable_caching;
 
   /* If non-NULL, limit the drawing effect to the specified region.  */
@@ -553,6 +707,11 @@ typedef struct
     The type #MDrawMetric is for a metric of a glyph and a drawn text.
     It is also used to represent a rectangle area of a graphic
     device.  */
+/***ja
+    @brief դȥƥȤˡη.
+
+    #MDrawMetric ϥդɽ줿ƥȤˡѤηǤ롣ޤ
+    ɽǥХζΰɽΤˤѤ롣 */
 
 typedef struct {
   int x, y;
@@ -567,40 +726,118 @@ typedef struct {
 
     The type #MDrawGlyphInfo is the structure that contains
     information about a glyph.  It is used by mdraw_glyph_info ().  */
+/***ja
+    @brief դ˴ؤη.
+
+    #MDrawGlyphInfo ϥդ˴ؤޤ๽¤ΤǤ롣
+    mdraw_glyph_info () ϤѤ롣  */
 
 typedef struct
 {
   /***en Character range corresponding to the glyph.  */
+  /***ja դбʸϰ.  */
   int from, to;
 
   /***en Character ranges corresponding to the line of the glyph.  */
+  /***ja  դбʸϰ.  */
   int line_from, line_to;
 
   /***en X/Y coordinates of the glyph.  */
+  /***ja դ X/Y ɸ.  */
   int x, y;
 
   /***en Metric of the glyph.  */
-  MDrawMetric this;
+  /***ja դˡ.  */
+  MDrawMetric metrics;
 
   /***en Font used for the glyph.  Set to NULL if no font is found for
       the glyph.  */
+  /***ja դ˻ȤեȡĤʤ NULL
+      the glyph.  */
   MFont *font;
 
   /***en Character ranges corresponding to logically previous and next
       glyphs.  Note that we do not need the members prev_to and
-      next_from because they must be the same as the memberse from and
+      next_from because they must be the same as the members from and
       to respectively.  */
+  /***ja ŪΥդбʸϰϡ prev_to 
+      next_from ϡ줾 from to ƱǤʤƤϤʤʤ
+      פǤ뤳Ȥա  */
   int prev_from, next_to;
 
   /***en Character ranges corresponding to visually left and right
       glyphs. */
+  /***ja ɽκΥդбʸϰϡ  */
   int left_from, left_to;
   int right_from, right_to;
 
+  /***en Logical width of the glyph.  Nominal distance to the next
+      glyph.  */
+  /***ja դŪΥդȤܾ̾εΥ  */
+  int logical_width;
 } MDrawGlyphInfo;
 
 /*=*/
 
+/*** @ingroup m17nDraw */
+/***en
+    @brief Type of information about a glyph metric and font.
+
+    The type #MDrawGlyph is the structure that contains information
+    about a glyph metric and font.  It is used by the function
+    mdraw_glyph_list ().  */
+/***ja
+    @brief դˡȥեȤ˴ؤη.
+
+    #MDrawGlyph ϥդˡȥեȤ˴ؤޤ๽¤ΤǤ
+    롣mdraw_glyph_list () ϤѤ롣  */
+
+typedef struct
+{
+  /***en Character range corresponding to the glyph.  */
+  /***ja դбʸϰ.  */
+  int from, to;
+
+  /***en Font glyph code of the glyph.  */
+  /***ja եΥեɡ  */
+  int glyph_code;
+
+  /***en Logical width of the glyph.  Nominal distance to the next
+      glyph.  */
+  /***ja դŪΥդȤܾ̾εΥ  */
+  int x_advance, y_advance;
+
+  /***en X/Y offset relative to the glyph position.  */
+  /***ja դΰ֤Ф X/Y եå.  */
+  int x_off, y_off;
+
+  /***en Metric of the glyph.  */
+  /***ja դˡ.  */
+  int lbearing, rbearing, ascent, descent;
+
+  /***en Font used for the glyph.  Set to NULL if no font is found for
+      the glyph.  */
+  /***ja դ˻ȤեȡĤʤ NULL
+      the glyph.  */
+  MFont *font;
+
+  /***en Type of the font.  One of Mx, Mfreetype, Mxft.  */
+  /***ja եȤΥסMxMfreetypeMxft Τ줫  */
+  MSymbol font_type;
+
+  /***en Pointer to the font structure.  The actual type is
+      (XFontStruct *) if <font_type> member is Mx, FT_Face if
+      <font_type> member is Mfreetype, and (XftFont *) if <font_type>
+      member is Mxft.  */
+  /***ja եȤι¤ΤؤΥݥ󥿡ºݤη <font_type> Ф
+      Mx ʤ (XFontStruct *) Mfreetype ʤ FT_FaceMxft ʤ
+      (XftFont *) */
+  void *fontp;
+
+} MDrawGlyph;
+
+/*=*/
+
 /***en
     @brief Type of text items.
 
@@ -609,34 +846,37 @@ typedef struct
     control the drawing of the M-text.  */
 
 /***ja
-    @brief textitem ѹ¤
+    @brief textitem η.
 
-     #MDrawTextItem  @e ƥȥƥ ֥Ѥι¤ΤǤ
-    롣ƥƥȥƥϡ1Ĥ M-text ȡɽ椹뤿
-    γƼޤǤ롣
+    #MDrawTextItem  @e ƥȥƥ ֥ѤηǤ롣
+    ƥȥƥϡ1Ĥ M-text ȡɽ椹뤿γƼ
+    ޤǤ롣
 
     @latexonly \IPAlabel{MTextItem} @endlatexonly  */
 
 typedef struct
 {
   /***en M-text. */
-  /***ja M-text */
+  /***ja M-text. */
   MText *mt;                      
 
   /***en Optional change in the position (in the unit of pixel) along
       the X-axis before the M-text is drawn.  */
-  /***ja ˹ԤʤXΰĴ (ԥñ) */
+  /***ja M-text ɽ˹ԤʤXΰĴ (ԥñ) */
   int delta;                     
 
   /***en Pointer to a face object.  Each property of the face, if not
       Mnil, overrides the same property of face(s) specified as a text
       property in <mt>.  */
-  /***ja եȥåȥ֥ȤؤΥݥ󥿡 M-text ǻ
-      ꤵ줿եΥեȥåȤͥ褹*/
+  /***ja ե֥ȤؤΥݥ󥿡եγƥץѥƥ 
+      Mnil Ǥʤ <mt> ǻꤵ줿եƱץѥƥͥ
+      */
   MFace *face;
 
   /***en Pointer to a draw control object.  The M-text <mt> is drawn
       by mdraw_text_with_control () with this control object.  */
+  /***ja ɽ楪֥ȤؤΥݥ󥿡 mdraw_text_with_control () 
+      ϤΥ֥ȤѤ M-text <mt> ɽ롣  */
   MDrawControl *control;
 
 } MDrawTextItem;
@@ -677,6 +917,10 @@ extern int mdraw_text_per_char_extents (MFrame *frame,
 extern int mdraw_glyph_info (MFrame *frame, MText *mt, int from, int pos,
 			     MDrawControl *control, MDrawGlyphInfo *info);
 
+extern int mdraw_glyph_list (MFrame *frame, MText *mt, int from, int to,
+			     MDrawControl *control, MDrawGlyph *glyphs,
+			     int array_size, int *num_glyphs_return);
+
 extern void mdraw_text_items (MFrame *frame, MDrawWindow win, int x, int y,
 			      MDrawTextItem *items, int nitems);
 
@@ -709,10 +953,10 @@ extern MInputDriver minput_gui_driver;
     input method.  */
 
 /***ja
-    @brief ؿ minput_create_ic () ΰ $ARG ǻؤ빽¤
+    @brief ؿ minput_create_ic () ΰη.
 
-    #MInputGUIArgIC ϡؿ minput_create_ic () ϥ᥽
-    ɤݤˡ $ARG ˤäƻؤ빽¤ΤǤ롣  */
+    #MInputGUIArgIC ϡؿ minput_create_ic () ϥ᥽åɤ
+    ϥƥȤݤΡ $ARG ѤηǤ롣  */
 
 typedef struct
 {
