@@ -36,6 +36,7 @@ extern "C"
 {
 #endif
 
+#if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
 extern void m17n_init (void);
 #undef M17N_INIT
 #define M17N_INIT() m17n_init ()
@@ -43,30 +44,39 @@ extern void m17n_init (void);
 extern void m17n_fini (void);
 #undef M17N_FINI
 #define M17N_FINI() m17n_fini ()
+#endif
 
 /***en @defgroup m17nShell SHELL API */
-/***ja @defgroup m17nShell SHELL API */
+/***ja @defgroup m17nShell  API */
 /*=*/
 
 /*
  *  (11) Functions related to the m17n database
  */
+/*=*/
 
 /*** @ingroup m17nShell */
 /***en @defgroup m17nDatabase Database */
 /***ja @defgroup m17nDatabase ǡ١ */
 /*=*/
+
 /* Directory of an application specific databases.  */
 extern char *mdatabase_dir;
-
+/*=*/
 /***
     @ingroup m17nDatabase  */ 
 /***en
     @brief Type of database.
 
     The type #MDatabase is for a database object.  Its internal
-    structure is concealed from application programs.  */
-/***ja ǡ١η */
+    structure is concealed from an application program.  */
+/***ja 
+    @brief ǡ١η.
+
+    #MDatabase ϥǡ١֥Ѥι¤ΤǤ롣¤
+    ϥץꥱץफϸʤ
+    */
+
 typedef struct MDatabase MDatabase;
 
 /*=*/
@@ -92,12 +102,16 @@ extern MDatabase *mdatabase_define (MSymbol tag1, MSymbol tag2,
 
 /*=*/
 /* (S2) Charset staffs */
+/*=*/
 
 /*** @ingroup m17nShell */
 /***en @defgroup m17nCharset Charset */
 /***ja @defgroup m17nCharset ʸå */
 /*=*/
+
+#if !defined (FOR_DOXYGEN) || defined (DOXYGEN_INTERNAL_MODULE)
 #define MCHAR_INVALID_CODE 0xFFFFFFFF
+#endif
 
 /* Predefined charsets */ 
 extern MSymbol Mcharset_ascii;
@@ -133,7 +147,7 @@ extern MSymbol Msuperset;
 /* etc. */
 extern MSymbol Mcharset;
 
-extern MSymbol mchar_define_charset (char *name, MPlist *plist);
+extern MSymbol mchar_define_charset (const char *name, MPlist *plist);
 
 extern MSymbol mchar_resolve_charset (MSymbol symbol);
 
@@ -150,7 +164,7 @@ extern int mchar_map_charset (MSymbol charset_name,
 /*=*/
 
 /* (S3) code conversion */
-
+/*=*/
 /*** @ingroup m17nShell */
 /***en @defgroup m17nConv Code Conversion */
 /***ja @defgroup m17nConv Ѵ */
@@ -211,33 +225,32 @@ extern MSymbol Mmaybe;
     One of these values is set in @c MConverter-\>result.   */
 
 /***ja
-    @brief Ѵη̤򼨤
+    @brief Ѵη̤򼨤.
 
     ͤΤĤ @c MConverter-\>result ꤵ롣  */
 
 enum MConversionResult
   {
     /***en Code conversion is successful. */
-    /***ja Ѵ */
+    /***ja Ѵ. */
     MCONVERSION_RESULT_SUCCESS,
 
     /***en On decoding, the source contains an invalid byte. */
-    /***ja ǥɤκݡʥХȤޤޤ롣 */
+    /***ja ǥɻʥХȤޤޤƤ. */
     MCONVERSION_RESULT_INVALID_BYTE,
 
     /***en On encoding, the source contains a character that cannot be
 	encoded by the specified coding system. */
-
-    /***ja 󥳡ɤκݡ˻ΥɷϤ
-	󥳡ɤǤʤʸޤޤ롣 */
+    /***ja 󥳡ɻΥɷϤǥ󥳡ɤǤʤʸ
+	˴ޤޤƤ. */
     MCONVERSION_RESULT_INVALID_CHAR,
 
     /***en On decoding, the source ends with an incomplete byte sequence. */
-    /***ja ǥɤκݡԴʥХǽ롣*/
+    /***ja ǥɻԴʥХǥäƤ롣*/
     MCONVERSION_RESULT_INSUFFICIENT_SRC,
 
     /***en On encoding, the destination is too short to store the result. */
-    /***ja 󥳡ɤκݡ̤Ǽΰ褬û롣 */
+    /***ja 󥳡ɻ̤Ǽΰ褬û롣 */
     MCONVERSION_RESULT_INSUFFICIENT_DST,
 
     /***en An I/O error occurred in the conversion.  */
@@ -250,10 +263,13 @@ enum MConversionResult
 /***en
     @brief Structure to be used in code conversion.
 
-    The first three members are to control the conversion.  */
+    Structure to be used in code conversion.  The first three members
+    are to control the conversion.  */
 
 /***ja
-    @brief ѴѤ빽¤ 
+    @brief ѴѤ빽¤.
+
+    ѴѤ빽¤ΡǽΣĤΥФѴ椹롣
 
     @latexonly \IPAlabel{MConverter} @endlatexonly  
 */
@@ -276,20 +292,21 @@ typedef struct
       with "<U+XXXX>" (if the character is a Unicode character) or
       with "<M+XXXXXX>" (otherwise).  */
 
-  /***ja
-      ̩ѴɬפǤʤˤΥե饰ӥåȤ򤿤Ƥ롣ǥե
-      ȤǤϡѴϸ̩Ǥ롣
+  /***ja 
+      ̩ѴɬפǤʤͤ 0 ʳˤ롣ǥեȤǤϡ
+      Ѵϸ̩Ǥ롣
 
       Ѵ̩ȤϡǥɤκݤˤϺǽʥХȤǥС
       ߤޤ뤳ȡ󥳡ɤκݤˤѴ륳ɷϤǥݡȤ
       ʤǽʸǥСߤޤ뤳Ȥؤξ硢
-      @c MConverter-\>result Ϥ줾@c
+      @c MConverter-\>result Ϥ줾 @c
       MCONVERSION_RESULT_INVALID_BYTE @c
       MCONVERSION_RESULT_INVALID_CHAR Ȥʤ롣
 
       Ѵ̩ǤʤˤϡǥɤκݤʥХȤϤΥХ
-      ȤΤޤ޻Ĥ롣ޤ󥳡ɤκݤˤϡʸϥɷϤ
-      줿ǥեȤʸ֤롣  */
+      Τޤ޻Ĥ롣ޤ󥳡ɤκݤˤϡʸ Unicode ʸ
+      ˤ "<U+XXXX>" ˡǤʤˤ "<M+XXXXXX>" ֤
+      롣  */
 
   int lenient;
 
@@ -319,33 +336,52 @@ typedef struct
 	If the value is zero, the conversion never produce such a byte
 	sequence at the end.  */
 
-    /***ja
-	ʸνüʬ򥨥󥳡ɤݤˤϡΥե饰Ω
-	롣ξϥɥݥΥƥȤ򸵤᤹
-	ХȤղŪ뤳Ȥ롣
+/***ja 
 
-	Υե饰ϥǥեȤǤΩäƤ餺С³¾
-	ʸ򥨥󥳡ɤΤȲꤷƤ롣
+     ХνüΥ֥åǥɤݡޤʸνüΥ
+     å򥨥󥳡ɤݤϡͤ 0 ʳˤ롣ͤϰʲΤ褦
+     Ѵ˱ƶ롣
 
-	Υե饰ϥǥɤˤϴطʤ  */
+     ǥǥ󥰤κݤ˺ǸοХȤХȥ󥹤
+     ˤû硧
+
+       ͤ 0 ǤʤСѴϤΥ󥹤κǽΥХȤˤơ
+       顼 (MCONVERSION_RESULT_INVALID_BYTE) ǽ롣
+
+       ͤ 0 ʤСѴƽ롣οХȤϥ꡼
+       ФȤƥСݻ졢³ѴԤݤѴХ
+       դ롣
+
+     󥳡ǥ󥰤κݤ˥ɷϤʸ̮¸ξ硢
+
+       ͤ 0 ǤʤСƥȤǽ᤹ΥХ󤬥
+       ʸȤʤѴη뤳Ȥ롣
+
+       ͤ 0 ʤСΤ褦ʥХʤ
+
+	 */
 
   int last_block;
 
   /***en
       If the value is nonzero, it specifies at most how many
       characters to convert.  */
+  /***ja
+      0 ǤʤСѴʸꤹ롣
+      */
 
   unsigned at_most;
 
   /***en
       The following three members are to report the result of the
       conversion.  */
+  /***ja
+      ʲΣĤΥФѴη̤𤹤롣  */
 
   /***en
       Number of characters most recently decoded or encoded. */
-
   /***ja
-      Ǹ˥ǥ/󥳡ɤ줿ʸ */
+      Ƕ˥ǥ/󥳡ɤ줿ʸ */
 
   int nchars;
 
@@ -353,7 +389,7 @@ typedef struct
       Number of bytes recently decoded or encoded. */
 
   /***ja
-      Ǹ˥ǥ/󥳡ɤ줿Хȿ */
+      Ƕ˥ǥ/󥳡ɤ줿Хȿ */
 
   int nbytes;
 
@@ -361,7 +397,7 @@ typedef struct
       Result code of the conversion. */
 
   /***ja
-      Ѵη̤򼨤 */
+      Ѵη̤򼨤ɡ */
 
   enum MConversionResult result;
 
@@ -372,10 +408,10 @@ typedef struct
       and at least 256 bytes of memory space can be used.  */
 
   /***ja
-      Ѵξ˴ؤƤϥɷϤΥפˤäưۤ
-      롣@c status ϤɤΤ褦ʷؤΥ㥹ȤФƤʤ褦˥
-      ꥢ饤󤵤Ƥꡢޤ256ХȤΥΰ褬Ȥ
-      ˤʤäƤ롣  */
+      Ѵξ˴ؤξƤϥɷϤΥפˤ
+      ưۤʤ롣@c status ϤɤΤ褦ʷؤΥ㥹ȤФƤʤ
+      ˥ꥢ饤󤵤Ƥꡢޤ256ХȤΥΰ褬
+      褦ˤʤäƤ롣  */
 
   union {
     void *ptr;
@@ -386,13 +422,17 @@ typedef struct
   /***en
       This member is for internally use only.  An application program
       should never touch it.  */
+  /***ja ΥФŪ˻Ѥ졢ץꥱץϿ
+      ƤϤʤʤ */
   void *internal_info;
 } MConverter;
 /*=*/
 
 /*** @ingroup m17nConv */
-/***en @brief Types of coding system  */
-/***ja @brief ɷϤΥ  */
+/***en 
+    @brief Types of coding system.  */
+/***ja 
+    @brief ɷϤΥ.  */
 
 enum MCodingType
   {
@@ -439,10 +479,9 @@ enum MCodingType
 	The m17n library provides decoding and encoding routines of
 	this type.  */
 
-    /***ja
-	ΥפΥɷϤϡISO-2022 ϤΥХ򥵥ݡȤ롣
-	ΥɷϤι¤ξܺ٤ @c MCodingInfoISO2022 ǻꤵ
-	롣
+    /***ja 
+        ΥפΥɷϤϡISO-2022 ϤΥХ򥵥ݡȤ롣
+	ƥɷϤι¤ξܺ٤ @c MCodingInfoISO2022 ǻꤵ롣
 
 	m17n 饤֥ϤΥѤΥǥեȤΥ󥳡ɡǥ
 	ɥ롼ƥ󶡤롣  */
@@ -462,7 +501,7 @@ enum MCodingType
 	롣
 
 	m17n 饤֥ϤΥѤΥ󥳡ɡǥɥ롼ƥ
-        󶡤ʤΤǡץꥱץ¦Ǥ
+        󶡤ʤΤǡץꥱץ¦ǽ
         ɬפ롣  */
 
     MCODING_TYPE_MISC
@@ -470,11 +509,10 @@ enum MCodingType
 /*=*/
 
 /*** @ingroup m17nConv */
-/***en @brief Bit-masks to specify the detail of coding system whose type is
-    MCODING_TYPE_ISO_2022.  */
-
-/***ja @brief MCODING_TYPE_ISO_2022 פΥɷϤξܺ٤ɽ魯ӥåȥޥ
-      */
+/***en 
+    @brief  Bit-masks to specify the detail of coding system whose type is MCODING_TYPE_ISO_2022.  */
+/***ja 
+    @brief MCODING_TYPE_ISO_2022 פΥɷϤξܺ٤ɽ魯ӥåȥޥ.  */
 
 enum MCodingFlagISO2022
   {
@@ -511,9 +549,9 @@ enum MCodingFlagISO2022
     /***en
 	On encoding, unless explicitly specified, designate charsets
 	to G0.  */
-    /***ja
-	󥳡ɤκݡä˻ꤵʤ¤ꡢʸ G0 
-	ؼ롣*/
+    /***ja 
+        󥳡ɤκݡä˻ꤵʤ¤ꡢʸ G0 ˻ؼ
+	롣*/
     MCODING_ISO_DESIGNATION_G0 =		0x10,
 
     /***en
@@ -553,13 +591,13 @@ enum MCodingFlagISO2022
     /***en
 	Use single shift (SS2 (0x8E or ESC N), SS3 (0x8F or ESC O)).  */
     /***ja
-	󥰥륷ե (SS2 or ESC N) Ȥ  */
+	󥰥륷ե (SS2 (0x8E or ESC N), SS3 (0x8F or ESC O)) Ȥ  */
     MCODING_ISO_SINGLE_SHIFT =	0x200,
 
     /***en
 	Use 7-bit single shift 2 (SS2 (0x19)).  */
     /***ja
-	󥰥륷ե (0x19) Ȥ  */
+	7ӥåȥ󥰥륷ե 2 (SS2 (0x19)) Ȥ  */
     MCODING_ISO_SINGLE_SHIFT_7 =	0x400,
 
     /***en
@@ -572,7 +610,7 @@ enum MCodingFlagISO2022
 	Use ISO-6429 escape sequences to indicate direction.
 	Not yet implemented.  */
     /***ja
-	ISO-6429 Υץ󥹤ؼ롣  */
+	ISO-6429 Υץ󥹤ؼ롣̤ */
     MCODING_ISO_ISO6429 =		0x1000,
 
     /***en
@@ -586,7 +624,7 @@ enum MCodingFlagISO2022
     /***en
 	Support all ISO-2022 charsets.  */
     /***ja
-	ISO-2022 ʸ򥵥ݡȤ  */
+	ISO-2022 ʸ򥵥ݡȤ롣  */
     MCODING_ISO_FULL_SUPPORT =		0x3000,
 
     MCODING_ISO_FLAG_MAX
@@ -595,14 +633,16 @@ enum MCodingFlagISO2022
 
 /*** @ingroup m17nConv */
 /***en
-    @brief Structure for a coding system of type MCODING_TYPE_ISO_2022.
+    @brief Structure for a coding system of type #MCODING_TYPE_ISO_2022.
 
     Structure for extra information about a coding system of type
     MCODING_TYPE_ISO_2022.  */
 
-/***ja
-    @brief MCODING_TYPE_ISO_2022 פΥɷϤɬ
-    ղþѹ¤
+/***ja 
+    @brief #MCODING_TYPE_ISO_2022 פΥɷϤɬפղþѹ¤.
+
+    MCODING_TYPE_ISO_2022 פΥɷϤɬפղþѤݻ
+    ι¤Ρ
 
     @latexonly \IPAlabel{MCodingInfoISO2022} @endlatexonly  */
 
@@ -635,17 +675,17 @@ typedef struct
       register.  */
 
   /***ja
+      ĥǤΥơ֥롣NܤǤϡ$CHARSET_NAMES  N 
+      ʸåȤб롣$CHARSET_NAMES ϴؿ 
+      mconv_define_coding () ΰȤʤ롣
 
-  ĥǤΥơ֥롣NܤǤϡ$CHARSET_NAMES  N 
-  ʸåȤб롣$CHARSET_NAMES ϴؿ 
-  mconv_define_coding () ΰȤƻȤ롣
+      ͤ 0..3 ä顢бʸåȤ G0..G3 Τ줾˻ؼ
+      뤳ȡ֤ǤǤ G0..G3 ˻ؼƤ뤳Ȥ̣
+      롣
 
-  ͤ 0..3 ä顢бʸåȤ G0..G3 Τ줾˻ؼ
-  뤳Ȥ̣롣ˡ֤ǤǤ G0..G3 ˻ؼ
-  롣
-
-  ͤ -4..-1 ä顢бʸåȤ G0..G3 Τ줾˻
-  뤬֤ǤϤɤˤؼƤʤȤ̣롣 */
+      ͤ -4..-1 ä顢бʸåȤ G0..G3 Τ줾˻
+       뤬֤ǤϤɤˤؼƤʤȤ̣롣
+        */
 
   char designations[32];
 
@@ -662,11 +702,11 @@ typedef struct
 
 /*** @ingroup m17nConv */
 /***en
-    @brief Structure for extra information about a coding system of
-    type #MCODING_TYPE_UTF.  */
+    @brief Structure for extra information about a coding system of type #MCODING_TYPE_UTF.
+    */
 
 /***ja
-    @brief MCODING_TYPE_UTF פΥɷϤɬפղþѤι¤
+    @brief #MCODING_TYPE_UTF פΥɷϤɬפղþѤι¤.
 
     @latexonly \IPApage{MCodingInfoUTF} @endlatexonly
 
@@ -676,6 +716,8 @@ typedef struct
 {
   /***en
       Specify bits of a code unit.  The value must be 8, 16, or 32.  */
+  /***ja
+      Ĺʥӥåȿˡͤ 8, 16, 32 Τ줫  */
   int code_unit_bits;
 
   /***en
@@ -701,18 +743,18 @@ typedef struct
       Ƭ BOM (Хȥޡ) μ갷ꤹ롣ͤ 0,
       1, 2 Τ줫Ǥꡢ줾ΰ̣ϰʲΤ褦ˤʤ롣
 
-      0: ǥɤκݤ˺ǽ2ХȤĴ٤롣⤷줬 BOM ǤС
-      ǥ򤽤Ƚꤹ롣⤷ǽ2ХȤ BOM ǤʤС
-       @c endian ˽äƥǥꤹ롣󥳡ɤκ
-      ˤ @c endian ˽äХ BOM դ롣
+      0: ǥɤκݤ˺ǽ 2 ХȤĴ٤롣⤷줬 BOM Ǥ
+      Сǥ򤽤Ƚꤹ롣ǤʤС @c
+      endian ˽äƥǥꤹ롣󥳡ɤκݤˤ @c
+      endian ˽äХƬ BOM դ롣
 
-      1: ǥɤκݡǽ2ХȤ BOM Ȥưʤ
-      ǥ @c endian Ƚꤹ롣󥳡ɤκݤˤϡBOM 
-      Ϥ@c endian ˱Х롣
+      1: ǥɤκݡǽ 2 ХȤ BOM Ȥư鷺ǥ
+       @c endian Ƚꤹ롣󥳡ɤκݤˤϡBOM Ϥ
+      @c endian ˱Х롣
 
-      2: ǥɤκݤ˺ǽ2ХȤ BOMȤư˽
-      ƥǥȽꤹ롣󥳡ɤκݤˤ @c endian 
-      Х BOM դ롣  */
+      2: ǥɤκݤ˺ǽ2ХȤ BOMȤư˽äƥ
+      ǥȽꤹ롣󥳡ɤκݤˤ @c endian ˱Х
+      Ƭ BOM դ롣  */
   int bom;
 
   /***en
@@ -722,14 +764,17 @@ typedef struct
       If <code_unit_bits> is 8, the value has no meaning.  */
   /***ja
       ǥΥפꤹ롣ͤ 0  1 Ǥꡢ0 ʤХ
-      륨ǥ1 ʤХӥåǥǤ롣*/
+      륨ǥ1 ʤХӥåǥǤ롣
+
+      <code_unit_bits>  8 ξˤϡͤϰ̣ʤ
+  */
   int endian;
 } MCodingInfoUTF;
 /*=*/
 
-extern MSymbol mconv_define_coding (char *name, MPlist *plist,
+extern MSymbol mconv_define_coding (const char *name, MPlist *plist,
 				    int (*resetter) (MConverter *),
-				    int (*decoder) (unsigned char *, int,
+				    int (*decoder) (const unsigned char *, int,
 						    MText *, MConverter *),
 				    int (*encoder) (MText *, int, int,
 						    unsigned char *, int,
@@ -779,14 +824,15 @@ extern int mconv_putc (MConverter *converter, int c);
 extern MText *mconv_gets (MConverter *converter, MText *mt);
 
 /* (S4) Locale related functions corresponding to libc functions */
-
+/*=*/
 /*** @ingroup m17nShell */
 /***en @defgroup m17nLocale Locale */
 /***ja @defgroup m17nLocale  */
 /*=*/
 
+/*** @ingroup m17nLocale */
 /***en
-    @brief @c struct @c MLocale
+    @brief @c struct @c MLocale.
 
     The structure @c MLocale is used to hold information about name,
     language, territory, modifier, codeset, and the corresponding
@@ -796,7 +842,7 @@ extern MText *mconv_gets (MConverter *converter, MText *mt);
     internal structure is concealed from application programs.  */
 
 /***ja
-    @brief @c MLocale ¤
+    @brief @c MLocale ¤.
 
     @c MLocale ¤Τϡ̾졢ϰ衢ǥե
     ɥåȡб륳ɷϤ˴ؤݻ뤿
@@ -844,24 +890,31 @@ extern MText *mtext_toupper (MText *mt);
 /*
  *  (10) Input method
  */
-
+/*=*/
 /*** @ingroup m17nShell */
 /***en @defgroup m17nInputMethod Input Method (basic) */
-/***ja @defgroup m17nInputMethod ϥ᥽å () */
+/***ja @defgroup m17nInputMethod ϥ᥽å (ʬ) */
 /*=*/
 
+/*** @addtogroup m17nInputMethod 
+     @{ */
+
 /* Struct forward declaration.  */
 typedef struct MInputMethod MInputMethod;
 typedef struct MInputContext MInputContext;
 
-/*** @ingroup m17nInputMethod */
-
 /***en
     @brief Type of input method callback functions.
 
     This is the type of callback functions called from input method
     drivers.  #IC is a pointer to an input context, #COMMAND is a name
     of callback for which the function is called.   */
+/***ja
+    @brief ϥ᥽åɥХåؿη.
+
+    ϥ᥽åɤƤФ륳ХåؿηǤ롣#IC ϥ
+    ƥȤؤΥݥ󥿡#COMMAND ϴؿƤФ륳Хå̾
+    Ǥ롣  */
 
 typedef void (*MInputCallbackFunc) (MInputContext *ic, MSymbol command);
 /*=*/
@@ -869,34 +922,34 @@ typedef void (*MInputCallbackFunc) (MInputContext *ic, MSymbol command);
 /***en
     @brief Structure of input method driver.
 
-    The type @c MInputDriver is the structure of an input driver that
+    The type @c MInputDriver is the structure of an input method driver that
     contains several functions to handle an input method.  */
 
 /***ja
-    @brief ϥɥ饤
+    @brief ϥɥ饤ѹ¤.
 
-    @c MInputDriver ϡϥ᥽åɤ갷ؿޤϥɥ饤
-    ι¤ΤǤ롣  */
+    @c MInputDriver ϡϥ᥽åɤ갷ؿޤϥ᥽åɥ
+    饤Фι¤ΤηǤ롣  */
 
 typedef struct MInputDriver
 {
   /***en
       @brief Open an input method.
 
-      This function opens the input method $IC.  It is called from the
+      This function opens the input method $IM.  It is called from the
       function minput_open_im () after all member of $IM but <info>
       set.  If opening $IM succeeds, it returns 0.  Otherwise, it
       returns -1.  The function can setup $IM->info to keep various
       information that is referred by the other driver functions.  */
 
   /***ja
-      @brief ϥ᥽åɤ򥪡ץ󤹤
+      @brief ϥ᥽åɤ򥪡ץ󤹤.
 
-      δؿϡϥ᥽å$IM򥪡ץ󤹤롣$IM <info>ʳ
-      СåȤ줿ǡؿ minput_open_im () ƤФ롣
-      $IM 򥪡ץǤ 0 򡢤Ǥʤ -1֤δؿ 
-      $IM->info ꤷơ¾Υɥ饤дؿ黲Ȥݻ
-      뤳ȤǤ롣
+      δؿϡϥ᥽å $IM 򥪡ץ󤹤롣$IM  <info> ʳ
+      СåȤ줿ǡؿ minput_open_im () Ƥ
+      롣$IM 򥪡ץǤ 0 򡢤Ǥʤ -1֤δ
+       $IM->info ꤷơ¾Υɥ饤дؿ黲Ȥ
+      ݻ뤳ȤǤ롣
       */
 
   int (*open_im) (MInputMethod *im);
@@ -911,12 +964,13 @@ typedef struct MInputDriver
       touched.  */
 
   /***ja
-      @brief ϥ᥽åɤ򥯥
+      @brief ϥ᥽åɤ򥯥.
 
-      δؿϴؿ minput_close_im () ƤФ졢ϥ᥽åɤ򥯥
-      롣ϥ᥽åɤΥ٤ƽλǡ
-      $IM->info ˳ƤƤ()롣
-      $IM ¾ΥФ˱ƶͿƤϤʤʤ       */
+      δؿϡϥ᥽å $IM 򥯥롣ؿ 
+      minput_close_im () ƤФ롣ϥ᥽åɤΥ٤
+      λǡ$IM->info ˳ƤƤ()
+      ٤Ƴ롣$IM ¾ΥФ˱ƶͿƤϤʤʤ
+        */
 
   void (*close_im) (MInputMethod *im);
 
@@ -931,13 +985,13 @@ typedef struct MInputDriver
       functions.  */
 
   /***ja
-      @brief ϥƥȤ
+      @brief ϥƥȤ.
 
-      ϥƥ $IC ؿδؿϡ$IC  <info> 
-      ʳСåȤ줿ǡؿ minput_create_ic () 
-      ƤФ롣$IC Ǥ 0 򡢤Ǥʤ -1 ֤
-      δؿ $IC->info ꤷơ¾Υɥ饤дؿ黲Ȥ
-      ݻ뤳ȤǤ롣  */
+      δؿϥƥ $IC 롣$IC  <info> ʳ
+      СåȤ줿ǡؿ minput_create_ic () Ƥ
+      롣$IC Ǥ 0 򡢤Ǥʤ -1 ֤δؿ
+       $IC->info ꤷơ¾Υɥ饤дؿ黲Ȥ
+      뤳ȤǤ롣  */
 
 
   int (*create_ic) (MInputContext *ic);
@@ -952,12 +1006,12 @@ typedef struct MInputDriver
       should not be touched.  */
 
   /***ja
-      @brief ϥƥȤ˲
+      @brief ϥƥȤ˲.
 
       ؿ minput_destroy_ic () ƤФ졢ϥƥ $IC 
       롣ϥƥȤ˲٤ƽλǡ$IC->info 
-      ˳ƤƤ()롣$IC ¾
-      Ф˱ƶͿƤϤʤʤ       */
+      ˳ƤƤ()٤Ƴ롣$IC 
+      ¾ΥФ˱ƶͿƤϤʤʤ  */
 
   void (*destroy_ic) (MInputContext *ic);
 
@@ -976,32 +1030,32 @@ typedef struct MInputDriver
       $IC->ncandidates, $IC->candidates, and $IC->produced if that is
       necessary for the member <callback>.
 
-      The meaning of $ARG depends on the input driver.  See the
+      The meaning of $ARG depends on the input method river.  See the
       documentation of @c minput_default_driver and @c
       minput_gui_driver for instance.  */
 
   /***ja
-      @brief ϥ٥Ȥ뤤ϥե륿
+      @brief ϥե륿.
 
       ؿ minput_filter () ƤФ졢ϥե륿롣 
       $KEY, $ARG ϴؿ minput_filter () ΤΤƱ
 
-      δؿ $KEY $IC ֤򥢥åץǡȤ롣 
-      $KEY ϥ᥽åɤ˵ۼƥƥȤʤä
-      ϡ 1 ֤Ǥʤ 0 ֤
+      δؿ $KEY $IC ֤򹹿롣 $KEY 
+      ᥽åɤ˵ۼƥƥȤʤäˤϡ 1 
+      Ǥʤ 0 ֤
 
        <callback> ɬפǤС$IC->status, $IC->preedit,
       $IC->cursor_pos, $IC->ncandidates, $IC->candidates,
-      $IC->produced 򥢥åץǡȤ뤳ȤǤ롣
+      $IC->produced 򹹿Ǥ롣
 
-      $ARG ΰ̣ϥɥ饤Ф˰¸롣 @c
-      minput_default_driver ޤ @c minput_gui_driver Υɥ
-      򻲾ȤΤȡ */
+      $ARG ΰ̣ϥ᥽åɥɥ饤Ф˰¸롣 @c
+      minput_default_driver ޤ @c minput_gui_driver 򻲾Ȥ
+      ȡ */
 
   int (*filter) (MInputContext *ic, MSymbol key, void *arg);
 
   /***en
-      @brief  Lookup a produced text in an input context
+      @brief  Lookup a produced text in an input context.
 
       It is called from the function minput_lookup () and looks up a
       produced text in the input context $IC.  This function
@@ -1009,21 +1063,21 @@ typedef struct MInputDriver
       M-text $MT.  If $KEY was correctly handled by the input method
       of $IC, it returns 0.  Otherwise, it returns 1.
 
-      The meaning of $ARG depends on the input driver.  See the
+      The meaning of $ARG depends on the input method driver.  See the
       documentation of @c minput_default_driver and @c
       minput_gui_driver for instance.  */
 
   /***ja
-      @brief ϥƥȤƥȤγ
+      @brief ϥƥȤƥȤγ.
 
       ؿ minput_lookup () ƤФ졢ϥƥ $IC 
       ƥȤ򸡺롣ϥ $KEY ˤäƥ
-      С$IC->produced ꤹ롣 $KEY ϥ᥽å $IC 
-      ä 0 ֤Ǥʤ 1 ֤
+      СM-text $MT ɲä롣 $KEY ϥ᥽å $IC ˤ
+       0 ֤Ǥʤ 1 ֤
 
-      $ARG ΰ̣ϥɥ饤Ф˰¸롣 @c
-      minput_default_driver ޤ @c minput_gui_driver Υɥ
-      򻲾ȤΤȡ */
+      $ARG ΰ̣ϥ᥽åɥɥ饤Ф˰¸롣 @c
+      minput_default_driver ޤ @c minput_gui_driver 򻲾Ȥ
+      ȡ */
 
   int (*lookup) (MInputContext *ic, MSymbol key, void *arg, MText *mt);
 
@@ -1035,8 +1089,18 @@ typedef struct MInputDriver
       #Minput_preedit_done, #Minput_status_start, #Minput_status_draw,
       #Minput_status_done, #Minput_candidates_start,
       #Minput_candidates_draw, #Minput_candidates_done,
-      #Minput_set_spot, and #Minput_toggle.  Values are functions of
-      type #MInputCallbackFunc.  */
+      #Minput_set_spot, #Minput_toggle, #Minput_reset.
+      Values are functions of type #MInputCallbackFunc.  */
+  /***ja
+      @brief ХåؿΥꥹ.
+
+      ХåؿΥꥹȡϼΤ줫
+      #Minput_preedit_start, #Minput_preedit_draw,
+      #Minput_preedit_done, #Minput_status_start, #Minput_status_draw,
+      #Minput_status_done, #Minput_candidates_start,
+      #Minput_candidates_draw, #Minput_candidates_done,
+      #Minput_set_spot, #Minput_toggle, #Minput_reset 
+      ͤ#MInputCallbackFunc δؿ  */
   MPlist *callback_list;
 
 } MInputDriver;
@@ -1060,6 +1124,7 @@ extern MSymbol Minput_candidates_draw;
 extern MSymbol Minput_candidates_done;
 extern MSymbol Minput_set_spot;
 extern MSymbol Minput_toggle;
+extern MSymbol Minput_reset;
 
 /***en
     @brief Structure of input method.
@@ -1067,37 +1132,37 @@ extern MSymbol Minput_toggle;
     The type @c MInputMethod is the structure of input method
     objects.  */
 /***ja
-    @brief ϥ᥽åѹ¤
+    @brief ϥ᥽åɤι¤.
 
-    @c MInputMethod ϡϥ᥽åɥ֥Ѥι¤ΤǤ롣  */
+    @c MInputMethod ϡϥ᥽åɥ֥Ѥι¤ΤηǤ롣  */
 
 struct MInputMethod
 {
   /***en Which language this input method is for.  The value is @c
       Mnil if the input method is foreign.  */
   /***ja ɤθѤϥ᥽åɤϥ᥽åɤΤΤǤ
-      ˤͤȤ @c ġ  */
+      ˤͤȤ @c Mnil ġ  */
   MSymbol language;
 
   /***en Name of the input method.  If the input method is foreign, it
       must has a property of key @c Minput_driver and the value must be a
-      pointer to a proper input driver.  */
+      pointer to a proper input method driver.  */
   /***ja ϥ᥽åɤ̾᥽åɤǤˤϡ@c
       Minput_driver 򥭡ȤץѥƥͤŬڤϥ
-      饤ФؤΥݥ󥿤ǤʤƤϤʤʤ*/
+      åɥɥ饤ФؤΥݥ󥿤ǤʤƤϤʤʤ*/
   MSymbol name;
 
-  /***en Input driver of the input method.  */
-  /***ja ϥ᥽åѤϥɥ饤  */
+  /***en Input method driver of the input method.  */
+  /***ja ϥ᥽åѤϥ᥽åɥɥ饤С  */
   MInputDriver driver;
 
   /***en The argument given to minput_open_im (). */
-  /***ja minput_open_im () Ϳ  */
+  /***ja minput_open_im () Ϥ  */
   void *arg;
 
   /***en Pointer to extra information that <driver>.open_im ()
       setups. */
-  /***ja <driver>.open_im () ꤹɲþؤΥݥ */
+  /***ja <driver>.open_im () ꤹɲþؤΥݥ󥿡 */
   void *info;
 };
 
@@ -1110,9 +1175,9 @@ struct MInputMethod
     objects.  */
 
 /***ja
-    @brief ϥƥѹ¤
+    @brief ϥƥѹ¤.
 
-    @c MInputContext ϡϥƥȥ֥Ѥι¤ΤǤ
+    @c MInputContext ϡϥƥȥ֥Ѥι¤Τη
     롣  */
 
 struct MInputContext
@@ -1139,35 +1204,41 @@ struct MInputContext
       ().  */
   /***ja ϥƥȤƥ֤ɤ򼨤ե饰ϥ
       Ȥ줿Ǥͤ 1 ʥƥ֡ˤǤꡢؿ 
-      minput_toggle () ˤäƥȥǤ롣  */
+      minput_toggle () ˤäƥȥ뤵롣  */
   int active;
 
 
   /***en Spot location and size of the input context.  */
-  /***ja ϥƥȤΥݥåȤΰ֤礭  */
+  /***ja ϥƥȤΥݥåȤΰ֤礭.  */
   struct {
     /***en X and Y coordinate of the spot.  */
-    /***ja ݥåȤ X, Y ɸ  */
+    /***ja ݥåȤ X, Y ɸ.  */
     int x, y;
+
     /***en Ascent and descent pixels of the line of the spot.  */
-    /***ja ݥåȤΥȤȥǥȤΥԥ  */
+    /***ja ݥåȤΥȤȥǥȤΥԥ.  */
     int ascent, descent;
 
     /***en Font size for preedit text in 1/10 point.  */
+    /***ja preedit ƥѤΥեȥ (1/10 ݥñ).  */
     int fontsize;
 
     /***en M-text at the spot, or NULL.  */
+    /***ja ݥåȾ M-textޤ NULL.  */
     MText *mt;
+
     /***en Character position in <mt> at the spot.  */
+    /***ja <mt> ˤ륹ݥåȤʸ.  */
     int pos;
   } spot;
 
   /***en The usage of the following members depends on the input
-      driver.  The descriptions below are for the input driver of an
+      method driver.  The descriptions below are for the driver of an
       internal input method.  They are set by the function
       <im>->driver.filter ().  */
-  /***ja ʲΥФλˡϥɥ饤Фˤäưۤʤ롣ʲ
-      ϡϥ᥽åѤϥɥ饤ФФΤǤ롣 */
+  /***ja ʲΥФλˡϥ᥽åɥɥ饤Фˤäưۤʤ롣
+      ϡϥ᥽åѤϥɥ饤ФФΤǤ롣
+      ϴؿ <im>->driver.filter () ˤäꤵ롣  */
 
   /***en Pointer to extra information that <im>->driver.create_ic ()
       setups.  It is used to record the internal state of the input
@@ -1183,15 +1254,20 @@ struct MInputContext
 
   /***en The function <im>->driver.filter () sets the value to 1 when
       it changes <status>.  */
+  /***ja ؿ <im>->driver.filter () ϡ<status> Ѥͤ 1 
+      ꤹ롣  */
   int status_changed;
 
   /***en M-text containing the current preedit text.  The function
       <im>->driver.filter () sets the value.  */
-  /***ja ߤ preedit ƥȤޤ M-text */
+  /***ja ߤ preedit ƥȤޤ M-textؿ 
+      <im>->driver.filter () ͤꤹ롣  */ 
   MText *preedit;
 
   /***en The function <im>->driver.filter () sets the value to 1 when
       it changes <preedit>.  */
+  /***ja ؿ <im>->driver.filter () ϡ<preedit> Ѥͤ 
+      1 ꤹ롣  */
   int preedit_changed;
 
   /***en Cursor position of <preedit>.  */
@@ -1201,7 +1277,7 @@ struct MInputContext
   int cursor_pos_changed;
 
   /***en Array of the current candidate texts.  */
-  /***ja ߤΥƥȸΥ롼פΥꥹ  */
+  /***ja ߤΥƥȸ.  */
   MPlist *candidate_list;
   int candidate_index;
   int candidate_from, candidate_to;
@@ -1209,6 +1285,8 @@ struct MInputContext
 
   /***en The function <im>->driver.filter () sets the value to 1 when
       it changes one of the above members.  */
+  /***ja ؿ <im>->driver.filter () ϡ嵭ΥФΣĤѤ
+      ͤ 1 ꤹ롣  */
   int candidates_changed;
 
   MPlist *plist;
@@ -1238,12 +1316,13 @@ extern void minput_set_spot (MInputContext *ic, int x, int y,
 
 extern void minput_toggle (MInputContext *ic);
 
-extern MSymbol minput_char_to_key (int c);
-
+extern void minput_reset_ic (MInputContext *ic);
 /*=*/
+/***     @} */
 
 extern MInputMethod *mdebug_dump_im (MInputMethod *im, int indent);
 
+
 #ifdef __cplusplus
 }
 #endif
