@@ -33,6 +33,19 @@
     If the key of a property is a @e managing @e key, its @e value is
     a @e managed @e object.  A property list itself is a managed
     objects.  */
+/***ja
+    @addtogroup m17nPlist
+
+    @brief ץѥƥꥹȥ֥ȤȤ˴ؤ API.
+
+    @e ץѥƥꥹ (ޤ @e plist)  0 İʾΥץѥƥΥ
+    ȤǤ롣ץѥƥ @e   @e  ʤ롣ϥܥ
+    Ǥꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤвǤ
+    
+
+    ץѥƥΥ @e  ʤС @e  @e 
+    ֥ Ǥ롣ץѥƥꥹȼΤ֥ȤǤ
+    롣  */
 
 /*=*/
 
@@ -43,6 +56,7 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "config.h"
 #include "m17n.h"
 #include "m17n-misc.h"
 #include "internal.h"
@@ -721,7 +735,12 @@ mplist__serialize (MText *mt, MPlist *plist)
     @brief Symbol whose name is "integer".
 
     The symbol @c Minteger has the name <tt>"integer"</tt>.  A value
-    of a plist whose key is @c Minteger must be an integer.  */
+    of a property whose key is @c Minteger must be an integer.  */
+/***ja
+    @brief "integer" ̾Ȥƻĥܥ.
+
+    ܥ @c Minteger  <tt>"integer"</tt> Ȥ̾ġ
+    @c Minteger ǤץѥƥͤͤǤʤƤϤʤʤ  */
 
 MSymbol Minteger;
 /*=*/
@@ -730,8 +749,14 @@ MSymbol Minteger;
     @brief Symbol whose name is "plist".
 
     The symbol @c Mplist has the name <tt>"plist"</tt>.  It is a
-    managing key.  A value of a plist whose key is @c Mplist must be a
-    plist.  */
+    managing key.  A value of a property whose key is @c Mplist must
+    be a plist.  */
+/***ja
+    @brief "plist" ̾Ȥƻĥܥ.
+
+    ܥ @c Mplist  <tt>"plist"</tt> Ȥ̾ġϴ
+    Ǥ롣 @c Mplist Ǥץѥƥͤ plist Ǥʤ
+    ƤϤʤʤ  */
 
 MSymbol Mplist;
 /*=*/
@@ -740,18 +765,18 @@ MSymbol Mplist;
     @brief Symbol whose name is "mtext".
 
     The symbol @c Mtext has the name <tt>"mtext"</tt>.  It is a
-    managing key.  A value of a plist whose key is @c Mtext must be an
+    managing key.  A value of a property whose key is @c Mtext must be an
     M-text.  */
 
 /***ja
-    @brief "text" ̾Ȥƻĥܥ
+    @brief "mtext" ̾Ȥƻĥܥ.
 
-    Ѥߥܥ @c Mtext  <tt>"text"</tt> Ȥ̾Ĵ
-    Ǥ롣 */
+    ܥ @c Mtext  <tt>"mtext"</tt> Ȥ̾ĴǤ
+    롣 @c Mtext Ǥץѥƥͤ M-text ǤʤƤϤʤ
+          */
 
 MSymbol Mtext;
 
-
 /*=*/
 /***en
     @brief Create a property list object.
@@ -764,6 +789,17 @@ MSymbol Mtext;
 
     @errors
     This function never fails.  */
+/***ja
+    @brief ץѥƥꥹȥ֥Ȥ.
+
+    ؿ mplist () Ĺ 0 ο줿ץѥƥꥹȥ֥
+    Ȥ֤
+
+    @returns
+    δؿϿ줿ץѥƥꥹȥ֥Ȥ֤
+
+    @errors
+    δؿϷ褷ƼԤʤ     */
 
 MPlist *
 mplist ()
@@ -776,17 +812,28 @@ mplist ()
 
 /*=*/
 /***en
-    @brief Copy a plist.
+    @brief Copy a property list.
 
-    The mplist_copy () function copies $PLIST.  In the copy, the
-    values are the same as those of $PLIST.
+    The mplist_copy () function copies property list $PLIST.  In the
+    copy, the values are the same as those of $PLIST.
 
     @return
     This function returns a newly created plist which is a copy of
-    $PLIST.  */
-/***
+    $PLIST.  
+
     @errors
     This function never fails.  */ 
+/***ja
+    @brief ץѥƥꥹȤ򥳥ԡ.
+
+    ؿ mplist_copy () ϥץѥƥꥹ $PLIST 򥳥ԡ롣ԡΤ٤Ƥ
+    ͤϥԡ $PLIST ͤƱǤ롣
+
+    @return
+    δؿϿ줿$PLIST ΥԡǤץѥƥꥹȤ֤    
+
+    @errors
+    δؿϷ褷ƼԤʤ     */
 
 MPlist *
 mplist_copy (MPlist *plist)
@@ -801,9 +848,9 @@ mplist_copy (MPlist *plist)
 /*=*/
 
 /***en
-    @brief Set the value of a property in a property list object.
+    @brief Set the value of a property in a property list.
 
-    The mplist_put () function searches property list object $PLIST
+    The mplist_put () function searches property list $PLIST
     from the beginning for a property whose key is $KEY.  If such a
     property is found, its value is changed to $VALUE.  Otherwise, a
     new property whose key is $KEY and value is $VALUE is appended at
@@ -818,6 +865,22 @@ mplist_copy (MPlist *plist)
     If the operation was successful, mplist_put () returns a sublist of
     $PLIST whose first element is the just modified or added one.
     Otherwise, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹΥץѥƥͤꤹ.
+
+    ؿ mplist_put () ϥץѥƥꥹ $PLIST Ϥᤫõơ
+     $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤ $VALUE 
+    ѹ롣ĤʤС $KEY ͤ $VALUE Ǥ뿷
+    ץѥƥ$PLIST ɲä롣$KEY  $VAL Ф¤
+    ĤƤϡmplist_add () 򻲾ȡ
+
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
+    ξ硢Ťͤλȿ @c NULL Ǥʤ 1 餵졢$VAL 
+    ȿ 1 䤵롣
+
+    @return 
+      mplist_put () ѹ줿ɲä줿Ǥ
+    ޤ $PLIST ʬꥹȤ֤Ǥʤ @c NULL ֤   */
 
 MPlist *
 mplist_put (MPlist *plist, MSymbol key, void *val)
@@ -838,9 +901,9 @@ mplist_put (MPlist *plist, MSymbol key, void *val)
 /*=*/
 
 /***en
-    @brief Get the value of a property in a property list object.
+    @brief Get the value of a property in a property list.
 
-    The mplist_get () function searches property list object $PLIST
+    The mplist_get () function searches property list $PLIST
     from the beginning for a property whose key is $KEY.  If such a
     property is found, a pointer to its value is returned as the type
     of <tt>(void *)</tt>.  If not found, @c NULL is returned.
@@ -850,6 +913,17 @@ mplist_put (MPlist *plist, MSymbol key, void *val)
     where a property is found and its value is @c NULL.  In case that
     these two cases must be distinguished, use the mplist_find_by_key ()
     function.  */
+/***ja
+    @brief ץѥƥꥹΥץѥƥͤ.
+
+    ؿ mplist_get () ϡץѥƥꥹ $PLIST Ϥᤫõơ
+     $KEY Ǥץѥƥ򸫤Ĥ롣ĤСͤؤΥݥ
+    󥿤 <tt>(void *)</tt> ֤Ĥʤ @c NULL ֤
+
+    @c NULL ֤äݤˤĤβǽ:嵭Τ褦˥ץѥƥ
+    ĤʤäȡץѥƥĤꡢͤ @c NULL 
+    Ǥ롣̤ɬפˤϴؿ 
+    mplist_find_by_key () Ȥȡ  */
 
 /***
     @seealso
@@ -865,11 +939,11 @@ mplist_get (MPlist *plist, MSymbol key)
 /*=*/
 
 /***en
-    @brief Add a property at the end of a property list object.
+    @brief Add a property at the end of a property list.
 
-    The mplist_add () function appends at the end of $PLIST a property
-    whose key is $KEY and value is $VAL.  $KEY can be any symbol
-    other than @c Mnil.
+    The mplist_add () function appends at the end of property list
+     $PLIST a property whose key is $KEY and value is $VAL.  $KEY can
+     be any symbol other than @c Mnil.
 
     If $KEY is a managing key, $VAL must be a managed object.  In this
     case, the reference count of $VAL is incremented by one.
@@ -878,6 +952,19 @@ mplist_get (MPlist *plist, MSymbol key)
     If the operation was successful, mplist_add () returns a sublist of
     $PLIST whose first element is the just added one.  Otherwise, it
     returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹ˥ץѥƥɲä.
+
+    ؿ mplist_add () ϡץѥƥꥹ $PLIST ˥ $KEY 
+    ͤ $VAL Ǥץѥƥɲä롣$KEY ϡ@c Mnil ʳǤ
+    դΥܥǤ褤
+
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
+    ξ硢$VAL λȿ 1 䤵롣
+
+    @return
+      mplist_add () ɲä줿ǤϤޤ $PLIST 
+    ʬꥹȤ֤Ǥʤ @c NULL ֤  */
 
 MPlist *
 mplist_add (MPlist *plist, MSymbol key, void *val)
@@ -896,10 +983,10 @@ mplist_add (MPlist *plist, MSymbol key, void *val)
 /*=*/
 
 /***en
-    @brief Push a property to a property list object.
+    @brief Add a property at the beginning of a property list.
 
-    The mplist_push () function pushes at the top of $PLIST a
-    property whose key is $KEY and value si $VAL.
+    The mplist_push () function inserts at the beginning of property
+    list $PLIST a property whose key is $KEY and value is $VAL.
 
     If $KEY is a managing key, $VAL must be a managed object.  In this
     case, the reference count of $VAL is incremented by one.
@@ -907,6 +994,18 @@ mplist_add (MPlist *plist, MSymbol key, void *val)
     @return
     If the operation was successful, this function returns $PLIST.
     Otherwise, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹȤƬ˥ץѥƥ.
+
+    ؿ mplist_push () ϥץѥƥꥹ $PLIST Ƭ˥ $KEY 
+    ͤ $VAL Ǥ륪֥Ȥ롣
+
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
+    ξ硢$VAL λȿ 1 䤵롣
+
+    @return
+    Фδؿ $PLIST ֤Ǥʤ@c NULL 
+    ֤  */
 
 MPlist *
 mplist_push (MPlist *plist, MSymbol key, void *val)
@@ -918,7 +1017,7 @@ mplist_push (MPlist *plist, MSymbol key, void *val)
   MPLIST_NEW (pl);
   MPLIST_KEY (pl) = MPLIST_KEY (plist);
   MPLIST_VAL (pl) = MPLIST_VAL (plist);
-  pl->next = plist->next;
+  MPLIST_NEXT (pl) = MPLIST_NEXT (plist);
   plist->next = pl;
   if (key->managing_key)
     M17N_OBJECT_REF (val);
@@ -930,15 +1029,25 @@ mplist_push (MPlist *plist, MSymbol key, void *val)
 /*=*/
 
 /***en
-    @brief Pop a property from a property list object.
+    @brief Remove a property at the beginning of a property list.
 
-    The mplist_pop () function pops the topmost property from $PLIST.
-    As a result, the key and value of $PLIST becomes those of the next
-    of $PLIST.
+    The mplist_pop () function removes a property at the beginning of
+    property list $PLIST.  As a result, the second key and value of
+    the original $PLIST become the first of those of the new $PLIST.
 
     @return
     If the operation was successful, this function return the value of
     the just popped property.  Otherwise, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹȤƬץѥƥ.
+
+    ؿ mplist_pop () ϥץѥƥꥹ $PLIST ƬΥץѥƥ
+    롣̤Ȥơ $PLIST 2ܤΥͤ 
+    $PLIST ƬΥͤˤʤ롣
+
+    @return 
+    СδؿϺ줿ץѥƥ֤ͤ
+    Ǥʤ @c NULL ֤  */
 
 void *
 mplist_pop (MPlist *plist)
@@ -949,14 +1058,14 @@ mplist_pop (MPlist *plist)
   if (MPLIST_TAIL_P (plist))
     return NULL;
   val = MPLIST_VAL (plist);
-  next = plist->next;
+  next = MPLIST_NEXT (plist);
   MPLIST_KEY (plist) = MPLIST_KEY (next);
   MPLIST_VAL (plist) = MPLIST_VAL (next);
   if (MPLIST_KEY (plist) != Mnil
       && MPLIST_KEY (plist)->managing_key
       && MPLIST_VAL (plist))
     M17N_OBJECT_REF (MPLIST_VAL (plist));
-  plist->next = next->next;
+  MPLIST_NEXT (plist) = MPLIST_NEXT (next);
   if (plist->next)
     M17N_OBJECT_REF (plist->next);
   M17N_OBJECT_UNREF (next);
@@ -965,15 +1074,25 @@ mplist_pop (MPlist *plist)
 
 /*=*/
 /***en
-    @brief Find a property of a specific key in a property list object.
+    @brief Find a property of a specific key in a property list.
 
-    The mplist_find_by_key () function searches property list object
+    The mplist_find_by_key () function searches property list
     $PLIST from the beginning for a property whose key is $KEY.  If
     such a property is found, a sublist of $PLIST whose first element
     is the found one is returned.  Otherwise, @c NULL is returned.
 
-    If $KEY is Mnil, it returns the last a sublist of $PLIST whose
+    If $KEY is @c Mnil, it returns a sublist of $PLIST whose
     first element is the last one of $PLIST.  */
+/***ja
+    @brief ץѥƥꥹ椫Υĥץѥƥõ.
+
+    ؿ mplist_find_by_key () ϥץѥƥꥹ $PLIST Ϥᤫõ
+    ơ $KEY Ǥץѥƥ򸫤Ĥ롣ĤСΥץ
+    ѥƥϤޤ $PLIST ʬꥹȤ֤Ǥʤ@c NULL 
+    ֤
+
+    $KEY  @c Mnil ʤС$PLIST κǸǤϤޤʬꥹȤ
+    ֤  */
 
 MPlist *
 mplist_find_by_key (MPlist *plist, MSymbol key)
@@ -986,12 +1105,19 @@ mplist_find_by_key (MPlist *plist, MSymbol key)
 
 /*=*/
 /***en
-    @brief Find a property of a specific value in a property list object.
+    @brief Find a property of a specific value in a property list.
 
-    The mplist_find_by_value () function searches property list object
-    $PLIST from the beginning for a property whose value is $VAL.  If
-    such a property is found, a sublist of $PLIST whose first element
-    is the found one is returned.  Otherwise, @c NULL is returned.  */
+    The mplist_find_by_value () function searches property list $PLIST
+    from the beginning for a property whose value is $VAL.  If such a
+    property is found, a sublist of $PLIST whose first element is the
+    found one is returned.  Otherwise, @c NULL is returned.  */
+/***ja
+    @brief ץѥƥꥹ椫ͤĥץѥƥõ.
+
+    ؿ mplist_find_by_value () ϥץѥƥꥹ $PLIST Ϥᤫ
+    õơͤ $VAL Ǥץѥƥ򸫤Ĥ롣ĤСΥץ
+    ѥƥϤޤ $PLIST ʬꥹȤ֤Ǥʤ@c NULL 
+    ֤ */
 
 MPlist *
 mplist_find_by_value (MPlist *plist, void *val)
@@ -1007,11 +1133,17 @@ mplist_find_by_value (MPlist *plist, void *val)
 /*=*/
 
 /***en
-    @brief Return the next sublist of a plist.
+    @brief Return the next sublist of a property list.
 
     The mplist_next () function returns a pointer to the sublist of
-    $PLIST, which begins at the second element in $PLIST.  If the
+    property list $PLIST, which begins at the second element in $PLIST.  If the
     length of $PLIST is zero, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹȤμʬꥹȤ֤.
+
+    ؿ mplist_next () ϥץѥƥꥹ $PLIST Σ
+    ǤϤޤʬꥹȤؤΥݥ󥿤֤$PLIST Ĺ 0 
+     @c NULL ֤  */
 
 MPlist *
 mplist_next (MPlist *plist)
@@ -1022,16 +1154,26 @@ mplist_next (MPlist *plist)
 /*=*/
 
 /***en
-    @brief Set the first property in a property list object.
+    @brief Set the first property in a property list.
 
-    The mplist_set () function sets the key and value of the first
-    property in property list object $PLIST to $KEY and $VALUE,
-    respectively.  See the documentation of mplist_add () for the
-    restriction on $KEY and $VAL.
+    The mplist_set () function sets the key and the value of the first
+    property in property list $PLIST to $KEY and $VALUE, respectively.
+    See the documentation of mplist_add () for the restriction on $KEY
+    and $VAL.
 
     @return
     If the operation was successful, mplist_set () returns $PLIST.
     Otherwise, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹȤκǽΥץѥƥꤹ.
+
+    ؿ mplist_set () ϥץѥƥꥹ $PLIST κǽΥץѥƥ
+    ͤ򤽤줾 $KEY  $VALUE ꤹ롣$KEY  $VAL Ф
+    ¤ˤĤƤϡmplist_add () 򻲾ȡ
+
+    @return
+     mplist_set ()  $PLIST ֤Ǥʤ
+    @c NULL ֤  */
 
 MPlist *
 mplist_set (MPlist *plist, MSymbol key, void * val)
@@ -1064,10 +1206,15 @@ mplist_set (MPlist *plist, MSymbol key, void * val)
 /*=*/
 
 /***en
-    @brief Return the length of a plist.
+    @brief Return the length of a property list.
 
     The mplist_length () function returns the number of properties in
-    property list object $PLIST.  */
+    property list  $PLIST.  */
+/***ja
+    @brief ץѥƥꥹȤĹ֤.
+
+    ؿ mplist_length () ϥץѥƥꥹ $PLIST Υץѥƥο
+    ֤  */
 
 int
 mplist_length (MPlist *plist)
@@ -1081,11 +1228,17 @@ mplist_length (MPlist *plist)
 /*=*/
 
 /***en
-    @brief Return the key of the first property in a property list object.
+    @brief Return the key of the first property in a property list.
 
     The mplist_key () function returns the key of the first property
-    in property list object $PLIST.  If the length of $PLIST is zero,
+    in property list $PLIST.  If the length of $PLIST is zero,
     it returns @c Mnil.  */
+/***ja
+    @brief ץѥƥꥹκǽΥץѥƥΥ֤.
+
+    ؿ mplist_key () ϡץѥƥꥹ $PLIST κ
+    ΥץѥƥΥ֤$PLIST Ĺ 0 ʤС @c Mnil 
+    ֤  */
 
 MSymbol
 mplist_key (MPlist *plist)
@@ -1096,11 +1249,17 @@ mplist_key (MPlist *plist)
 /*=*/
 
 /***en
-    @brief Return the value of the first property in a property list object.
+    @brief Return the value of the first property in a property list.
 
     The mplist_value () function returns the value of the first
-    property in property list object $PLIST.  If the length of $PLIST
+    property in property list  $PLIST.  If the length of $PLIST
     is zero, it returns @c NULL.  */
+/***ja
+    @brief ץѥƥꥹκǽΥץѥƥ֤ͤ.
+
+    ؿ mplist_value () ϡץѥƥꥹ $PLIST 
+    ǽΥץѥƥ֤ͤ$PLIST Ĺ 0 ʤС @c Mnil 
+    ֤  */
 
 void *
 mplist_value (MPlist *plist)
@@ -1109,7 +1268,7 @@ mplist_value (MPlist *plist)
 }
 
 /***en
-    @brief Generate a plist by deserializaing an M-text.
+    @brief Generate a property list by deserializaing an M-text.
 
     The mplist_deserialize () function parses M-text $MT and returns a
     property list.
@@ -1127,23 +1286,56 @@ mplist_value (MPlist *plist)
 
     M-TEXT ::= '"' character-sequence '"'
 
-    Each kind of @c ELEMENT is assigned one of these keys:
-	@c Msymbol, @c Minteger, @c Mtext, @c Mplist
+    Each alternatives of @c ELEMENT is assigned one of these keys: @c
+    Msymbol, @c Minteger, @c Mtext, @c Mplist
 
     In an ascii-character-sequence, a backslush (\) is used as the escape
     character, which means that, for instance, <tt>"abc\ def"</tt>
     produces a symbol whose name is of length seven with the fourth
     character being a space.  */
+/***ja
+    @brief M-text ǥꥢ饤ƥץѥƥꥹȤ.
+
+    ؿ mplist_deserialize ()  M-text $MT Ϥƥץѥƥꥹ
+    Ȥ֤
+
+    $MT Υ󥿥åϰʲ̤ꡣ
+
+    MT ::= '(' ELEMENT * ')'
+
+    ELEMENT ::= SYMBOL | INTEGER | M-TEXT | PLIST
+
+    SYMBOL ::= ʸ
+
+    INTEGER ::= '-' ? [ '0' | .. | '9' ]+
+		| '0x' [ '0' | .. | '9' | 'A' | .. | 'F' | 'a' | .. | 'f' ]+
+
+    M-TEXT ::= '"' character-sequence '"'
+
+    @c ELEMENT γϥ@c Msymbol, @c Minteger, @c Mtext,
+    @c Mplist Τ줫ƤƤ롣
+
+    ʸǤϡХåå (\) ʸȤ
+    Ѥ롣Ȥ <tt>"abc\ def"</tt> ϣʸܤʸǤ
+    ĹǤ̾ĥܥ롣   */
 
 MPlist *
 mplist_deserialize (MText *mt)
 {
+  MPlist *plist;
+  MText *tmp = NULL;
+
   if (mt->format > MTEXT_FORMAT_UTF_8)
     {
-      if (mtext__adjust_format (mt, MTEXT_FORMAT_UTF_8) < 0)
-	MERROR (MERROR_PLIST, NULL);
+      if (MTEXT_READ_ONLY_P (mt))
+	mt = tmp = mtext_cpy (mtext (), mt);
+      else
+	mtext__adjust_format (mt, MTEXT_FORMAT_UTF_8);
     }
-  return mplist__from_string (MTEXT_DATA (mt), mtext_nbytes (mt));
+  plist = mplist__from_string (MTEXT_DATA (mt), mtext_nbytes (mt));
+  if (tmp)
+    M17N_OBJECT_UNREF (tmp);
+  return plist;
 }
 
 /*** @}  */
@@ -1153,15 +1345,23 @@ mplist_deserialize (MText *mt)
 /*** @{  */
 
 /***en
-    @brief Dump a plist.
+    @brief Dump a property list.
 
-    The mdebug_dump_plist () function prints $PLIST in a human
-    readable way to the stderr.  $INDENT specifies how many columns to
-    indent the lines but the first one.
+    The mdebug_dump_plist () function prints a property list $PLIST in
+    a human readable way to the stderr.  $INDENT specifies how many
+    columns to indent the lines but the first one.
 
     @return
     This function returns $PLIST.  */
+/***ja
+    @brief ץѥƥꥹȤפ.
 
+    ؿ mdebug_dump_plist () ϥץѥƥꥹ $PLIST  stderr ˿
+    ֤˲ɤ ǰ롣 $UNDENT ϣܰʹߤΥǥȤꤹ
+    롣
+
+    @return
+    δؿ $PLIST ֤  */
 MPlist *
 mdebug_dump_plist (MPlist *plist, int indent)
 {
