@@ -29,8 +29,8 @@
     identifiers.  Symbols are similar to atoms in the X library, but a
     symbol can have zero or more @e symbol @e properties.  A symbol
     property consists of a @e key and a @e value, where key is also a
-    symbol and value is anything that can be cast to <tt>(void
-    *)</tt>.  "The symbol property that belongs to the symbol S and
+    symbol and value is anything that can be cast to <tt>(void *)</tt>.  
+    "The symbol property that belongs to the symbol S and
     whose key is K" may be shortened to "K property of S".
 
     Symbols are used mainly in the following three ways.
@@ -47,41 +47,32 @@
     value of a property whose key is a managing key must be a @e
     managed @e object.  See @ref m17nObject for the detail.
 */
-
 /***ja
     @addtogroup m17nSymbol ܥ
 
-    @brief ܥ륪֥ȤȤ˴ؤ API
+    @brief ܥ륪֥ȤȤ˴ؤ API.
 
-    m17n 饤֥ϰդ˷ޤ뵭һҤȤ @e ܥ ȸƤ֥֥
+    m17n 饤֥ϰդ˷ޤ뼱̻ҤȤ @e ܥ ȸƤ֥֥
     ȤѤ롣ܥ X 饤֥ΥȥȻƤ뤬0 İ
-     @e ܥץѥƥ ĤȤǤǡȥⵡ
-    ǽǤ롣ܥץѥƥ @e   @e  ʤ롣Ϥ
-    켫ΥܥǤꡢͤ <tt>(void *)</tt> ˥㥹ȤǤ
-    Τʤ鲿Ǥ褤֥ܥ S ĥܥץѥƥΤ
-     K ΤΡפñˡS  K ץѥƥפȸƤ֤Ȥ롣
+     @e ܥץѥƥ ĤȤǤ롣ܥץѥƥ
+     @e   @e  ʤ롣Ϥ켫ΥܥǤꡢͤ 
+    <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤֥ܥ 
+    S ĥܥץѥƥΤ K ΤΡפñˡS  K 
+    ץѥƥפȸƤ֤Ȥ롣
 
     ܥӤϼ˰ʲ3̤Ǥ롣
 
-    @li ܥץѥƥ (¾Υץѥƥ) Υɽ魯
+    @li ܥץѥƥ¾ΥץѥƥΥɽ魯
 
     @li ʸåȡɷϡեȥåȤʤɤγƼ索֥Ȥɽ
     魯
 
     @li m17n 饤֥ؿΰȤʤꡢؿεư椹롣
 
-     @c Mmanaging_key ͤ @c NULL ʳΥܥץѥƥ
-    ĥܥϡ@e  ȸƤФ롣륷ܥץѥƥΥ
-    ξ硢ͤ @e ֥ Ȥư롣
-    ֥Ȥ˴ؤܺ٤ @e ֥ ξϤ򻲾
-    Τȡ
-
-    ܥץѥƥˤͤ @e ץХؿ 뤳
-    Ǥ롣ץХؿϤΥͤ褦Ȥ˸ƤФ졢
-    ֤ͤ륷ܥץѥƥͤȤʤ롣λȤߤˤäƥ
-    ܥץѥƥٱɾǽˤʤ롣ץХؿ֤ͤ
-    Ʊͤꤹ뤳ȤǤξˤϡץХؿΤ
-    ʹߤ̵Ȥʤ롣  */
+    @e  ȸƤФ̤ʥܥ뤬ꡢ򥭡Ȥƻ
+    ĥץѥƥͤ @e ֥ ǤʤƤϤʤʤܺ
+     @ref m17nObject ȡ
+*/
 
 /*=*/
 
@@ -278,10 +269,11 @@ MTextPropDeserializeFunc msymbol__deserializer = deserialize_symbol;
     zero.  #Mnil can't have any symbol property.  */
 
 /***ja
-    @brief ѥܥ Mnil
+    @brief "nil" ̾Ȥƻĥܥ.
 
-    ܥ #Mnil  <tt>"nil"</tt> Ȥ̤̾ˡֵ
-    ̣롣#Mnil ȤϤʤ륷ܥץѥƥʤ  */
+    ܥ #Mnil  <tt>"nil"</tt> Ȥ̤̾ˡֵפ
+    ϡפ̣롣"int" Ѵ줿硢ͤ 0 Ǥ롣
+    #Mnil ȤϤʤ륷ܥץѥƥʤ  */
 
 MSymbol Mnil;
 
@@ -294,10 +286,10 @@ MSymbol Mnil;
     represents @e true or @e yes.  */
 
 /***ja
-    @brief ѥܥ Mt
+    @brief "t" ̾Ȥƻĥܥ.
 
-    ܥ #Mt  <tt>"t"</tt> Ȥ̤̾ˡֿפ
-    ̣롣  */
+    ܥ #Mt  <tt>"t"</tt> Ȥ̤̾ˡֿפޤ
+    ֹפ̣롣  */
 
 MSymbol Mt;
 
@@ -311,10 +303,10 @@ MSymbol Mt;
     etc.  */
 
 /***ja
-    @brief "string" ̾Ȥƻĥܥ
+    @brief "string" ̾Ȥƻĥܥ.
 
-    Ѥߥܥ #Mstring  <tt>"string"</tt> Ȥ̾
-    ؿ mchar_define_property () ΰȤƻȤ롣  */
+    ܥ #Mstring  <tt>"string"</tt> Ȥ̾ؿ 
+    mchar_define_property () ΰȤѤ롣  */
 
 MSymbol Mstring;
 
@@ -328,7 +320,7 @@ MSymbol Mstring;
     etc.  */
 
 /***ja
-    @brief "symbol" ̾Ȥƻĥܥ
+    @brief "symbol" ̾Ȥƻĥܥ.
 
     Ѥߥܥ #Msymbol  <tt>"symbol"</tt> Ȥ̾
     ؿ mchar_define_property () ΰȤƻȤ롣  */
@@ -354,10 +346,20 @@ MSymbol Msymbol;
     This function never fails.  */
 
 /***ja
-    @brief ꤵ줿̾ĥܥ֤
+    @brief ܥ.
 
-    ؿ msymbol ()  $NAME Ȥ̾Υܥ֤Τ褦ʥ
-    ܥ뤬¸ߤʤˤϡܥ뤬ưŪ˺롣
+    ؿ msymbol ()  $NAME Ȥ̾줿ܥ
+    Τ褦ʥܥ뤬¸ߤʤˤϡ롣줿
+    ܥϴǤϤʤ
+
+    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤ
+    Ѥ롣
+
+    @return
+    δؿϸĤܥ֤
+
+    @errors
+    δؿϷ褷ƼԤʤ
 
     @latexonly \IPAlabel{msymbol} @endlatexonly  */
 
@@ -406,6 +408,19 @@ msymbol (const char *name)
     @return
     If the operation was successful, this function returns the created
     symbol.  Otherwise, it returns #Mnil.  */
+/***ja
+    @brief .
+
+    ؿ msymbol_as_managing_key () ̾ $NAME Ŀ줿
+    ֤Ǥ̾ $NAME ĥܥ뤬С#Mnil 
+    ֤
+
+    ʸĤǻϤޤ륷ܥ m17n 饤֥ѤǤꡢŪˤ
+    Ѥ롣
+
+    @return
+    Сδؿܥ֤Ǥʤ
+    #Mnil ֤   */
 
 /***
     @errors
@@ -459,13 +474,16 @@ msymbol_as_managing_key (const char *name)
     This function never fails.  */
 
 /***ja
-    @brief ꤵ줿̾ĥܥõ
+    @brief ꤵ줿̾ĥܥõ.
 
-    ؿ msymbol_exist ()  $NAME Ȥ̾Υܥõ
+    ؿ msymbol_exist ()  $NAME Ȥ̾ĥܥõ
 
     @return
     ⤷Τ褦ʥܥ뤬¸ߤʤФΥܥ֤Ǥ
-    СѤߥܥ #Mnil ֤  */
+    СѤߥܥ #Mnil ֤  
+
+    @errors
+    δؿϷ褷ƼԤʤ     */
 
 /***@seealso
     msymbol_name (), msymbol ()  */
@@ -498,15 +516,16 @@ msymbol_exist (const char *name)
     The msymbol_name () function returns a pointer to a string
     containing the name of $SYMBOL.
 
-    @return
-    Name of the specified symbol.
-
     @errors
     This function never fails.  */
 /***ja
-    @brief ܥ֤̾
+    @brief ܥ̾.
 
-    ؿ msymbol_name () ϻꤵ줿ܥ $SYMBOL ֤̾  */
+    ؿ msymbol_name () ϻꤵ줿ܥ $SYMBOL ̾ޤʸ
+    ؤΥݥ󥿤֤
+
+    @errors
+    δؿϷ褷ƼԤʤ     */
 
 /***@seealso
     msymbol (), msymbol_exist ()  */
@@ -536,12 +555,16 @@ msymbol_name (MSymbol symbol)
     variable #merror_code.  */
 
 /***ja
-    @brief ܥץѥƥͤꤹ
+    @brief ܥץѥƥͤꤹ.
 
     ؿ msymbol_put () ϡܥ $SYMBOL ǥ $KEY Ǥ륷
     ܥץѥƥͤ $VAL ꤹ롣Υܥץѥƥˤ
-    Ǥͤо񤭤롣$SYMBOL, $KEY Ȥ줿
-    ǤʤƤϤʤʤ
+    Ǥͤо񤭤롣$SYMBOL, $KEY Ȥ #Mnil ǤäƤϤʤ
+    ʤ
+
+    $KEY ʤС$VAL ϴ֥ȤǤʤƤϤʤʤ
+    ξ硢Ťͤλȿ @c NULL Ǥʤ 1 餵졢$VAL 
+    ȿ 1 䤵롣
 
     @return
     Сmsymbol_put ()  0 ֤Ǥʤ -1 
@@ -573,23 +596,20 @@ msymbol_put (MSymbol symbol, MSymbol key, void *val)
     $SYMBOL has such a symbol property, its value is returned.
     Otherwise @c NULL is returned.
 
-
     @return
     If an error is detected, msymbol_get () returns @c NULL and
     assigns an error code to the external variable #merror_code.  */
 
 /***ja
-    @brief ܥץѥƥͤõ
+    @brief ܥץѥƥͤ.
 
     ؿ msymbol_get () ϡܥ $SYMBOL ĥܥץѥƥ
     Τ $KEY ǤΤõ⤷륷ܥץѥƥ
-    ¸ߤС֤ͤ⤷륷ܥץѥƥ¸
-     $KEY ФץХؿ¸ߤСδؿƤ
-    Ǥ֤ͤ륷ܥץѥƥץХؿ¸
-    ߤʤϳѿ #merror_code Ѥ @c NULL ֤
+    ¸ߤС֤ͤǤʤ @c NULL ֤
 
-    顼Ф줿 @c NULL ֤ѿ 
-    #merror_code ˥顼ɤꤹ롣  */
+    @return 
+    顼Ф줿硢msymbol_get ()  @c NULL ֤
+    ѿ #merror_code ˥顼ɤꤹ롣  */
 
 /***
     @errors
@@ -621,13 +641,24 @@ msymbol_get (MSymbol symbol, MSymbol key)
 /***en
     @brief Dump a symbol.
 
-    The mdebug_dump_symbol () function prints $SYMBOL in a human
+    The mdebug_dump_symbol () function prints symbol $SYMBOL in a human
     readable way to the stderr.  $INDENT specifies how many columns to
     indent the lines but the first one.
 
     @return
     This function returns $SYMBOL.
 
+    @errors
+    MERROR_DEBUG  */
+/***ja
+    @brief ܥפ.
+
+    ؿ mdebug_dump_symbol () ϥܥ $symbol  stderr ˿ʹ֤˲
+    ɤ ǰ롣 $UNDENT ϣܰʹߤΥǥȤꤹ롣
+
+    @return
+    δؿ $SYMBOL ֤ 
+
     @errors
     MERROR_DEBUG  */
 
@@ -670,6 +701,17 @@ mdebug_dump_symbol (MSymbol symbol, int indent)
 
     @errors
     MERROR_DEBUG  */
+/***ja
+    @brief ٤ƤΥܥ̾פ.
+
+    ؿ mdebug_dump_all_symbols () ϡ٤ƤΥܥ̾ 
+    stderr ˰롣 $UNDENT ϣܰʹߤΥǥȤꤹ롣
+
+    @return
+    δؿ #Mnil ֤ 
+
+    @errors
+    MERROR_DEBUG  */
 
 
 MSymbol
