@@ -32,27 +32,26 @@
     application programs can be simple.
 
     A text property consists of a @e key and @e values, where key is a
-    symbol and values are anything that can be cast to <tt>(void
-    *)</tt>.  Unlike other types of properties, a text property can
+    symbol and values are anything that can be cast to <tt>(void *)
+    </tt>.  Unlike other types of properties, a text property can
     have multiple values.  "The text property whose key is K" may be
     shortened to "K property".  */
 
 /***ja
     @addtogroup m17nTextProperty
-    @brief ƥȥץѥƥ뤿δؿ
+    @brief ƥȥץѥƥ뤿δؿ.
 
     M-text γʸϡ@e ƥȥץѥƥ ȸƤФץѥƥ
-    ĤȤǤ롣ƥȥץѥƥˤäơƥȤγ̤˴
-    ͡ʾ M-text ݻ뤳ȤǤ롣Τᡢ
-    ξ򥢥ץꥱץŪ˰ȤǤ롣
-    M-text Τ˭٤ʾĤᡢץꥱץ
-    γƴؿǲ뤳ȤǤ롣
+    ĤȤǤ롣ƥȥץѥƥϡM-text γ̤ղä
+    ޤޤʾݻƤꡢץꥱץϤ
+    ξŪ˰ȤǤ롣M-text Τ˭٤ʾĤ
+    ᡢץꥱץδؿǲ뤳ȤǤ롣
 
     ƥȥץѥƥ @e   @e  ʤ롣ϥܥǤ
     ꡢͤ <tt>(void *)</tt> ˥㥹ȤǤΤʤ鲿Ǥ褤
-    ¾ΥפΥץѥƥȰۤʤꡢƱΥƥȥץѥƥʣ
-    ĤȤ롣@c Mxxx Ȥĥƥȥץѥƥ
-    ΤȤñˡxxx ץѥƥפȸƤ֤Ȥ롣  */
+    ¾ΥפΥץѥƥȰۤʤꡢĤΥƥȥץѥƥʣ
+    ĤȤ롣֥ K ǤƥȥץѥƥפΤ
+    ñˡK ץѥƥפȸƤ֤Ȥ롣  */
 
 /*=*/
 
@@ -501,7 +500,7 @@ maybe_merge_interval (MTextPlist *plist, MInterval *interval)
 }
 
 
-/*** Adjust start and end positions of intervals between HEAD and TAIL
+/** Adjust start and end positions of intervals between HEAD and TAIL
      (both inclusive) by diff.  Adjust also start and end positions
      of text properties belonging to those intervals.  */
 
@@ -1430,6 +1429,20 @@ mtext__adjust_plist_for_insert (MText *mt, int pos, int nchars,
     }
 }
 
+void
+mtext__adjust_plist_for_change (MText *mt, int from, int to)
+{
+  MTextPlist *plist;
+
+  prepare_to_modify (mt, from, to, Mnil);
+  for (plist = mt->plist; plist; plist = plist->next)
+    {
+      pop_all_properties (plist, from, to);
+      xassert (check_plist (plist, 0) == 0);
+    }
+}
+
+
 /*** @} */
 #endif /* !FOR_DOXYGEN || DOXYGEN_INTERNAL_MODULE */
 
@@ -1466,28 +1479,27 @@ mtext__adjust_plist_for_insert (MText *mt, int pos, int nchars,
     mtext_get_prop_values () function instead.  */
 
 /***ja
-    @brief ƥȥץѥƥΰ־ͤ
+    @brief ƥȥץѥƥΰ־ͤ.
 
-    ؿ mtext_get_prop () ϡM-text $MT  $POS Ȥ֤ˤʸ
-    ΥץѥƥΤ $KEY ǤΤõ
+    ؿ mtext_get_prop () ϡM-text $MT ΰ $POS ˤʸΥ
+    ȥץѥƥΤ $KEY ǤΤõ
 
     @return
-    Τ褦ʥץѥƥ¸ߤʤ顢mtext_get_prop () Ϥͤ
-    ͤʣ¸ߤȤϡ־֤ͤĤʤг
-    ѿ #merror_code ѹ뤳Ȥʤ @c NULL ֤
+    ƥȥץѥƥߤĤСmtext_get_prop () ϤΥץѥƥ
+    ֤ͤͤʣ¸ߤȤϡ־֤ͤĤʤ
+    гѿ #merror_code ѹ뤳Ȥʤ @c NULL ֤
 
-    顼Ф줿 @c NULL ֤ѿ #merror_code 
-    顼 ɤꤹ롣
+    顼Ф줿 mtext_get_prop ()  @c NULL ֤
+     #merror_code ˥顼ɤꤹ롣
 
     @note 顼ʤ @c NULL ֤줿ˤĤβǽ롣
 
     @li $POS ΰ֤ʸ $KEY 򥭡Ȥץѥƥʤ
 
-    @li ʸ $KEY 򥭡ȤץѥƥĤͤ @c
-        NULL Ǥ롣
+    @li ʸϤΤ褦ʥץѥƥͤ @c NULL Ǥ롣
 
     Ĥ̤ɬפˤϡؿ mtext_get_prop_values ()
-    Ѥ뤳ȡ
+    ˻Ѥ뤳ȡ
 
      @latexonly \IPAlabel{mtext_get_prop} @endlatexonly  */
 
@@ -1521,7 +1533,7 @@ mtext_get_prop (MText *mt, int pos, MSymbol key)
 /*=*/
 
 /***en
-    @brief Get multiple values of a text property
+    @brief Get multiple values of a text property.
 
     The mtext_get_prop_values () function searches the character at
     $POS in M-text $MT for the property whose key is $KEY.  If such
@@ -1537,18 +1549,18 @@ mtext_get_prop (MText *mt, int pos, MSymbol key)
     assigns an error code to the external variable #merror_code.  */
 
 /***ja
-    @brief ƥȥץѥƥͤʣ
+    @brief ƥȥץѥƥͤʣ.
 
     ؿ mtext_get_prop_values () ϡM-text $MT  $POS Ȥ
     ˤʸΥץѥƥΤ $KEY ǤΤõ⤷
-    Τ褦ʥץѥƥĤС줬(ʣ) $VALUES 
-    ؤΰ˳Ǽ롣$NUM ϳǼͤοξ¤Ǥ롣
+    Τ褦ʥץѥƥĤС줬 (ʣ)  $VALUES 
+    λؤΰ˳Ǽ롣$NUM ϳǼͤοξ¤Ǥ롣
 
     @return
     Сmtext_get_prop_values () ϼºݤ˥˳Ǽ
     줿ͤο֤$POS ΰ֤ʸ $KEY 򥭡Ȥץѥƥ
-    ʤ 0 ֤顼Ф줿 -1 ֤ѿ 
-    #merror_code ˥顼ɤꤹ롣
+    ʤ 0 ֤顼Ф줿 -1 ֤
+    ѿ #merror_code ˥顼ɤꤹ롣
 
     @latexonly \IPAlabel{mtext_get_prop_values} @endlatexonly  */
 
@@ -1599,7 +1611,7 @@ mtext_get_prop_values (MText *mt, int pos, MSymbol key,
 /*=*/
 
 /***en
-    @brief Get list of text property keys at a position of an M-text.
+    @brief Get a list of text property keys at a position of an M-text.
 
     The mtext_get_prop_keys () function creates an array whose
     elements are the keys of text properties found at position $POS in
@@ -1608,7 +1620,6 @@ mtext_get_prop_values (MText *mt, int pos, MSymbol key,
     the array.
 
     @returns
-
     If the operation was successful, mtext_get_prop_keys () returns
     the length of the key list.  Otherwise it returns -1 and assigns
     an error code to the external variable #merror_code.
@@ -1616,11 +1627,11 @@ mtext_get_prop_values (MText *mt, int pos, MSymbol key,
 */
 
 /***ja
-    @brief ƥȥץѥƥΥΥꥹȤ
+    @brief M-text λꤷ֤ΥƥȥץѥƥΥΥꥹȤ.
 
     ؿ mtext_get_prop_keys () ϡM-text $MT  $POS ΰ֤ˤ
-    ʸäƤ뤹٤ƤΥƥȥץѥƥΥʤꡢ
-    Υɥ쥹 *$KEYS ˥åȤ롣Τ˳ݤ
+    ٤ƤΥƥȥץѥƥΥǤȤꡢ
+    ɥ쥹 *$KEYS ꤹ롣Τ˳ݤ줿
     Τϥ桼ǤǤ롣
 
     @return
@@ -1664,12 +1675,12 @@ mtext_get_prop_keys (MText *mt, int pos, MSymbol **keys)
 /*=*/
 
 /***en
-    @brief Set a text property
+    @brief Set a text property.
 
     The mtext_put_prop () function sets a text property to the
     characters between $FROM (inclusive) and $TO (exclusive) in M-text
     $MT.  $KEY and $VAL specify the key and the value of the text
-    property.
+    property.  With this function,
 
 @verbatim
                      FROM                   TO
@@ -1691,11 +1702,11 @@ PROP  :  <-- OLD_VAL-><-------- VAL -------><-- OLD_VAL-->
     variable #merror_code.  */
 
 /***ja
-    @brief ƥȥץѥƥꤹ
+    @brief ƥȥץѥƥꤹ.
 
     ؿ mtext_put_prop () ϡM-text $MT  $FROM ʴޤޤˤ 
     $TO ʴޤޤʤˤϰϤʸˡ $KEY ͤ $VAL Ǥ
-    ʥƥȥץѥƥ򥻥åȤ롣
+    ʥƥȥץѥƥꤹ롣δؿˤä
 
 
 @verbatim
@@ -1753,7 +1764,7 @@ mtext_put_prop (MText *mt, int from, int to, MSymbol key, void *val)
 /*=*/
 
 /***en
-    @brief Set multiple text properties with the same key
+    @brief Set multiple text properties with the same key.
 
     The mtext_put_prop_values () function sets a text property to the
     characters between $FROM (inclusive) and $TO (exclusive) in M-text
@@ -1766,11 +1777,11 @@ mtext_put_prop (MText *mt, int from, int to, MSymbol key, void *val)
     external variable #merror_code.  */
 
 /***ja
-    @brief ƱΥƥȥץѥƥʣꤹ
+    @brief ƱΥƥȥץѥƥʣꤹ.
 
     ؿ mtext_put_prop_values () ϡM-Text $MT $FROM ʴޤޤ
-     $TO ʴޤޤʤˤϰϤʸˡƥȥץѥƥ򥻥å
-    롣ƥȥץѥƥΥ$KEYˤäơ(ʣ)$VALUES 
+     $TO ʴޤޤʤˤϰϤʸˡƥȥץѥƥꤹ
+    롣ƥȥץѥƥΥ $KEY ˤäơ(ʣ) $VALUES 
     ˤäƻꤵ롣$NUM ꤵͤθĿǤ롣
 
     @return
@@ -1822,11 +1833,12 @@ mtext_put_prop_values (MText *mt, int from, int to,
 /*=*/
 
 /***en
-    @brief Push a text property
+    @brief Push a text property.
 
     The mtext_push_prop () function pushes a text property whose key
     is $KEY and value is $VAL to the characters between $FROM
-    (inclusive) and $TO (exclusive) in $MT.
+    (inclusive) and $TO (exclusive) in M-text $MT.  With this
+    function,
 
 @verbatim
                     FROM                    TO
@@ -1849,11 +1861,11 @@ PROP  :               <-------- VAL ------->
     variable #merror_code.  */
 
 /***ja
-    @brief ƥȥץѥƥץå夹
+    @brief ƥȥץѥƥץå夹.
 
     ؿ mtext_push_prop () ϡ $KEY ͤ $VAL Ǥƥ
     ץѥƥM-text $MT  $FROM ʴޤޤˤ $TO ʴޤޤ
-    ˤϰϤʸ˥ץå夹롣
+    ˤϰϤʸ˥ץå夹롣δؿˤä
 
 @verbatim
                     FROM                    TO
@@ -1973,14 +1985,14 @@ mtext_push_prop (MText *mt, int from, int to,
 /*=*/
 
 /***en
-    @brief Pop a text property
+    @brief Pop a text property.
 
     The mtext_pop_prop () function removes the topmost text property
     whose key is $KEY from the characters between $FROM (inclusive)
     and and $TO (exclusive) in $MT.
 
     This function does nothing if characters in the region have no
-    such text property.
+    such text property. With this function,
 
 @verbatim
                     FROM                    TO
@@ -2002,14 +2014,14 @@ PROP  :  <--OLD_VAL-->|                     |<--OLD_VAL-->|
     variable #merror_code.  */
 
 /***ja
-    @brief ƥȥץѥƥݥåפ
+    @brief ƥȥץѥƥݥåפ.
 
     ؿ mtext_pop_prop () ϡ $KEY Ǥƥȥץѥƥ
     ־ΤΤM-text $MT  $FROM ʴޤޤˤ $TOʴޤ
     ʤˤϰϤʸ
 
     ϰϤʸΤ褦ʥץѥƥʤʤСδؿϲ
-    ⤷ʤ
+    ⤷ʤδؿˤäơ
 
 @verbatim
                     FROM                    TO
@@ -2141,14 +2153,14 @@ mtext_pop_prop (MText *mt, int from, int to, MSymbol key)
 /***ja
     @brief ƥȥץѥƥƱͤȤϰϤĴ٤.
 
-    ؿ mtext_prop_range () ϡϢ³ʸƱץѥƥͤ
-    ƤϰϤĴ٤롣ޤ M-text $MT  $POS ΰ֤ˤʸΥץ
-    ѥƥΤ $KEY ǻꤵ줿Τͤ򸫤Ĥ롣
-    ʸ$KEY ΥץѥƥƱͤäƤ뤫ɤĴ٤롣
-    ĤϰϤκǽȺǸ򡢤줾 $FROM  $TO ݥ󥿤Ȥ
-    ¸롣$FROM ¸ʸΰ֤ϸĤϰϤ˴ޤޤ
-    $TO ϴޤޤʤ$TO ƱͤȤϰϤϽ롣ˤ
-    ϰϻˡϡؿ mtext_put_prop () ʤɤȶ̤ΤΤǤ롣
+    ؿ mtext_prop_range () ϡꤷƥȥץѥƥͤƱ
+    ǤϢ³ʸϰϤĴ٤롣ޤ M-text $MT  $POS ΰ֤
+    ʸΥץѥƥΤ $KEY ǻꤵ줿ͤ򸫤Ĥ
+    롣ʸ $KEY ΥץѥƥͤƱǤ뤫ɤ
+    Ĵ٤롣ĤϰϤκǽȺǸ򡢤줾 $FROM  $TO ˥ݥ
+    Ȥѿ¸롣$FROM ¸ʸΰ֤ϸĤϰ
+    ˴ޤޤ뤬$TO ϴޤޤʤ$TO ƱͤȤϰϤϽ
+    롣ˤϰϻˡϡؿ mtext_put_prop () ʤɤȶ̤Ǥ롣
 
     $DEEPER  0 ǤʤС$KEY ȤĥץѥƥΤ
     ΤΤǤʤåΤ٤ƤΤΤӤ롣
@@ -2233,12 +2245,22 @@ mtext_prop_range (MText *mt, MSymbol key, int pos,
     @brief Create a text property.
 
     The mtext_property () function returns a newly allocated text
-    property whose key is $KEY and value is $VAL.  The text created
+    property whose key is $KEY and value is $VAL.  The created text 
     property is not attached to any M-text, i.e. it is detached.
 
     $CONTROL_BITS must be 0 or logical OR of @c enum @c
     MTextPropertyControl.  */
 
+/***ja
+    @brief ƥȥץѥƥ.
+
+    ؿ mtext_property ()  $KEY 򥭡$VAL ͤȤ뿷
+    Ƥ줿ƥȥץѥƥ֤ƥȥץѥƥϤ
+    ʤ M-text ˤղäƤʤʤʬΥ (detached) 롣
+
+    $CONTROL_BITS  0 Ǥ뤫 @c enum @c MTextPropertyControl  
+    OR ǤʤƤϤʤʤ  */
+
 MTextProperty *
 mtext_property (MSymbol key, void *val, int control_bits)
 {
@@ -2252,6 +2274,13 @@ mtext_property (MSymbol key, void *val, int control_bits)
     text property $PROP is attached.  If $PROP is currently detached,
     NULL is returned.  */
 
+/***ja
+    @brief ƥȥץѥƥ M-text ֤.
+
+    ؿ mtext_property_mtext () ϡƥȥץѥƥ$PROP ղä
+    Ƥ M-text ֤λ $PROP ʬΥƤ NULL 
+    ֤  */
+
 MText *
 mtext_property_mtext (MTextProperty *prop)
 {
@@ -2264,6 +2293,12 @@ mtext_property_mtext (MTextProperty *prop)
     The mtext_property_key () function returns the key (symbol) of
     text property $PROP.  */
 
+/***ja
+    @brief ƥȥץѥƥΥ֤.
+
+    ؿ mtext_property_key () ϡƥȥץѥƥ $PROP Υʥ
+    ܥˤ֤  */
+
 MSymbol
 mtext_property_key (MTextProperty *prop)
 {
@@ -2276,6 +2311,12 @@ mtext_property_key (MTextProperty *prop)
     The mtext_property_value () function returns the value of text
     property $PROP.  */
 
+/***ja
+    @brief ƥȥץѥƥ֤ͤ.
+
+    ؿ mtext_property_value () ϡƥȥץѥƥ $PROP ͤ
+      */
+
 void *
 mtext_property_value (MTextProperty *prop)
 {
@@ -2290,6 +2331,13 @@ mtext_property_value (MTextProperty *prop)
     of an M-text where $PROP begins.  If $PROP is detached, it returns
     -1.  */
 
+/***ja
+    @brief ƥȥץѥƥγϰ֤֤.
+
+    ؿ mtext_property_start () ϡƥȥץѥƥ $PROP γϰ
+    ֤֤ϰ֤Ȥ M-text  $PROP Ϥޤʸ֤Ǥ롣
+    $PROP ʬΥƤС-1 ֤  */
+
 int
 mtext_property_start (MTextProperty *prop)
 {
@@ -2304,6 +2352,13 @@ mtext_property_start (MTextProperty *prop)
     an M-text where $PROP ends.  If $PROP is detached, it returns
     -1.  */
 
+/***ja
+    @brief ƥȥץѥƥνλ֤֤.
+
+    ؿ mtext_property_end () ϡƥȥץѥƥ $PROP νλ
+    ֤λ֤Ȥ M-text  $PROP ʸ֤Ǥ롣$PROP 
+    ʬΥƤС-1 ֤  */
+
 int
 mtext_property_end (MTextProperty *prop)
 {
@@ -2313,8 +2368,8 @@ mtext_property_end (MTextProperty *prop)
 /***en
     @brief Get the topmost text property.
 
-    The mtext_get_property () function searches the character at $POS
-    in M-text $MT for a text property whose key is $KEY.
+    The mtext_get_property () function searches the character at
+    position $POS in M-text $MT for a text property whose key is $KEY.
 
     @return
     If a text property is found, mtext_get_property () returns it.  If
@@ -2325,6 +2380,20 @@ mtext_property_end (MTextProperty *prop)
     If an error is detected, mtext_get_property () returns @c NULL and
     assigns an error code to the external variable #merror_code.  */
 
+/***ja
+    @brief ־Υƥȥץѥƥ.
+
+    ؿ mtext_get_property ()  M-text $MT ΰ $POS ʸ
+     $KEY ǤƥȥץѥƥĤɤĴ٤롣
+
+    @return 
+    ƥȥץѥƥĤСmtext_get_property () Ϥ֤
+    ʣˤϡ־ΤΤ֤ĤʤСѿ 
+    #merror_code Ѥ뤳Ȥʤ @c NULL ֤
+
+    顼Ф줿 mtext_get_property ()  @c NULL ֤
+    ѿ #merror_code ˥顼ɤꤹ롣    */
+
 MTextProperty *
 mtext_get_property (MText *mt, int pos, MSymbol key)
 {
@@ -2359,6 +2428,21 @@ mtext_get_property (MText *mt, int pos, MSymbol key)
     is 0. If an error is detected, mtext_get_properties () returns -1
     and assigns an error code to the external variable #merror_code.  */
 
+/***ja
+    @brief ʣΥƥȥץѥƥ.
+
+    ؿ mtext_get_properties ()  M-text $MT ΰ $POS ʸ
+     $KEY ǤƥȥץѥƥĤɤĴ٤롣Τ褦
+    ץѥƥߤĤС$PROPS ؤΰ¸롣$NUM 
+    ¸ץѥƥοξ¤Ǥ롣
+
+    @return 
+    Сmtext_get_properties () ϼºݤ¸ץѥƥ
+    ο֤$POS ΰ֤ʸ $KEY Ǥץѥƥ
+    ʤС0 ֤롣顼Ф줿ˤϡ
+    mtext_get_properties ()  -1 ֤ѿ #merror_code ˥顼
+    ɤꤹ롣  */
+
 int
 mtext_get_properties (MText *mt, int pos, MSymbol key,
 		      MTextProperty **props, int num)
@@ -2408,6 +2492,18 @@ mtext_get_properties (MText *mt, int pos, MSymbol key,
     0.  Otherwise it returns -1 and assigns an error code to the
     external variable #merror_code.  */
 
+/***ja
+    @brief  M-text˥ƥȥץѥƥղä.
+
+    ؿ mtext_attach_property () ϡM-text $MT  $FROM  $TO 
+    Ǥΰ˥ƥȥץѥƥ $PROP ղä롣⤷ $PROP 
+    M-text ղäƤС$MT ղäʬΥ롣
+
+    @return 
+    Сmtext_attach_property ()  0 ֤Ǥʤ
+     -1 ֤Ƴѿ#merror_code ˥顼ɤꤹ롣      */
+
+
 int
 mtext_attach_property (MText *mt, int from, int to, MTextProperty *prop)
 {     
@@ -2447,6 +2543,14 @@ mtext_attach_property (MText *mt, int from, int to, MTextProperty *prop)
     @return
     This function always returns 0.  */
 
+/***ja
+    @brief  M-text ƥȥץѥƥʬΥ.
+
+    ؿ mtext_detach_property () ϥƥȥץѥƥ $PROP ʬΥ롣
+
+    @return
+    δؿϾ 0 ֤  */
+
 int
 mtext_detach_property (MTextProperty *prop)
 {
@@ -2465,14 +2569,27 @@ mtext_detach_property (MTextProperty *prop)
 /***en
     @brief Push a text property onto an M-text.
 
-    The mtext_push_property () function attaches text property $PROP on
-    M-text MT by the "push" manner.
+    The mtext_push_property () function pushes text property $PROP to
+    the characters between $FROM (inclusive) and $TO (exclusive) in
+    M-text $MT.
 
     @return
     If the operation was successful, mtext_push_property () returns
     0.  Otherwise it returns -1 and assigns an error code to the
     external variable #merror_code.  */
 
+/***ja
+    @brief M-text ˥ƥȥץѥƥץå夹.
+
+    ؿ mtext_push_property () ϡƥȥץѥƥ $PROP 
+    M-text $MT  $FROM ʴޤޤˤ $TO ʴޤޤʤˤϰϤ
+    ʸ˥ץå夹롣
+
+    @return
+    Сmtext_push_property ()  0 ֤Ǥʤ
+     -1 ֤Ƴѿ#merror_code ˥顼ɤꤹ롣      */
+
+
 int
 mtext_push_property (MText *mt, int from, int to, MTextProperty *prop)
 {
@@ -2569,7 +2686,20 @@ mtext_push_property (MText *mt, int from, int to, MTextProperty *prop)
     property whose key is #Mtext_prop_serializer and value is a
     pointer to an appropriate serializer function.
 
-    @seealso Mtext_prop_serializer (), MTextPropSerializeFunc
+    @seealso
+    mtext_serialize (), #MTextPropSerializeFunc
+  */
+
+/***ja
+    @brief ꥢ饤ؿꤹ륷ܥ.
+
+    ƥȥץѥƥ򥷥ꥢ饤뤿ˤϡΥƥȥץ
+    ƥѤΥꥢ饤ؿͿʤƤϤʤʤŪˤϡ
+    #Mtext_prop_serializer 򥭡ȤŬڤʥꥢ饤ؿؤΥݥ
+    󥿤ͤȤ륷ܥץѥƥꤹ롣
+
+    @seealso
+    mtext_serialize (), #MTextPropSerializeFunc
   */
 MSymbol Mtext_prop_serializer;
 
@@ -2581,7 +2711,20 @@ MSymbol Mtext_prop_serializer;
     property whose key is #Mtext_prop_deserializer and value is a
     pointer to an appropriate deserializer function.
 
-    @seealso Mtext_prop_serializer (), MTextPropSerializeFunc
+    @seealso
+    mtext_deserialize (), #MTextPropSerializeFunc
+  */
+
+/***ja
+    @brief ǥꥢ饤ؿꤹ륷ܥ.
+
+    ƥȥץѥƥǥꥢ饤뤿ˤϡΥƥȥץ
+    ѥƥѤΥǥꥢ饤ؿͿʤƤϤʤʤŪˤϡ
+    #Mtext_prop_deserializer 򥭡ȤŬڤʥǥꥢ饤ؿؤ
+    ݥ󥿤ͤȤ륷ܥץѥƥꤹ롣
+
+    @seealso
+    mtext_deserialize (), #MTextPropSerializeFunc
   */
 MSymbol Mtext_prop_deserializer;
 
@@ -2589,13 +2732,15 @@ MSymbol Mtext_prop_deserializer;
     @brief Serialize text properties in an M-text.
 
     The mtext_serialize () function serializes the text between $FROM
-    and $TO in M-text $MT.  The serialized result is an M-text in the
+    and $TO in M-text $MT.  The serialized result is an M-text in a
     form of XML.  $PROPERTY_LIST limits the text properties to be
-    serialized.  If a symbol 1) appears as the value of an element in
-    $PROPERTY_LIST (the key must be @c Mt ) and 2) has the symbol
-    property #Mtext_prop_serializer, a text property having that
-    symbol as its key is turned into the "property" element in the
-    resulting XML representation.
+    serialized. Only those text properties whose key 
+
+    @li appears as the value of an element in $PROPERTY_LIST, and
+    @li has the symbol property #Mtext_prop_serializer
+
+    are serialized as a "property" element in the resulting XML
+    representation.
 
     The DTD of the generated XML is as follows:
 
@@ -2621,7 +2766,47 @@ MSymbol Mtext_prop_deserializer;
     error code to the external variable #merror_code.
 
     @seealso
-    mtext_deserialize (), Mtext_prop_serializer  */
+    mtext_deserialize (), #Mtext_prop_serializer  */
+
+/***ja
+    @brief M-text Υƥȥץѥƥ򥷥ꥢ饤.
+
+    ؿ mtext_serialize ()  M-text $MT  $FROM  $TO ޤǤΥƥ
+    Ȥ򥷥ꥢ饤롣ꥢ饤̤ XML  M-text 
+    롣 $PROPERTY_LIST ϥꥢ饤ƥȥץѥƥ
+    롣оݤȤʤƥȥץѥƥϡΥ
+
+    @li $PROPERTY_LIST ǤͤȤƸ졢
+    @li ܥץѥƥ #Mtext_prop_serializer 
+    
+    ΤΤߤǤ롣ξƥȥץѥƥϡ 
+    XML ɽ "property" Ǥ˥ꥢ饤롣
+
+     XML  DTD ϰʲ̤:
+
+@verbatim
+<!DOCTYPE mtext [
+  <!ELEMENT mtext (property*,body+)>
+  <!ELEMENT property EMPTY>
+  <!ELEMENT body (#PCDATA)>
+  <!ATTLIST property key CDATA #REQUIRED>
+  <!ATTLIST property value CDATA #REQUIRED>
+  <!ATTLIST property from CDATA #REQUIRED>
+  <!ATTLIST property to CDATA #REQUIRED>
+  <!ATTLIST property control CDATA #REQUIRED>
+ ]>
+@endverbatim
+
+    δؿ libxml2 饤֥˰¸롣m17n 饤֥꤬libxml2 
+    ̵ꤵƤ硢δؿϾ˼Ԥ롣
+
+    @return 
+    Сmtext_serialize ()  XML  M-text ֤
+    Ǥʤ @c NULL ֤Ƴѿ#merror_code ˥顼
+    ꤹ롣
+
+    @seealso
+    mtext_deserialize (), #Mtext_prop_serializer  */
 
 MText *
 mtext_serialize (MText *mt, int from, int to, MPlist *property_list)
@@ -2734,7 +2919,37 @@ mtext_serialize (MText *mt, int from, int to, MPlist *property_list)
     code to the external variable #merror_code.
 
     @seealso
-    mtext_serialize (), Mtext_prop_deserializer  */
+    mtext_serialize (), #Mtext_prop_deserializer  */
+
+/***ja
+    @brief M-text Υƥȥץѥƥǥꥢ饤.
+
+    ؿ mtext_deserialize ()  M-text $MT ǥꥢ饤롣$MT
+    ϼ DTD  XML ǤʤƤϤʤʤ
+ 
+@verbatim
+<!DOCTYPE mtext [
+  <!ELEMENT mtext (property*,body+)>
+  <!ELEMENT property EMPTY>
+  <!ELEMENT body (#PCDATA)>
+  <!ATTLIST property key CDATA #REQUIRED>
+  <!ATTLIST property value CDATA #REQUIRED>
+  <!ATTLIST property from CDATA #REQUIRED>
+  <!ATTLIST property to CDATA #REQUIRED>
+  <!ATTLIST property control CDATA #REQUIRED>
+ ]>
+@endverbatim
+
+    δؿ libxml2 饤֥˰¸롣m17n 饤֥꤬libxml2 
+    ̵ꤵƤ硢δؿϾ˼Ԥ롣
+
+    @return 
+    Сmtext_serialize () 줿 M-text 
+    ֤Ǥʤ @c NULL ֤Ƴѿ #merror_code ˥顼
+    ɤꤹ롣
+
+    @seealso
+    mtext_serialize (), #Mtext_prop_deserializer  */
 
 MText *
 mtext_deserialize (MText *mt)
